﻿#include "adjust_ppk.h"
#include "voxel_map/libs/utils/basic_functions.h"
#include <boost/algorithm/string/classification.hpp>
#include <boost/algorithm/string/split.hpp>
using namespace boost::filesystem;

namespace juefx{

AdjustPPK::AdjustPPK(const string &ieBaseDir, const string &mapDir)
    :ieBaseDir_(ieBaseDir), mapDir_(mapDir)
{
    LOG(INFO) << "ieBaseDir_: " << ieBaseDir_;
    LOG(INFO) << "mapDir_: " << mapDir_;
    ReadCenter();
    LoadPPK();
    LoadMapInfo();
    CalLocalPose();
    CalPPKPeriod();
    std::ofstream ofs("/home/juefx/wangdawei/CloudPredictPose.txt", ios::out);
    ofs.close();
    ofs.open("/home/juefx/wangdawei/LocalPPK.txt", ios::out);
    ofs.close();
    Matrix4d m;
    /////////////// cpt ///////////////
    m << 0.9999146, 00.0128011, 00.0026317, 00.3097527,
            0.0128028, -0.9999178, -0.0006307, -0.1673601,
            00.0026234, -0.0006643, 00.9999963, 00.2500000,
            00.0000000, 00.0000000, 00.0000000, 01.0000000;

    calib_.translation() = m.block(0, 3, 3, 1);
    calib_.linear() = m.block(0, 0, 3, 3);
    boost::thread trd(boost::bind(&AdjustPPK::LoadMap, this));
}

void AdjustPPK::ConfigConvert(const string &firetimeFile, const string &correctionFile)
{
    convert_.reset(new pandar_pointcloud::Convert(firetimeFile, correctionFile));
    LOG(INFO) << "ConfigConvert done";
}

void AdjustPPK::ReadBag(const string &bagPath)
{
    if(!convert_){
        LOG(FATAL) << "!convert_";
        return;
    }
    LOG(INFO) << "ReadBag: " << bagPath;
    rosbag::Bag bag;
    boost::shared_ptr<rosbag::View> view;
    rosbag::View::iterator viewIterator;
    bag.open(bagPath);
    LOG(INFO) << "bag_ opened";
    view.reset(new rosbag::View(bag));
    LOG(INFO) << "view_ reset";
    viewIterator = view->begin();
    for(; viewIterator != view->end();)
    {
        const rosbag::MessageInstance &m = *viewIterator;
        LOG_EVERY_N(INFO, 1) << setprecision(15) << "m.getTime(): " << m.getTime().toSec();
        if(!m.isType<pandar_msgs::PandarScan>()){
            viewIterator++;
            continue;
        }
        LOG_EVERY_N(INFO, 1) << "processScan";
        PPointCloud cloud;
        if(convert_->processScan(m.instantiate<pandar_msgs::PandarScan>(), cloud)){
            OnReceivedCloud(cloud);
        }
    }
}

void AdjustPPK::ReadCenter()
{
    string centerPath = mapDir_ + "/2329_station_jf.txt";
    std::ifstream ifs(centerPath);
    string line;
    vector<string> line_vec;
    if(ifs) {
        getline(ifs, line);
        boost::split(line_vec, line, boost::is_any_of(":"), boost::token_compress_on);
        center_.x() = stof(line_vec.at(1));
        getline(ifs, line);
        boost::split(line_vec, line, boost::is_any_of(":"), boost::token_compress_on);
        center_.y() = stof(line_vec.at(1));
        getline(ifs, line);
        boost::split(line_vec, line, boost::is_any_of(":"), boost::token_compress_on);
        center_.z() = stof(line_vec.at(1));
    }
}

void AdjustPPK::OnReceivedCloud(const PPointCloud &cloud)
{
    CloudPacket cloudPacket;
    ros::Time rosTime;
    pcl_conversions::fromPCL(cloud.header.stamp, rosTime);
    cloudPacket.timestamp = rosTime.toSec();
    cloudPacket.cloud.reserve(cloud.size());
    LOG(INFO) << setprecision(15) << "OnReceivedCloud: " << cloudPacket.timestamp;
    for(const auto &p : cloud){
        PointInter point;
        point.x = p.x;
        point.y = p.y;
        point.z = p.z;
        point.intensity = p.intensity;
        point.time = p.timestamp;
        cloudPacket.cloud.push_back(point);
    }

    CloudInfoForMatch cloudInfo;
    Undisort(cloudPacket, cloudInfo);

    ConfigMap(cloudInfo.timestamp);

    LocateCloud(cloudInfo);
}

void AdjustPPK::LoadPPK()
{
    IEReader::Ptr ieReader(new IEReader);
//    directory_iterator nav_iter(ieBaseDir_);
//    directory_iterator bag_end_iter;
//    for(; nav_iter != bag_end_iter; nav_iter++) {
//        if(nav_iter->path().extension().string() == ".nav"){
//            LOG(INFO) << "nav: " << nav_iter->path().string();
//            ieReader->Init(nav_iter->path().string());
//            break;
//        }
//    }
    ieReader->Init(ieBaseDir_ + "/gnss.txt");
    rawData_ = ieReader->GetData();
    LOG(INFO) << "rawData_.size(): " << rawData_.size();
}

void AdjustPPK::LoadMapInfo()
{
//    string meshDir = mapDir_ + "/mesh";
//    if(is_directory(meshDir)){
//        directory_iterator mesh_iter(meshDir);
//        directory_iterator mesh_end_iter;
//        for(; mesh_iter != mesh_end_iter; mesh_iter++) {
//            if (!is_directory(mesh_iter->path())){
//                continue;
//            }
//            string meshId = mesh_iter->path().stem().string();
//            MashMap meshMap;
//            meshMap.pcdPathVec.reserve(10);
//            directory_iterator pcd_iter(mesh_iter->path());
//            directory_iterator pcd_end_iter;
//            for(; pcd_iter != pcd_end_iter; pcd_iter++) {
//                if(pcd_iter->path().extension().string() != ".pcd"){
//                    continue;
//                }
//                meshMap.pcdPathVec.push_back(pcd_iter->path().string());
//            }
//            meshVec_.insert(make_pair(ulong(stoi(meshId)), meshMap));
//        }
//    }
    string meshDir = mapDir_ + "/pcd_cart";
    if(is_directory(meshDir)){
        directory_iterator pcd_iter(meshDir);
        directory_iterator pcd_end_iter;
        for(; pcd_iter != pcd_end_iter; pcd_iter++) {
            if(pcd_iter->path().extension().string() != ".pcd"){
                continue;
            }
            auto fileName = pcd_iter->path().filename().string();
            ulong meshId = stoi(split_string(fileName, '_').front());
            if(meshVec_.find(meshId) == meshVec_.end()){
                MashMap meshMap;
                meshMap.pcdPathVec.push_back(pcd_iter->path().string());
                meshVec_.insert(make_pair(meshId, meshMap));
            }else{
                meshVec_.at(meshId).pcdPathVec.push_back(pcd_iter->path().string());
            }
        }
    }
    LOG(INFO) << "meshVec_.size(): " << meshVec_.size();
}

void AdjustPPK::CalLocalPose()
{
    localPoseVec_.reserve(rawData_.size());
    LocalCartesian proj(center_.x(), center_.y(), center_.z());
    std::ofstream ofs("/home/juefx/wangdawei/LocalPPK.txt", ios::app);
    for(const IERawData& rawPPK : rawData_){
        IsometryData localPose;
        proj.Forward(rawPPK.latitude, rawPPK.longitude, rawPPK.height,
                     localPose.pose.translation().x(),
                     localPose.pose.translation().y(),
                     localPose.pose.translation().z());
        localPose.pose.linear() = rawPPK.getOrientation();
        localPose.timestamp = rawPPK.unix_time;
        localPoseVec_.emplace_back(localPose);
//        LOG_EVERY_N(INFO, 10000) << "localPoseVec_.size(): " << localPoseVec_.size();

        ofs << setprecision(16) << localPose.timestamp << ", " << localPose.pose.translation().x()
            << ", " << localPose.pose.translation().y() << ", " << localPose.pose.translation().z() << endl;
    }
    ofs.close();
    LOG(INFO) << "localPoseVec_.size(): " << localPoseVec_.size();
}

void AdjustPPK::CalPPKPeriod()
{
    ppkPeriodVec_.reserve(1000);
    PPKPeriod period;
    period.meshId = 0;
    period.period.reserve(10000);
    for(const IERawData& rawPPK : rawData_){
        ulong meshId = GetMeshFromPosition(rawPPK.longitude, rawPPK.latitude);
        if(meshId != period.meshId){
            if(period.period.size()){
                period.endTime = rawPPK.unix_time;
                ppkPeriodVec_.emplace_back(period);
                period.meshId = meshId;
                period.period.resize(0);
            }
            period.startTime = rawPPK.unix_time;
        }
        period.period.emplace_back(rawPPK);
    }
    period.endTime = rawData_.back().unix_time;
    ppkPeriodVec_.emplace_back(period);
    LOG(INFO) << "ppkPeriodVec_.size(): " << ppkPeriodVec_.size();
}

PointCloudInter::Ptr AdjustPPK::GetBaseFrame(
        const CloudPacket &cloudData)
{
    PointCloudInter::Ptr frame(new PointCloudInter);
    *frame = cloudData.cloud;
    pcl::transformPointCloud(*frame, *frame, calib_.cast<float>());
    pcl::transformPointCloud(*frame, *frame, imu2base_.cast<float>());
    return frame;
}

void AdjustPPK::Undisort(
        CloudPacket &rawFrame, CloudInfoForMatch &cloudInfo)
{
    vector<IsometryData> periodPose;
    periodPose.reserve(150);
    double backPTime = rawFrame.timestamp;
    double frontPTime = rawFrame.timestamp + rawFrame.cloud.front().time;
    for(; ppkIndex_ < localPoseVec_.size(); ppkIndex_++){
        const auto &pose = localPoseVec_.at(ppkIndex_);
        if(pose.timestamp > backPTime){
            break;
        }
        if(pose.timestamp <= frontPTime){
            periodPose.emplace_back(pose);
        }
    }
    PointCloudInter::Ptr frame = GetBaseFrame(rawFrame);
    Isometry3d framePose, increasePose;
    PointCloudJ::Ptr undisortedFrame(new PointCloudJ);
    PointCloudJ mappedFrame;
    CloudPreprocess::Instance()->Undisort(
                            frame, undisortedFrame, &mappedFrame, backPTime, &periodPose, framePose, increasePose);
    cloudInfo.frame = undisortedFrame;
    cloudInfo.filterPoseInfo.increasePose = increasePose;
    cloudInfo.timestamp = rawFrame.timestamp;
    cloudInfo.filterPoseInfo.filterPose = framePose;
}

void AdjustPPK::ConfigMap(double timestamp)
{
    while(!ppkPeriodVec_.at(periodIndex_).InPeriod(timestamp)){
        periodIndex_++;
        if(periodIndex_ >= ppkPeriodVec_.size()){
            return;
        }
        LOG(INFO) << "next period " << periodIndex_;
    }
    auto meshId = ppkPeriodVec_.at(periodIndex_).meshId;
    while(!meshVec_.at(meshId).matcher){
        std::this_thread::sleep_for(std::chrono::milliseconds(100));
    }
    currMatcher_ = meshVec_.at(meshId).matcher;
    LOG(INFO) << "set currMatcher_: " << meshId;
}

bool AdjustPPK::LocateCloud(const CloudInfoForMatch &cloudInfo)
{
    static Isometry3f mapPose = Isometry3f::Identity();
    static uint32_t cnt = 0;
    bool success = true;
    GuessPose guessPose;
    static std::ofstream ofs("/home/juefx/wangdawei/CloudPredictPose.txt", ios::app);
    if(0 == cnt){
        mapPose = cloudInfo.filterPoseInfo.filterPose.cast<float>();
    }
    if(cnt % 10 != 0){
        mapPose = mapPose * cloudInfo.filterPoseInfo.increasePose.cast<float>();
    }else{
        guessPose.map_point = mapPose.cast<double>();
        auto increaseRpy = RotationQuaternionToEulerVector(
                    Quaterniond(cloudInfo.filterPoseInfo.increasePose.linear()));
        guessPose.movement = cloudInfo.filterPoseInfo.increasePose.translation().norm();
        guessPose.rotation = fabs(increaseRpy.z());
        guessPose.use_gnss = false;
        guessPose.precision_type = MATCH_PRECISION_FULL;

        GnssPoint finalGnssPose;
        if(!currMatcher_->IsMapLoaded(mapPose.translation().cast<double>())){
            currMatcher_->loadArea(mapPose.translation().cast<double>());
        }
        success = currMatcher_->MatchPointCloud(cloudInfo.frame, guessPose, finalGnssPose, mapPose);
        ofs << setprecision(16) << cloudInfo.timestamp << ", " << mapPose.translation().x()
            << ", " << mapPose.translation().y() << ", " << mapPose.translation().z() << endl;
    }

    cnt++;
    return success;
}

void AdjustPPK::LoadMap()
{
    string outputDir = "/home/juefx/wangdawei/stream/";
    if(!boost::filesystem::exists(outputDir)){
        boost::filesystem::create_directory(outputDir);
    }
    for(const auto &period : ppkPeriodVec_){
        auto meshId = period.meshId;
        if(meshVec_.find(meshId) == meshVec_.end()){
            LOG(WARNING) << "meshVec_ no meshId: " << meshId;
            continue;
        }
        auto &meshInfo = meshVec_.at(meshId);
        if(meshInfo.matcher){
            continue;
        }
        string streamPath = outputDir + to_string(meshId) + ".stream";

        if(!exists(streamPath)){
            Compress(meshInfo.pcdPathVec, streamPath);
            LOG(WARNING) << "mesh compressed: " << meshId;
        }else{
            LOG(WARNING) << "mesh compressed: " << meshId;
        }

        LoadMesh(streamPath, meshInfo);
        LOG(WARNING) << "mesh loaded: " << meshId;
    }
}

void AdjustPPK::Compress(const vector<string> &pcdPathVec,
                         const string &streamPath)
{
    PointCloudExport::Ptr pointcloud(new PointCloudExport);
    for(const string &pcd_path : pcdPathVec){
        PointCloudExport pc;
        pcl::io::loadPCDFile(pcd_path, pc);
        *pointcloud += pc;
    }
    boost::shared_ptr<Dispatch> dispatch(new Dispatch(pointcloud));
    // dispatch.setMeshId(meshId);
    dispatch->configOffset();
    dispatch->dispatch();
    LOG(INFO) << "dispatch done";
    boost::shared_ptr<GridLayer> meshLayer =
            dispatch->getMeshLayer();

    BlockSerializer blockSerializer(meshLayer);
    blockSerializer.setOffset(dispatch->getOffset());
    blockSerializer.setOutputPath(streamPath);
    blockSerializer.configSerialization();
}

void AdjustPPK::LoadMesh(const string &streamPath, MashMap &meshMap)
{
    VoxelMapMatcherOption option;
    option.option.filter_resolution = 0.5;
    option.option.cloud_range = 80;
    option.fast_option.accepted_score = 0.45;
    option.fast_option.accepted_low_score = 0.55;
    meshMap.matcher.reset(new VoxelMapMatcher(option));
    meshMap.matcher->SetMapPath(streamPath);
}

} // end of namespace
