#include <iostream>
#include "signal.h"
#include <gflags/gflags.h>

#include <glog/logging.h>
#include <pcl/io/pcd_io.h>
#include <pcl/common/transforms.h>

#include "system/syscontroller.h"
#include "input/sensor_subscriber.h"

#ifdef ENABLE_ROS
#include "input/ros_bridge/cloud_subscriber.h"
#include "input/ros_bridge/gnss_subscriber.h"
#include "input/ros_bridge/imu_subscriber.h"
#include "input/ros_bridge/wheel_subscriber.h"
#include "input/ros_bridge/imupose_subscriber.h"
#include "output/ros_publisher.h"
#else
#ifdef ENABLE_BAG
#include "input/rosbag/rosbag_reader.h"
#include "output/ros_publisher.h"
#else
#include "input/file_reader/cloud_receiver.h"
#include "input/file_reader/gnss_receiver.h"
#include "input/file_reader/imu_receiver.h"
#include "input/file_reader/wheel_receiver.h"
#include "input/file_reader/file_reader.h"
#include "output/netpublisher.h"
#endif
#endif

#include "adjust_ppk_by_locate/adjust_ppk.h"

DEFINE_string(ie_base_dir, "", "ie_base_dir");
DEFINE_string(map_dir, "", "map_dir");

using namespace std;
using namespace juefx;

void failureHandle(const char* data, int size){
    string str = data;
    LOG(INFO) << "failureHandle: " << str;
}

int main(int argc, char *argv[])
{
    gflags::ParseCommandLineFlags(&argc, &argv, true);
    google::InitGoogleLogging(argv[0]);
    google::InstallFailureSignalHandler();
    google::InstallFailureWriter(&failureHandle);
    FLAGS_alsologtostderr = true;
    FLAGS_colorlogtostderr = true;
    FLAGS_stderrthreshold = 0;
    FLAGS_logbufsecs = 0;
    if(boost::filesystem::exists(FLAGS_ie_base_dir)){
        FLAGS_log_dir = FLAGS_ie_base_dir + "/log";
        boost::filesystem::create_directory(FLAGS_log_dir);
    }
    AdjustPPK adjustPPK(FLAGS_ie_base_dir, FLAGS_map_dir);
    adjustPPK.ConfigConvert(FLAGS_ie_base_dir + "/Pandar128_Firetimes.csv",
                            FLAGS_ie_base_dir + "/Pandar128_Correction.csv");
    directory_iterator bag_iter(FLAGS_ie_base_dir);
    directory_iterator bag_end_iter;
    for(; bag_iter != bag_end_iter; bag_iter++) {
        if(bag_iter->path().extension().string() == ".bag"){
            LOG(INFO) << "bag found: " << bag_iter->path().string();
            adjustPPK.ReadBag(bag_iter->path().string());
            break;
        }
    }


    return 0;
}
