#pragma once

#include <pcl/point_types.h>
#include <pcl/point_cloud.h>

struct PointXYZIT {
    PCL_ADD_POINT4D
    float intensity;
    float ring;  /// laser ring number
    double timestamp; /// absolute time
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW // make sure our new allocators are aligned
} EIGEN_ALIGN16;

POINT_CLOUD_REGISTER_POINT_STRUCT(PointXYZIT,
                                  (float, x, x)(float, y, y)(float, z, z)
                                  (float, intensity, intensity)(float, ring, ring)(double, timestamp, timestamp))

typedef PointXYZIT PPoint;
typedef pcl::PointCloud<PPoint> PPointCloud;

// whether add EIGEN_ALIGN16 or not, sizeof(XYZXT) is at least 32 bytes
struct PointXYZXT
{
    PCL_ADD_POINT4D
    //PCL_ADD_RGB
    float intensity;
    float ring;  /// laser ring number
    float label;
    float time;  /// relative time to the end of cloud
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW // make sure our new allocators are aligned
} EIGEN_ALIGN16;

POINT_CLOUD_REGISTER_POINT_STRUCT(PointXYZXT,
                                  (float, x, x)(float, y, y)(float, z, z) //(float, rgb, rgb)
                                  (float, intensity, intensity)(float, ring, ring)(float, label, label)
                                  (float, time, time))

typedef PointXYZXT OutPoint;
typedef pcl::PointCloud<OutPoint> OutPointCloud;

// to support fast_lio and faster_lio
struct PointXYZIT2
{
    PCL_ADD_POINT4D;
    float intensity;
    float time;
    std::uint16_t ring;
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW
} EIGEN_ALIGN16;

POINT_CLOUD_REGISTER_POINT_STRUCT(PointXYZIT2,
                                  (float, x, x)(float, y, y)(float, z, z)(float, intensity, intensity)
                                  (float, time, time)(std::uint16_t, ring, ring))

typedef PointXYZIT2 LioPoint;
typedef pcl::PointCloud<LioPoint> LioPointCloud;

