#include "qrpconfig.h"

#include <iostream>
#include <fstream>

int QRPConfigFileReader::loadQRPConfigFile(FILE *QRPConfigFile)
{
    struct QRPConfigRawData QRPConfig_data;

    fscanf(QRPConfigFile, "q_acc_noise:%lf\n", &(QRPConfig_data.sigma_accl_noise));
    fscanf(QRPConfigFile, "q_gyro_noise:%lf\n", &(QRPConfig_data.sigma_gyro_noise));

    fscanf(QRPConfigFile, "r_pos:%lf,%lf,%lf\n", &(QRPConfig_data.sigma_pos[0]), &(QRPConfig_data.sigma_pos[1]), &(QRPConfig_data.sigma_pos[2]));
    fscanf(QRPConfigFile, "r_theta:%lf,%lf,%lf\n", &(QRPConfig_data.sigma_theta[0]), &(QRPConfig_data.sigma_theta[1]), &(QRPConfig_data.sigma_theta[2]));
    fscanf(QRPConfigFile, "r_vel:%lf,%lf,%lf\n", &(QRPConfig_data.sigma_vel[0]), &(QRPConfig_data.sigma_vel[1]), &(QRPConfig_data.sigma_vel[2]));

    fscanf(QRPConfigFile, "p_pos:%lf\n", &(QRPConfig_data.sigma_init_pos));
    fscanf(QRPConfigFile, "p_vel:%lf\n", &(QRPConfig_data.sigma_init_vel));
    fscanf(QRPConfigFile, "p_dtheta:%lf\n", &(QRPConfig_data.sigma_init_dtheta));

    printf("qrp   = %lf\n", QRPConfig_data.sigma_accl_noise);
    printf("qrp   = %lf\n", QRPConfig_data.sigma_gyro_noise);
    printf("qrp   = %lf,%lf,%lf\n", QRPConfig_data.sigma_pos[0], QRPConfig_data.sigma_pos[1], QRPConfig_data.sigma_pos[2]);
    printf("qrp   = %lf,%lf,%lf\n", QRPConfig_data.sigma_theta[0], QRPConfig_data.sigma_theta[1], QRPConfig_data.sigma_theta[2]);
    printf("qrp   = %lf,%lf,%lf\n", QRPConfig_data.sigma_vel[0], QRPConfig_data.sigma_vel[1], QRPConfig_data.sigma_vel[2]);
    printf("qrp   = %lf\n", QRPConfig_data.sigma_init_pos);
    printf("qrp   = %lf\n", QRPConfig_data.sigma_init_vel);
    printf("qrp   = %lf\n", QRPConfig_data.sigma_init_dtheta);

    raw_data.push_back(QRPConfig_data);

    return raw_data.size();
}

int QRPConfigFileReader::loadQRPConfigAutoFile(FILE *QRPConfigFile)
{
    struct QRPConfigAutoData QRPConfigAutoData;
    
    fscanf(QRPConfigFile, "%*[^\n]\n");

    fscanf(QRPConfigFile, "q_acc_noise:%lf\n", &(QRPConfigAutoData.sigma_accl_noise));
    fscanf(QRPConfigFile, "q_gyro_noise:%lf\n", &(QRPConfigAutoData.sigma_gyro_noise));

    fscanf(QRPConfigFile, "r_pos_x:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_pos_x[0]), &(QRPConfigAutoData.sigma_pos_x[1]), &(QRPConfigAutoData.sigma_pos_x[2]),&(QRPConfigAutoData.sigma_pos_x[3]));
    fscanf(QRPConfigFile, "r_pos_y:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_pos_y[0]), &(QRPConfigAutoData.sigma_pos_y[1]), &(QRPConfigAutoData.sigma_pos_y[2]),&(QRPConfigAutoData.sigma_pos_y[3]));
    fscanf(QRPConfigFile, "r_pos_z:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_pos_z[0]), &(QRPConfigAutoData.sigma_pos_z[1]), &(QRPConfigAutoData.sigma_pos_z[2]),&(QRPConfigAutoData.sigma_pos_z[3]));
    
    fscanf(QRPConfigFile, "r_theta_roll:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_theta_roll[0]), &(QRPConfigAutoData.sigma_theta_roll[1]), &(QRPConfigAutoData.sigma_theta_roll[2]),&(QRPConfigAutoData.sigma_theta_roll[3]));
    fscanf(QRPConfigFile, "r_theta_pitch:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_theta_pitch[0]), &(QRPConfigAutoData.sigma_theta_pitch[1]), &(QRPConfigAutoData.sigma_theta_pitch[2]),&(QRPConfigAutoData.sigma_theta_pitch[3]));
    fscanf(QRPConfigFile, "r_theta_yaw:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_theta_yaw[0]), &(QRPConfigAutoData.sigma_theta_yaw[1]), &(QRPConfigAutoData.sigma_theta_yaw[2]),&(QRPConfigAutoData.sigma_theta_yaw[3]));
    
    fscanf(QRPConfigFile, "r_vel_x:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_vel_x[0]), &(QRPConfigAutoData.sigma_vel_x[1]), &(QRPConfigAutoData.sigma_vel_x[2]),&(QRPConfigAutoData.sigma_vel_x[3]));
    fscanf(QRPConfigFile, "r_vel_y:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_vel_y[0]), &(QRPConfigAutoData.sigma_vel_y[1]), &(QRPConfigAutoData.sigma_vel_y[2]),&(QRPConfigAutoData.sigma_vel_y[3]));
    fscanf(QRPConfigFile, "r_vel_z:%lf,%lf,%lf,%lf\n", &(QRPConfigAutoData.sigma_vel_z[0]), &(QRPConfigAutoData.sigma_vel_z[1]), &(QRPConfigAutoData.sigma_vel_z[2]),&(QRPConfigAutoData.sigma_vel_z[3]));

    fscanf(QRPConfigFile, "p_pos:%lf\n", &(QRPConfigAutoData.sigma_init_pos));
    fscanf(QRPConfigFile, "p_vel:%lf\n", &(QRPConfigAutoData.sigma_init_vel));
    fscanf(QRPConfigFile, "p_dtheta:%lf\n", &(QRPConfigAutoData.sigma_init_dtheta));

    printf("qrp   = %lf\n", QRPConfigAutoData.sigma_accl_noise);
    printf("qrp   = %lf\n", QRPConfigAutoData.sigma_gyro_noise);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_pos_x[0], QRPConfigAutoData.sigma_pos_x[1], QRPConfigAutoData.sigma_pos_x[2],QRPConfigAutoData.sigma_pos_x[3]);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_pos_y[0], QRPConfigAutoData.sigma_pos_y[1], QRPConfigAutoData.sigma_pos_y[2],QRPConfigAutoData.sigma_pos_y[3]);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_pos_z[0], QRPConfigAutoData.sigma_pos_z[1], QRPConfigAutoData.sigma_pos_z[2],QRPConfigAutoData.sigma_pos_z[3]);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_theta_roll[0], QRPConfigAutoData.sigma_theta_roll[1], QRPConfigAutoData.sigma_theta_roll[2],QRPConfigAutoData.sigma_theta_roll[3]);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_theta_pitch[0], QRPConfigAutoData.sigma_theta_pitch[1], QRPConfigAutoData.sigma_theta_pitch[2],QRPConfigAutoData.sigma_theta_pitch[3]);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_theta_yaw[0], QRPConfigAutoData.sigma_theta_yaw[1], QRPConfigAutoData.sigma_theta_yaw[2],QRPConfigAutoData.sigma_theta_yaw[3]);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_vel_x[0], QRPConfigAutoData.sigma_vel_x[1], QRPConfigAutoData.sigma_vel_x[2],QRPConfigAutoData.sigma_vel_x[3]);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_vel_y[0], QRPConfigAutoData.sigma_vel_y[1], QRPConfigAutoData.sigma_vel_y[2],QRPConfigAutoData.sigma_vel_y[3]);
    printf("qrp   = %lf,%lf,%lf,%lf\n", QRPConfigAutoData.sigma_vel_z[0], QRPConfigAutoData.sigma_vel_z[1], QRPConfigAutoData.sigma_vel_z[2],QRPConfigAutoData.sigma_vel_z[3]);
    printf("qrp   = %lf\n", QRPConfigAutoData.sigma_init_pos);
    printf("qrp   = %lf\n", QRPConfigAutoData.sigma_init_vel);
    printf("qrp   = %lf\n", QRPConfigAutoData.sigma_init_dtheta);

    auto_data.push_back(QRPConfigAutoData);

    return auto_data.size();
}