#pragma once

#include <ceres/ceres.h>

#include "../utils/vector_point_type.h"

namespace juefx {

class TranslationDeltaCostFunction {
public:
    static ceres::CostFunction *CreateAutoDiffCostFunction(
        const double scaling_factor, const double scaling_factor_z,
        const Eigen::Vector3d &target_translation) {
        return new ceres::AutoDiffCostFunction<TranslationDeltaCostFunction,
                                               3 /* residuals */,
                                               3 /* translation variables */>(
            new TranslationDeltaCostFunction(scaling_factor,
                                             scaling_factor_z, target_translation));
    }

    template <typename T>
    bool operator()(const T *const translation, T *residual) const {
        residual[0] = scaling_factor_xy_ * (translation[0] - x_);
        residual[1] = scaling_factor_xy_ * (translation[1] - y_);
        residual[2] = scaling_factor_z_ * (translation[2] - z_);
        return true;
    }

private:
    explicit TranslationDeltaCostFunction(
        const double scaling_factor, const double scaling_factor_z,
        const Eigen::Vector3d &target_translation)
        : scaling_factor_xy_(scaling_factor),
          scaling_factor_z_(scaling_factor_z),
          x_(target_translation.x()),
          y_(target_translation.y()),
          z_(target_translation.z()) {}

    TranslationDeltaCostFunction(const TranslationDeltaCostFunction &) = delete;
    TranslationDeltaCostFunction &operator=(const TranslationDeltaCostFunction &) = delete;

    const double scaling_factor_xy_;
    const double scaling_factor_z_;
    const double x_;
    const double y_;
    const double z_;
};
} 