#pragma once

#include "map_matcher_interface.h"
#include "voxel_map/options.h"

namespace juefx {

class CeresVoxelMatcher;

class CeresVoxelMapMatcher : public MapMatcherInterface
{
public:
    CeresVoxelMapMatcher(const MapMatcherOption &option, const CeresVoxelMatcherOption ceres_options);
    ~CeresVoxelMapMatcher();

private:
    float InnerMatch(const VectorCloud &vector_cloud, const GuessPose guess_pose,
                     const Rigid3d &predict_pose, Rigid3d &estimate_pose);

    CeresVoxelMatcherOption ceres_options_;
    CeresVoxelMatcher* matcher_;
};

class FastVoxelMatcher;

class FastVoxelMapMatcher : public MapMatcherInterface
{
public:
    FastVoxelMapMatcher(const MapMatcherOption &option, const FastVoxelMatcherOption fast_options);
    ~FastVoxelMapMatcher();

private:
    float InnerMatch(const VectorCloud &vector_cloud, const GuessPose guess_pose,
                     const Rigid3d &predict_pose, Rigid3d &estimate_pose);

    FastVoxelMatcherOption fast_options_;
    FastVoxelMatcher *matcher_;
};

struct VoxelMapMatcherOption {
    MapMatcherOption option;
    CeresVoxelMatcherOption ceres_option;
    FastVoxelMatcherOption fast_option;
};

class VoxelMapMatcher : public MapMatcherInterface
{
public:
    VoxelMapMatcher(const VoxelMapMatcherOption &options);
    ~VoxelMapMatcher();

private:
    float InnerMatch(const VectorCloud &vector_cloud, const GuessPose guess_pose,
                     const Rigid3d &predict_pose, Rigid3d &estimate_pose);

    CeresVoxelMatcherOption ceres_options_;
    FastVoxelMatcherOption fast_options_;
    CeresVoxelMatcher *ceres_matcher_;
    FastVoxelMatcher *fast_matcher_;
};

}
