#ifndef ADJUSTPPK_H
#define ADJUSTPPK_H
#include "localize_utils/utils.h"
#include "input/file_reader/file_reader.h"

#include "locate/cloudpreprocess.h"
#include "locate/cloudlocalize.h"
#include <sensor_msgs/PointCloud2.h>
#include "convert.h"
#include <rosbag/bag.h>
#include <rosbag/view.h>

#include "dispatch/dispatch.h"
#include "serialize/blockserializer.h"

struct PPKPeriod{
    vector<IERawData> period;
    ulong meshId;
    double startTime = -1;
    double endTime = -1;

    bool InPeriod(double timestamp){
        LOG(INFO) << setprecision(15) << "timestamp " << timestamp
                  << " startTime " << startTime
                  << " endTime " << endTime;
        return (timestamp >= startTime && timestamp < endTime);
    }
};

namespace juefx{

struct MashMap{
    vector<string> pcdPathVec;
    boost::shared_ptr<VoxelMapMatcher> matcher;
};

class AdjustPPK
{
public:
    using Ptr = boost::shared_ptr<AdjustPPK>;
    AdjustPPK(const string &ieBaseDir, const string &mapDir);

    void ConfigConvert(const std::string& firetimeFile, const std::string &correctionFile);

    void ReadBag(const string &bagPath);

private:
    void ReadCenter();

    void OnReceivedCloud(const PPointCloud& cloud);

    void LoadPPK();

    void LoadMapInfo();

    void CalLocalPose();

    void CalPPKPeriod();

    PointCloudInter::Ptr GetBaseFrame(const CloudPacket &cloudData);

    void Undisort(CloudPacket &baseFrame, CloudInfoForMatch &cloudInfo);

    void ConfigMap(double timestamp);

    bool LocateCloud(const CloudInfoForMatch &cloudInfo);

    void LoadMap();

    void Compress(const vector<string> &pcdPathVec,
                  const string &streamPath);

    void LoadMesh(const string &streamPath,
                  MashMap &meshMap);
private:
    string ieBaseDir_;

    string mapDir_;

    vector<IERawData> rawData_;

    vector<PPKPeriod> ppkPeriodVec_;

    unordered_map<ulong, MashMap> meshVec_;

    vector<IsometryData> localPoseVec_;

    boost::shared_ptr<pandar_pointcloud::Convert> convert_;

    size_t ppkIndex_ = 0;

    size_t periodIndex_ = 0;

    boost::shared_ptr<VoxelMapMatcher> currMatcher_;

    Isometry3d imu2base_ = Isometry3d::Identity();

    Isometry3d calib_ = Isometry3d::Identity();

    Vector3d center_ = Vector3d::Zero();
};

} // end of namespace
#endif // ADJUSTPPK_H
