#include "file_reader.h"

#include <iostream>
#include <fstream>
using namespace boost::filesystem;
#define TIME_BUFFER 0.01
DataReader::Ptr dataReaderInstance;

double get_unix_time(int week, double seconds)
{
    static time_t gnssStart = 0;
    if (week == 0)
        return seconds;
    if (gnssStart == 0)
    {
        struct tm gnssTm;
        gnssTm.tm_year = 1980 - 1900; // mktime from 1900
        gnssTm.tm_mon = 0;            // mktime from month 0
        gnssTm.tm_mday = 6;           // GNSS from day 6
        gnssTm.tm_hour = 0;
        gnssTm.tm_min = 0;
        gnssTm.tm_sec = 0;
        gnssTm.tm_isdst = 0; // MUST SET TO 0
        gnssStart = mktime(&gnssTm);
    }
    double final_time = (double)gnssStart + week * 7 * 86400.0 + seconds;
    final_time -= 18; // 18 seconds from 1980.1.6
    return final_time;
}

///////////////////////////////////////////////////////////
// Gps File

#define IND_ 1
bool IERawData::parseFromString(std::string line, IERawData &ie_data)
{
    std::vector<std::string> list = split_string(line, ',');
    if (list.size() == 13)
    {
        ie_data.week = atoi(list[IND_ + 0].c_str());
        ie_data.seconds = atof(list[IND_ + 1].c_str());
        ie_data.unix_time = get_unix_time(ie_data.week, ie_data.seconds);

        ie_data.latitude = atof(list[IND_ + 2].c_str());
        ie_data.longitude = atof(list[IND_ + 3].c_str());
        ie_data.height = atof(list[IND_ + 4].c_str());

        ie_data.n_velocity = atof(list[IND_ + 5].c_str());
        ie_data.e_velocity = atof(list[IND_ + 6].c_str());
        ie_data.u_velocity = atof(list[IND_ + 7].c_str());

        ie_data.roll = atof(list[IND_ + 8].c_str()) * (M_PI / 180); // degree
        ie_data.pitch = atof(list[IND_ + 9].c_str()) * (M_PI / 180);
        ie_data.yaw = atof(list[IND_ + 10].c_str()) * (M_PI / 180);

        ie_data.status = atoi(list[IND_ + 11].c_str());

        if (std::isnan(ie_data.latitude) || std::isnan(ie_data.longitude) || std::isnan(ie_data.height) ||
            std::isnan(ie_data.roll) || std::isnan(ie_data.pitch) || std::isnan(ie_data.yaw)) {
            std::cout << line << std::endl;
            throw;
        }
        return true;
    }
    return false;
}

int IEFileReader::loadIEFile(std::string ieFile, double start_time, double end_time,
                             bool split, std::string outputPath)
{
    char buf[1024];
    std::ifstream ifs(ieFile);
    struct IERawData ie_data;

    std::ofstream* ofs = NULL;
    std::ofstream *ofs2 = NULL;
    int current_seg_start = 0;

    // buffer
    start_time -= TIME_BUFFER;
    end_time += TIME_BUFFER;

    if (!ifs.is_open())
    {
        std::cout << "error when opening ie file, TERMINATED!!!\n"
                  << std::endl;
        throw;
    }

    while (!ifs.eof())
    {
        ifs.getline(buf, 1024);
        std::string line(buf);
        if (IERawData::parseFromString(line, ie_data))
        {
            raw_data.push_back(ie_data);
//            if (start_time <= ie_data.unix_time && ie_data.unix_time <= end_time) {
//                raw_data.push_back(ie_data);
//                if (split) {
//                    int seg_time = (int)(ie_data.unix_time / IE_SPLIT_SEC_LENGTH) * IE_SPLIT_SEC_LENGTH;
//                    if (ofs2 == NULL && seg_time > current_seg_start) {
//                        current_seg_start = seg_time;
//                        std::string output_file = outputPath + "/" + std::to_string(seg_time) + ".txt";
//                        ofs2 = new std::ofstream(output_file.c_str());
//                    }
//                    if (ofs != NULL)
//                        *ofs << line << std::endl;
//                    if (ofs2 != NULL)
//                        *ofs2 << line << std::endl;
//                    if ((ofs == NULL || ie_data.unix_time > current_seg_start + 1) && ofs2 != NULL)
//                    {
//                        if (ofs != NULL) {
//                            ofs->close();
//                            delete ofs;
//                        }
//                        ofs = ofs2;
//                        ofs2 = NULL;
//                    }
//                }
//            }
//            if (ie_data.unix_time > end_time)
//                break;
        }
    }
    ifs.close();
    if (ofs != NULL) {
        ofs->close();
        delete ofs;
    }
    if (ofs2 != NULL)
    {
        ofs2->close();
        delete ofs2;
    }
    return raw_data.size();
}

int IEFileReader::loadIEStartFile(std::string ieStartFile)
{
    char buf[1024];
    std::ifstream ifs(ieStartFile);
    if (!ifs.is_open())
    {
        std::cout << "error when opening ie_start file, TERMINATED!!!\n"
                  << std::endl;
        throw;
    }

    if (!ifs.eof())
    {
        ifs.getline(buf, 1024);
        std::string line(buf);
        std::vector<std::string> list = split_string(line, ',');
        if (list.size() >= 3)
        {   // BLH
            start_data.latitude = atof(list[0].c_str());
            start_data.longitude = atof(list[1].c_str());
            start_data.height = atof(list[2].c_str());
        }
    }
    ifs.close();
    return raw_data.size();
}

IERawData IEFileReader::getIEData(double unix_time)
{
    assert(raw_data[cursor_].unix_time <= unix_time);
    while (cursor_ < raw_data.size() - 1 
            && raw_data[cursor_].unix_time < unix_time 
            && fabs(raw_data[cursor_].unix_time - unix_time) > fabs(raw_data[cursor_ + 1].unix_time - unix_time))
        cursor_++;
    assert(cursor_ != 0);
    assert(cursor_ != raw_data.size() - 1);
    if (raw_data[cursor_].unix_time == unix_time)
        return raw_data[cursor_];
    const IERawData &prev = raw_data[cursor_];
    const IERawData &next = raw_data[cursor_ + 1];
    const double prev_ratio = (next.unix_time - unix_time) / (next.unix_time - prev.unix_time);
    const double next_ratio = (unix_time - prev.unix_time) / (next.unix_time - prev.unix_time);
    IERawData ie_data;
    ie_data.unix_time = unix_time;
#define linear_interpolate(field) \
    ie_data.field = prev_ratio * prev.field + next_ratio * next.field;
    linear_interpolate(latitude);
    linear_interpolate(longitude);
    linear_interpolate(height);
    linear_interpolate(n_velocity);
    linear_interpolate(e_velocity);
    linear_interpolate(u_velocity);
    linear_interpolate(roll);
    linear_interpolate(pitch);
    double next_yaw = (next.yaw - prev.yaw > M_PI) ? 
            next.yaw - M_PI * 2 :
            ((next.yaw - prev.yaw < -M_PI) ? next.yaw + M_PI * 2 : next.yaw);
    ie_data.yaw = prev_ratio * prev.yaw + next_ratio * next_yaw;
    ie_data.status = std::max(prev.status, next.status);
    return ie_data;
}

///////////////////////////////////////////////////////////
// Imu File

bool IMURawData::parseFromString(std::string line, int type, IMURawData &imu_data)
{
    std::vector<std::string> list = split_string(line, ',');
    if (list.size() == 8)
    {
        imu_data.week = atoi(list[0].c_str());
        imu_data.seconds = atof(list[1].c_str());
        imu_data.unix_time = get_unix_time(imu_data.week, imu_data.seconds);
        if (IS_PANDAR40P(type))
        {
            double up_x_acc = atof(list[5].c_str());
            double front_y_acc = atof(list[6].c_str());
            double left_z_acc = atof(list[7].c_str());
            double up_x_angular = atof(list[2].c_str()); // rad
            double front_y_angular = atof(list[3].c_str());
            double left_z_angular = atof(list[4].c_str());

            // origin xyz: up - front - left; need to load as xyz: right - front - up
            imu_data.x_acc = up_x_acc;
            imu_data.y_acc = front_y_acc ;
            imu_data.z_acc = left_z_acc;
            imu_data.x_angular = up_x_angular;
            imu_data.y_angular = front_y_angular;
            imu_data.z_angular = left_z_angular;
            return true;
        }
        else if (IS_HESAIXT32(type) || type == MODE_INNOVUSION)
        {
            // origin xyz: right - front - up
            imu_data.x_acc = atof(list[5].c_str());
            imu_data.y_acc = atof(list[6].c_str());
            imu_data.z_acc = atof(list[7].c_str());
            imu_data.x_angular = atof(list[2].c_str()); // rad
            imu_data.y_angular = atof(list[3].c_str());
            imu_data.z_angular = atof(list[4].c_str());
            return true;
        }
        else
        {
            // default xyz: right - front - up
            // MODE_INCEPTIO xyz: front - left - up
            imu_data.x_acc = atof(list[5].c_str());
            imu_data.y_acc = atof(list[6].c_str());
            imu_data.z_acc = atof(list[7].c_str());
            imu_data.x_angular = atof(list[2].c_str()); // rad
            imu_data.y_angular = atof(list[3].c_str());
            imu_data.z_angular = atof(list[4].c_str());
            return true;
        }
    }
    return false;
}

int IMUFileReader::loadIMUFile(std::string imuFile, int type, double start_time, double end_time)
{
    if (imuFile.empty())
        return makeFakeIMU(start_time, end_time);

    char buf[1024];
    std::ifstream ifs(imuFile);
    struct IMURawData imu_data;

    // buffer
    start_time -= TIME_BUFFER;
    end_time += TIME_BUFFER;

    if (!ifs.is_open()) {
        std::cout << "error when opening imu file, TERMINATED!!!\n" 
                  << std::endl;
        throw;
    }

    ifs.getline(buf, 1024);
    while (!ifs.eof())
    {
        ifs.getline(buf, 1024);
        std::string line(buf);
        if (IMURawData::parseFromString(line, type, imu_data))
        {
            raw_data.push_back(imu_data);
//            if (start_time <= imu_data.unix_time && imu_data.unix_time <= end_time)
//                raw_data.push_back(imu_data);
//            if (imu_data.unix_time > end_time)
//                break;
        }
    }
    ifs.close();
    return raw_data.size();
}

int IMUFileReader::makeFakeIMU(double start_time, double end_time)
{
    for (double time = start_time; time <= end_time; time += 0.01)
    {
        IMURawData imu_data;
        imu_data.week = 0;
        imu_data.seconds = 0;
        imu_data.unix_time = time;
        imu_data.x_acc = 0;
        imu_data.y_acc = 0;
        imu_data.z_acc = 10;
        imu_data.x_angular = 0;
        imu_data.y_angular = 0;
        imu_data.z_angular = 0;
        raw_data.push_back(imu_data);
    }
    return raw_data.size();
}

///////////////////////////////////////////////////////////
// Wheel File

bool WheelFileReader::parseFromString(std::string line, WheelRawData &wheel_data)
{
    std::vector<std::string> list = split_string(line, ',');
    if (list.size() >= 4)
    {
        wheel_data.week = atoi(list[0].c_str());
        wheel_data.seconds = atof(list[1].c_str());
        wheel_data.unix_time = get_unix_time(wheel_data.week, wheel_data.seconds);
        double unit = atof(list[2].c_str());
        wheel_data.movement = (float)(atof(list[3].c_str())/* / unit*/);
        if (list.size() == 5)
            total_movement = atof(list[4].c_str()) / unit;
        else
            total_movement += wheel_data.movement;
        wheel_data.total_movement = total_movement;
        return true;
    }
    return false;
}

int WheelFileReader::loadWheelFile(std::string wheelFile, double start_time, double end_time)
{
    char buf[1024];
    std::ifstream ifs(wheelFile);
    struct WheelRawData wheel_data;

    // buffer
    start_time -= TIME_BUFFER;
    end_time += TIME_BUFFER;

    if (!ifs.is_open())
    {
        std::cout << "error when opening wheel file, TERMINATED!!!\n"
                  << std::endl;
        throw;
    }

    while (!ifs.eof())
    {
        ifs.getline(buf, 1024);
        std::string line(buf);
        if (parseFromString(line, wheel_data))
        {
            raw_data.push_back(wheel_data);
//            if (start_time <= wheel_data.unix_time && wheel_data.unix_time <= end_time)
//                raw_data.push_back(wheel_data);
//            if (wheel_data.unix_time > end_time)
//                break;
        }
    }
    ifs.close();
    return raw_data.size();
}

DataReader::Ptr DataReader::Instance()
{
    if(!dataReaderInstance){
        dataReaderInstance.reset(new DataReader());
    }
    return dataReaderInstance;
}

/////////////////////////////////////////
//data file
int DataReader::LoadIMUFile(std::string imuFile, int type)
{
    count_imu = imu_reader->loadIMUFile(imuFile, type, imu_start_time, imu_end_time);
    return count_imu;
}

int DataReader::LoadWheelFile(std::string wheelFile)
{
    count_wheel = wheel_reader->loadWheelFile(wheelFile, wheel_start_time, wheel_end_time);
    return count_wheel;
}

int DataReader::GetFileNames(std::string path)
{
#ifdef TEST_ESKF
    std::ifstream ifs(path);
    if(ifs){
        string line;
        while(getline(ifs, line))
        {
            double timestamp;
            std::istringstream iss(line);
            string subline;
            getline(iss, subline, ',');
            std::istringstream(subline) >> timestamp;
            timePcdMap_.insert(make_pair(timestamp, ""));
        }
        ifs.close();
    }
#else
    directory_iterator pcd_iter(path);
    directory_iterator pcd_end_iter;
    for(; pcd_iter != pcd_end_iter; pcd_iter++) {
        if(pcd_iter->path().extension().string() != ".pcd"){
            continue;
        }
        string pcd_path = pcd_iter->path().string();
        string pcd_name = pcd_iter->path().stem().string();
        double pcdId;
        try {
            pcdId = stod(pcd_name);
        }
        catch (std::invalid_argument&){
            LOG(INFO) << "header.pcd_id() is not int";
            continue;
        }
        timePcdMap_.insert(make_pair(pcdId, pcd_path));
    }
#endif
    return 0;
}

void DataReader::process()
{
    size_t imuIndex = 0;
    size_t wheelIndex = 0;
    for(auto it = timePcdMap_.begin(); it != timePcdMap_.end(); it++){
        double currCloudTime = it->first;
//        LOG(INFO) << setprecision(15) << "currCloudTime: " << currCloudTime;
        bool imu_preprocess;
        bool wheel_preprocess;
        do{
            imu_preprocess = false;
            wheel_preprocess = false;
            double currImuTime = -1;
            IMURawData imuRawData;
            if(imuIndex < imu_reader->raw_data.size()){
                imuRawData = imu_reader->raw_data.at(imuIndex);
                currImuTime = imuRawData.unix_time;
            }
            double currWheelTime = -1;
            WheelRawData wheelRawData;
            if(wheelIndex < wheel_reader->raw_data.size()){
                wheelRawData = wheel_reader->raw_data.at(wheelIndex);
                currWheelTime = wheelRawData.unix_time;
            }
            if(currImuTime >= 0 && currImuTime <= currWheelTime
                    && currImuTime <= (currCloudTime/*+ 0.1*/)){
                imuCallback_(imuRawData);
                imuIndex++;
                imu_preprocess = true;
                LOG(INFO) << setprecision(15) << "currImuTime: " << currImuTime;
                continue;
            }
            if(currWheelTime >= 0 && currWheelTime <= currImuTime
                    && currWheelTime <= (currCloudTime/* + 0.1*/)){
                wheelCallback_(wheelRawData);
                wheelIndex++;
                wheel_preprocess = true;
                LOG(INFO) << setprecision(15) << "currWheelTime: " << wheelRawData.unix_time;
                continue;
            }

        } while (imu_preprocess || wheel_preprocess);
        PointCloudInter cloud;
#ifdef TEST_ESKF
        usleep(10000);
#else
        pcl::io::loadPCDFile(it->second, cloud);
        usleep(100000);
#endif
        LOG(INFO) << setprecision(15) << "currCloudTime: " << currCloudTime;
        cloudCallback_(cloud, currCloudTime);
    }
}

int IEReader::LoadIEFile(string ieFile)
{
    count_ie = ie_reader->loadIEFile(ieFile, ie_start_time, ie_end_time);
    return count_ie;
}

