#include "wheel_receiver.h"

namespace juefx{

WheelReceiver::WheelReceiver(const SysController::Ptr &sys)
    : SensorSubscriber(sys)
{
    boost::function<void(const WheelRawData&)> wheelCallback =
            boost::bind(&WheelReceiver::Callback, this, _1);
    DataReader::Instance()->registWheelCb(wheelCallback);
}

void WheelReceiver::Callback(
        const WheelRawData &wheelRawData)
{
    if(lastWheelTime_ >= 0){
        double wheelDeltaTime = wheelRawData.unix_time - lastWheelTime_;
        WheelPacketPtr packet(new WheelPacket);
        packet->sensorDataType = WHEEL_DATA;
        packet->timestamp = wheelRawData.unix_time;
        packet->linear = {wheelRawData.movement / wheelDeltaTime, 0, 0};
        sys_->OnNewDataReceived(packet);
    }
    lastWheelTime_ = wheelRawData.unix_time;
}

} // end of namespace
