#include "cloud_subscriber.h"

namespace juefx{

CloudSubscriber::CloudSubscriber(
        const SysController::Ptr &sys,
        ros::NodeHandle* node_handle,
        const std::string& topic)
    :RosSubscriber(sys, node_handle, topic)
{
    StartListeningToRos();
}

void CloudSubscriber::StartListeningToRos()
{
    _subscriber.reset(new Subscriber(
        *_node_handle, _topic, _msg_queue_size));
    _subscriber->registerCallback(
        &CloudSubscriber::Callback, this);
}

void CloudSubscriber::Callback(const CloudT::Ptr &msg)
{
//    LOG(INFO) << setprecision(15) << "recv cloud: " << msg->header.stamp.toSec();
    CloudPacketPtr packet(new CloudPacket);
    packet->sensorDataType = CLOUD_DATA;
    packet->timestamp = msg->header.stamp.toSec();
    packet->cloud = RosCloudToCloud(msg);
    sys_->OnNewDataReceived(packet);
}

PointCloudInter CloudSubscriber::RosCloudToCloud(
    const CloudT::Ptr& msg) {
  pcl::PCLPointCloud2 pcl_pc2;
  pcl_conversions::toPCL(*msg, pcl_pc2);
  PointCloudInter pointcloud;
  pcl::fromPCLPointCloud2(pcl_pc2, pointcloud);
  return pointcloud;
}

} // end of namespace
