#include "imu_subscriber.h"

namespace juefx{

IMUSubscriber::IMUSubscriber(
        const SysController::Ptr &sys,
        ros::NodeHandle* node_handle,
        const std::string& topic)
    :RosSubscriber(sys, node_handle, topic)
{
    StartListeningToRos();
}

void IMUSubscriber::StartListeningToRos()
{
    _subscriber.reset(new Subscriber(
        *_node_handle, _topic, _msg_queue_size));
    _subscriber->registerCallback(
        &IMUSubscriber::Callback, this);
}

void IMUSubscriber::Callback(const ImuT::Ptr &msg)
{
    IMUPacketPtr packet(new IMUPacket);
    packet->sensorDataType = IMU_DATA;
    packet->timestamp = msg->header.stamp.toSec();
    packet->angular_velocity = {(float)msg->angular_velocity.x,
                                (float)msg->angular_velocity.y,
                                (float)msg->angular_velocity.z};
    packet->linear_acceleration = {(float)msg->linear_acceleration.x,
                                   (float)msg->linear_acceleration.y,
                                   (float)msg->linear_acceleration.z};


//    std::ofstream ofs("/home/juefx/guess_data.txt", ios::app);
//    LOG(INFO) << setprecision(15) << "imu," << packet->timestamp << ","
//        << packet->angular_velocity.x() << "," << packet->angular_velocity.y() << ","
//        << packet->angular_velocity.z()<< "," << packet->linear_acceleration.x() << ","
//        << packet->linear_acceleration.y() << "," << packet->linear_acceleration.z() << endl;
    sys_->OnNewDataReceived(packet);
}

} // end of namespace
