#include "imupose_subscriber.h"

namespace juefx{

IMUPoseSubscriber::IMUPoseSubscriber(
        const SysController::Ptr &sys,
        ros::NodeHandle* node_handle,
        const std::string& topic)
    :RosSubscriber(sys, node_handle, topic)
{
    StartListeningToRos();
}

void IMUPoseSubscriber::StartListeningToRos()
{
    _subscriber.reset(new Subscriber(
        *_node_handle, _topic, _msg_queue_size));
    _subscriber->registerCallback(
        &IMUPoseSubscriber::Callback, this);
}

void IMUPoseSubscriber::Callback(const PoseT::Ptr &msg)
{
    ImuPosePacketPtr packet(new ImuPosePacket);
    packet->sensorDataType = IMU_POSE_DATA;
    packet->timestamp = msg->header.stamp.toSec();
    packet->rotation = Quaterniond(msg->pose.orientation.w,
                                   msg->pose.orientation.x,
                                   msg->pose.orientation.y,
                                   msg->pose.orientation.z);

    sys_->OnNewDataReceived(packet);
}

} // end of namespace
