#include "rosbag_reader.h"

namespace juefx{

RosBagReader::RosBagReader(
        const SysController::Ptr &sys, const string &bagPath)
    :SensorSubscriber(sys)
{
    bag_.open(bagPath);

}

RosBagReader::~RosBagReader()
{
    bag_.close();
}

PointCloudInter RosCloudToCloud(
    const CloudT::Ptr& msg) {
  pcl::PCLPointCloud2 pcl_pc2;
  pcl_conversions::toPCL(*msg, pcl_pc2);
  PointCloudInter pointcloud;
  pcl::fromPCLPointCloud2(pcl_pc2, pointcloud);
  return pointcloud;
}

void RosBagReader::Play()
{
    rosbag::View view(bag_);
    for(const rosbag::MessageInstance &m : view){
        if(m.isType<CloudT>()){
            const CloudT::Ptr &mm
                    = m.instantiate<CloudT>();
            CloudPacketPtr packet(new CloudPacket);
            packet->sensorDataType = CLOUD_DATA;
            packet->timestamp = mm->header.stamp.toSec();
            packet->cloud = RosCloudToCloud(mm);
            sys_->OnNewDataReceived(packet);
        }else if(m.isType<ImuT>()){
            const ImuT::Ptr &mm
                    = m.instantiate<ImuT>();
            IMUPacketPtr packet(new IMUPacket);
            packet->sensorDataType = IMU_DATA;
            packet->timestamp = mm->header.stamp.toSec();
            packet->angular_velocity = {(float)mm->angular_velocity.x,
                                        (float)mm->angular_velocity.y,
                                        (float)mm->angular_velocity.z};
            packet->linear_acceleration = {(float)mm->linear_acceleration.x,
                                           (float)mm->linear_acceleration.y,
                                           (float)mm->linear_acceleration.z};
            sys_->OnNewDataReceived(packet);
        }else if(m.isType<OdomT>()){
            const OdomT::Ptr &mm
                    = m.instantiate<OdomT>();
            WheelPacketPtr packet(new WheelPacket);
            packet->sensorDataType = WHEEL_DATA;
            packet->timestamp = mm->header.stamp.toSec();
            packet->linear = {mm->twist.twist.linear.x,
                             mm->twist.twist.linear.y,
                             mm->twist.twist.linear.z};
            sys_->OnNewDataReceived(packet);
        }
        if(m.isType<CloudT>()){
            usleep(80000);
        }else{
            usleep(100);
        }
    }
}

}
