#pragma once
#define PCL_NO_PRECOMPILE
#include "utils.h"
#include <pcl/point_types.h>
#include <pcl/point_cloud.h>
#include <pcl/io/pcd_io.h>

typedef pcl::PointXYZI PointJ;
typedef pcl::PointCloud<PointJ> PointCloudJ;
typedef pcl::PointCloud<pcl::PointXYZ> PointCloudXYZ;

struct PointInter {
    PCL_ADD_POINT4D
    float intensity;
    float ring;
    float label = 0;
    float time;
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW  // make sure our new allocators are aligned
} EIGEN_ALIGN16;

POINT_CLOUD_REGISTER_POINT_STRUCT(
    PointInter, (float, x, x)(float, y, y)(float, z, z)
        (float, intensity, intensity)(float, ring, ring)(float, label, label)
        (float, time, time))

typedef pcl::PointCloud<PointInter> PointCloudInter;

struct CloudPacket : public SensorData
{
    CloudPacket() {}
    PointCloudInter cloud;
};
typedef boost::shared_ptr<CloudPacket> CloudPacketPtr;

struct FilterPoseInfo
{
    Isometry3d filterPose = Isometry3d::Identity();
    Isometry3d increasePose = Isometry3d::Identity();
    bool hasGnss = false;
    Vector3d blh;
};

struct CloudInfoForMatch : public ResultData
{
    CloudInfoForMatch() {
        resultDataType = CLOUDINFO_DATA;
    }
    PointCloudJ::Ptr frame;
    FilterPoseInfo filterPoseInfo;

};
typedef boost::shared_ptr<CloudInfoForMatch> CloudInfoForMatchPtr;
