#ifndef CLOUDLOCALIZE_H
#define CLOUDLOCALIZE_H

#include "localize_utils/utils.h"
#include "localize_utils/pcl_point_type.h"
#include "system/parameter.h"
#include "state/state.h"
#include "navigation/poseguesser.h"

#include "../libs/localize/voxel_map_matcher.h"

namespace juefx{

struct MatchResult
{
    MatchPrecisionState state;
    Isometry3f finalPose = Isometry3f::Identity();
    double timestamp = 0;
};

class CloudLocalize
{
public:
    using Ptr = boost::shared_ptr<CloudLocalize>;
    CloudLocalize();

    void Reset();

    static CloudLocalize::Ptr Instance();

    void LoadMap();

    bool FindMapPose(const FilterPoseInfo &filterPoseInfo,
                     Isometry3d &mapPose,
                     bool isCurrPos = true);

    bool IsMapLoaded(const Vector3d& pose);

    void LoadArea(const Vector3d &pose);

    bool MatchMap(const PointCloudJ::Ptr pcloud,
                  GuessPose &guessPose,
                  MatchResult &result);

    Isometry3d GetGuessFilter2map(){
        return guessFilter2map;
    }
private:
    void DeterminePrecision(float rotation);

    void CheckMatchResult(float score);

    bool FindReferencePose(bool hasGnss, const Vector3d &blh, Isometry3d &mapPose);

    int CheckCloud(const PointCloudJ::Ptr &cloud);
private:
    boost::shared_ptr<VoxelMapMatcher> matcher_;

    int Precision = MATCH_PRECISION_INIT;
    uint32_t fastPassCnt = 0;
    bool LastMatchedSuccess = false;
    bool InitFastMatched = false;
    bool InitMatched = false;
    Isometry3d MatchedPose;
    Isometry3d guessFilter2map;
    Isometry3d currFilterPose;


};

} // end of namespace
#endif // CLOUDLOCALIZE_H
