#include "cloudpreprocess.h"

namespace juefx{

CloudPreprocess::Ptr cloudPreprocessInstance;

CloudPreprocess::CloudPreprocess()
{

}

CloudPreprocess::Ptr CloudPreprocess::Instance()
{
    if(!cloudPreprocessInstance){
        cloudPreprocessInstance.reset(new CloudPreprocess());
    }
    return cloudPreprocessInstance;
}

PointCloudInter::Ptr CloudPreprocess::GetBaseFrame(
        const CloudPacket &cloudData)
{
    PointCloudInter::Ptr frame(new PointCloudInter);
    *frame = cloudData.cloud;
    pcl::transformPointCloud(*frame, *frame, Parameter::Instance()->GetCalib().cast<float>());
    pcl::transformPointCloud(*frame, *frame, Parameter::Instance()->GetImu2Base().cast<float>());
    return frame;
}

inline Isometry3d interpFromMapPoseVec(
        const double &time,
        const vector<IsometryData> *periodPoseVec,
        bool fromBegin = true){
    size_t size = periodPoseVec->size();
    static size_t periodPoseId = 1;
    if(fromBegin){
        periodPoseId = 1;
    }
    for(; periodPoseId < size; periodPoseId++){
        const IsometryData& pre = periodPoseVec->at(periodPoseId - 1);
        const IsometryData& pro = periodPoseVec->at(periodPoseId);
        if(periodPoseId == 1 && time < pre.timestamp){
            return pre.pose;
        }else if(time >= pre.timestamp &&
                  time < pro.timestamp){
              double deltTime = pro.timestamp - pre.timestamp;
              double timeDiff = time - pre.timestamp;
              double radio = timeDiff / deltTime;
              Isometry3d interpedMapPose;
              interpedMapPose.translation() = pre.pose.translation() + (pro.pose.translation() - pre.pose.translation()) * radio;
              interpedMapPose.linear() =
                      Quaterniond(pre.pose.linear()).slerp(radio, Quaterniond(pro.pose.linear())).toRotationMatrix();
              return interpedMapPose;
        }
    }
    if(fromBegin){
        periodPoseId = 1;
    }
    return periodPoseVec->back().pose;
}

bool CloudPreprocess::Undisort(
        PointCloudInter::Ptr &input, PointCloudJ::Ptr &undisortedFrame,
        PointCloudJ *mappedFrame, double frameTime,
        const vector<IsometryData> *periodPoseVec,
        Isometry3d &framePose, Isometry3d &increasePose)
{
    framePose = interpFromMapPoseVec(frameTime, periodPoseVec);
    Isometry3f framePoseInv = framePose.inverse().cast<float>();
    Isometry3f lastPPose;
    double lastPTime = -1;
    for(auto &p : *input){
        double pTime = p.time + frameTime;
        Isometry3f currPPose;
        if(pTime - lastPTime < 1e-6){
            currPPose = lastPPose;
        }else{
            currPPose = interpFromMapPoseVec(pTime, periodPoseVec, false).cast<float>();
            lastPPose = currPPose;
            lastPTime = pTime;
        }
        if(0 == mappedFrame->size()){
            increasePose = currPPose.inverse().cast<double>() * framePose;
        }
        Vector3f mappedPoint = currPPose * Vector3f(p.x, p.y, p.z);
        PointJ pointMapped;
        pointMapped.x = mappedPoint.x();
        pointMapped.y = mappedPoint.y();
        pointMapped.z = mappedPoint.z();
        pointMapped.intensity = p.intensity;
        mappedFrame->push_back(pointMapped);
        Vector3f undisortedPoint = framePoseInv * mappedPoint;
        PointJ point;
        point.x = undisortedPoint.x();
        point.y = undisortedPoint.y();
        point.z = undisortedPoint.z();
        point.intensity = p.intensity;
        undisortedFrame->push_back(point);
    }
    return true;
}

} // end of namespace
