#ifndef FILTER_H
#define FILTER_H

#include "localize_utils/utils.h"
#include "system/parameter.h"
#include "state/state.h"

#include "eskf/lib/ESKF.h"
#include "eskf/lib/qrpconfig.h"
#include "localize_utils/ring_queue.h"

namespace juefx{

struct EskfState{
    ESKF eskf;
    double timestamp = 0;
};

class Filter
{
public:
    using Ptr = boost::shared_ptr<Filter>;
    Filter();

    void Reset();

    static Filter::Ptr Instance();

    bool Init();

    void UpdateImu(const Eigen::Vector3f &acc,
                   const Eigen::Vector3f &gryo,
                   lTime now);

    void MeasureWheel(const Eigen::Vector3f &wheel_vel,
                      double speed_range,
                      lTime now);


    bool GetStateTransform(IsometryData &pose);

    bool SwitchBack(double timestamp);

    void MeasureLidar(const Isometry3f &pose,
                      double timestamp,
                      MatchPrecisionState precisionState);

private:
    boost::shared_ptr<ESKF> eskf_;

    std::vector<QRPConfigAutoData> qrpconfig_data;

    RingQueue<EskfState> eskfQueue_;
};

} // end of namespace
#endif // FILTER_H
