#ifndef POSEGUESSER_H
#define POSEGUESSER_H

#include "localize_utils/utils.h"
#include "localize_utils/pcl_point_type.h"
#include <GeographicLib/LocalCartesian.hpp>
#include <pcl/octree/octree.h>
#include "localize_utils/ring_queue.h"

using namespace GeographicLib;

namespace juefx{

class PoseGuesser
{
public:
    using Ptr = boost::shared_ptr<PoseGuesser>;
    PoseGuesser();

    void Reset();

    static PoseGuesser::Ptr Instance();

    void SetMapName(const string& mapName);

    bool LoadSlamTraj(const string &slamTrajPath,
                      const string &gnssPath);

    float GetMapPoseByBlh(const Vector3d &blh, Isometry3d &mapPose);

    static bool InterpBlh(const RingQueue<GNSSPacket> *gnssQueue,
                   double timestamp,
                   Vector3d& interpedBlh);

    string GetMapName(){
        return mapName_;
    }
private:
    void ConstractLocalTraj(const vector<IsometryData> &blhVec,
                            const vector<IsometryData> &trajPoseVec);

    void CalGnssLocal2Map();
private:
    vector<Isometry3d> interpedTrajPoseVec_;

    PointCloudXYZ gnssLocalTraj_;

    boost::shared_ptr<LocalCartesian> proj_;

    Isometry3d gnssLocal2Map_ = Isometry3d::Identity();

    pcl::octree::OctreePointCloudSearch<pcl::PointXYZ> octree_;

    string mapName_;
};

} // end of namespace
#endif // POSEGUESSER_H
