#ifndef STATE_H
#define STATE_H

#include "localize_utils/utils.h"

namespace juefx{

#define IDLE 0
#define IMU_CALIBING 0xF
#define IMU_CALIBED 0xF0
#define TRAJ_HAS_GNSS 0xF00
#define USING_GNSS 0xF000
#define GNSS_LOCKING 0xF0000
#define GNSS_LOCKED 0xF00000
#define LOADING_MAP 0xF000000
#define MAP_LOADED 0xF0000000
#define FILTER_INIT 0xF00000000
#define INIT_MATCHING 0xF000000000
#define INIT_MATCHED 0xF0000000000
#define MATCHING 0xFF000000000

class State
{
public:
    using Ptr = boost::shared_ptr<State>;
    State();

    void Reset();

    static State::Ptr Instance();

    inline ulong GetState(){
        return state_;
    }

    void AddState(ulong state);

    void RemoveState(ulong state);

    bool HasState(ulong state);

    bool IsIdle(){
        return state_ == 0;
    }

private:
    ulong state_ = 0;
};

} // end of namespace
#endif // STATE_H
