#include "parameter.h"
namespace juefx{

Parameter::Ptr paramInstance;

Parameter::Parameter()
{

}

void Parameter::Reset()
{
    paramInstance.reset();
}

Parameter::Ptr Parameter::Instance()
{
    if(!paramInstance){
        LOG(INFO) << "new paramInstance";
        paramInstance.reset(new Parameter());
    }
    return paramInstance;
}

void Parameter::init(const string &baseDir)
{
    baseDir_ = baseDir;
    mapPath_ = baseDir_ + "/stream/";
    Matrix4d m;
    /////////////// cpt ///////////////
    m << -0.043905, -0.999033, 00.002312, 00.54,
            00.994052, -0.043455, 00.099861, 00.28,
            -0.099664, 00.006683, 00.994999, 00.06,
            00.0000000, 00.0000000, 00.0000000, 01.0000000;

    /////////////// ins ///////////////
//    m << 0.0393253, -0.999222, -0.00291833, 0.179266,
//            00.999226, 0.0393211, 0.00148739, -0.0160456,
//            -0.00137148, -0.00297457, 00.999995, 0.0218012,
//            00.0000000, 00.0000000, 00.0000000, 01.0000000;

    /////////////// 11 ///////////////
//    m << -0.9999964, 0.0024037, 0.0012044, 00.0249618,
//            -0.0024065, -0.9999942, -0.0023893, -0.0044889,
//            0.0011987, -0.0023922, 0.9999964, 00.3328200,
//            00.0000000, 00.0000000, 00.0000000, 01.0000000;

    calib_.translation() = m.block(0, 3, 3, 1);
    calib_.linear() = m.block(0, 0, 3, 3);
    Quaterniond calibQ(calib_.linear());
    auto calibRpy = RotationQuaternionToEulerVector(calibQ);
    LOG(INFO) << "calibRpy: " << calibRpy.transpose();

    std::ifstream ifs(baseDir_ + "/stream.offset");
    if(ifs) {
        string line;
        getline(ifs, line);
        {
            std::istringstream iss(line);
            string subline;
            getline(iss, subline, ',');
            std::istringstream(subline) >> offset_.x();
            getline(iss, subline, ',');
            std::istringstream(subline) >> offset_.y();
        }
        ifs.close();
    }
    ifs.open(baseDir_ + "/imu2base.calib");
    if(ifs) {
        string line;
        float r, p, y;
        getline(ifs, line);
        {
            std::istringstream iss(line);
            string subline;
            getline(iss, subline, ',');
            std::istringstream(subline) >> r;
            getline(iss, subline, ',');
            std::istringstream(subline) >> p;
            getline(iss, subline, ',');
            std::istringstream(subline) >> y;
        }
        ifs.close();

  //      fprintf(stderr, "imu2base_   %f, %f, %f\n", r, p, y);
        imu2base_.linear() =
                Matrix3f(AngleAxisf(y, Vector3f::UnitZ())
                * AngleAxisf(p, Vector3f::UnitY())
                * AngleAxisf(r, Vector3f::UnitX())).cast<double>();
    }
    localToSlamstart_.x() = slamStart_.x() - offset_.x();
    localToSlamstart_.y() = slamStart_.y() - offset_.y();

    ifs.open(baseDir_ + "/init_pose.txt");
    if(ifs) {
        string line;
        Vector3d rpy;
        getline(ifs, line);
        {
            std::istringstream iss(line);
            string subline;
            getline(iss, subline, ',');
            std::istringstream(subline) >> initPose_.translation().x();
            getline(iss, subline, ',');
            std::istringstream(subline) >> initPose_.translation().y();
            getline(iss, subline, ',');
            std::istringstream(subline) >> initPose_.translation().z();
            getline(iss, subline, ',');
            std::istringstream(subline) >> rpy.x();
            getline(iss, subline, ',');
            std::istringstream(subline) >> rpy.y();
            getline(iss, subline, ',');
            std::istringstream(subline) >> rpy.z();
        }
        LOG(INFO) << "initPose_.translation(): " << initPose_.translation().transpose();
        LOG(INFO) << "rpy: " << rpy.transpose();
        ifs.close();
        initPose_.linear() =
        Matrix3d(AngleAxisd(rpy.z(), Vector3d::UnitZ())
        * AngleAxisd(rpy.y(), Vector3d::UnitY())
        * AngleAxisd(rpy.x(), Vector3d::UnitX()));
    }
    ifs.open(baseDir_ + "/imu2base_lever_arm.txt");
    if(ifs) {
        string line;
        getline(ifs, line);
        {
            std::istringstream iss(line);
            string subline;
            getline(iss, subline, ',');
            std::istringstream(subline) >> arm_.x();
            getline(iss, subline, ',');
            std::istringstream(subline) >> arm_.y();
            getline(iss, subline, ',');
            std::istringstream(subline) >> arm_.z();
        }
        ifs.close();
        LOG(INFO) << "arm_: " << arm_.transpose();
    }
    ifs.open(baseDir_ + "/eskf_init_pose.txt");
    if(ifs) {
        string line;
        Vector3d rpy;
        getline(ifs, line);
        {
            std::istringstream iss(line);
            string subline;
            getline(iss, subline, ',');
            std::istringstream(subline) >> eskf_init_pose_.translation().x();
            getline(iss, subline, ',');
            std::istringstream(subline) >> eskf_init_pose_.translation().y();
            getline(iss, subline, ',');
            std::istringstream(subline) >> eskf_init_pose_.translation().z();
            getline(iss, subline, ',');
            std::istringstream(subline) >> rpy.x();
            getline(iss, subline, ',');
            std::istringstream(subline) >> rpy.y();
            getline(iss, subline, ',');
            std::istringstream(subline) >> rpy.z();
        }
        LOG(INFO) << "eskf_init_rot_ rot: " << rpy.transpose();
        ifs.close();
        eskf_init_pose_.linear() =
        Matrix3f(AngleAxisf(rpy.z(), Vector3f::UnitZ())
        * AngleAxisf(rpy.y(), Vector3f::UnitY())
        * AngleAxisf(rpy.x(), Vector3f::UnitX()));
    }

    string center_dir = baseDir_ + "/meshes.center";
    std::ifstream ifs_center(center_dir);
    if(!ifs_center) {
        LOG(WARNING) << "open centerPath_ file error: " << center_dir;
    }else{
        string line;
        getline(ifs_center, line);
        {
            double lat, lon, h;
            istringstream iss(line);
            string subline;
            getline(iss, subline, ',');
            istringstream(subline) >> lat;
            getline(iss, subline, ',');
            istringstream(subline) >> lon;
            getline(iss, subline, ',');
            istringstream(subline) >> h;
            center_ = {lat, lon, h};
            LOG(INFO) << setprecision(15) << "get center: " << center_.transpose();
        }
        ifs_center.close();
    }
}





} // end of namespace
