#ifndef SYSCONTROLLER_H
#define SYSCONTROLLER_H
#include "localize_utils/utils.h"
#include "localize_utils/pcl_point_type.h"
#include "localize_utils/ring_queue.h"
#include "parameter.h"
#include "state/state.h"

#include "navigation/imucaliber.h"
#include "navigation/filter.h"
#include "navigation/poseguesser.h"
#include "navigation/mapdetector.h"

#include "locate/cloudpreprocess.h"
#include "locate/cloudlocalize.h"

#include "output/publisher.h"

namespace juefx{

class SysController
{
public:
    using Ptr = boost::shared_ptr<SysController>;
    SysController();

    ~SysController();

    void OnNewDataReceived(const SensorDataPtr& data);

    void SetCloseMapCb(boost::function<void(const vector<string>&)> closeMapCb){
        closeMapCb_ = closeMapCb;
    }
private:
    void WorkTrdFunction();

    void BackendTrdFunction();

    bool WaitForData(boost::mutex& mtx,
                     boost::condition_variable& cond);

    void EnqueueData(const SensorDataPtr &data);

    void HandleImu(IMUPacket imu);

    void HandleWheel(WheelPacket wheel);

    void HandleGnss(GNSSPacket gnss);

    void HandleCloud(CloudPacket cloud);

    template<typename T>
    vector<T> GetPeriod(RingQueue<T> &queue, double frontPTime, double backPTime){
        T pose;
        vector<T> ret;
        queue.SwitchBack(frontPTime);
        while(queue.getNext(pose)){
//            LOG(INFO) << setprecision(15) << "pose.timestamp: " << pose.timestamp;
            if(pose.timestamp < frontPTime){
                continue;
            }
            if(pose.timestamp >= backPTime){
                break;
            }
            ret.emplace_back(pose);
        }
        return ret;
    }

    bool Measure();

    void GetPoseAndPublish();

    Isometry3d GuessFutureFilterPose(const FilterPoseInfo &filterPoseInfo, float sec);

    void ReadMeasureLidarPose();

    void SetNewFilterFramePose(CloudInfoForMatch &cloudInfo);
private:
    ThreadPtr workTrd_, backendTrd_;

    bool requestToEnd_ = false;

    boost::mutex netMtx_, workMtx_, backendMtx_;

    boost::condition_variable workCond_, backendCond_;

    float lidarInterval_ = 0;

    RingQueue<IMUPacket> imuQueue;
    RingQueue<WheelPacket> wheelQueue;
    RingQueue<GNSSPacket> gnssQueue;
    RingQueue<IsometryData> poseQueue;
    queue<CloudInfoForMatch> cloudInfoQueue;
    queue<MatchResult> measureLidarQueue;
    queue<Vector3d> positionToLoadQueue;
    queue<CloudPacket> cloudQueue;
    double CurrTimestamp = -1;
    uint16_t FutureDisReselution = 30;

    boost::function<void(const vector<string>&)> closeMapCb_;
};

} // end of namespace
#endif // SYSCONTROLLER_H
