#pragma once

#include "matcher_interface.h"

namespace juefx {

// Default setting:
//      search x-y-z for 1+ meters, r-p-y for 3-4 degrees
//      T 0.4 R 32 = high: 4.44983 low: 7.31353 - high: 1.4307 low: 3.25313 trans: 0.145357 rotat: 2.16764
//      T 0.4 R 10 = high: 4.44983 low: 7.31353 - high: 1.26212 low: 3.18282 trans: 0.153509 rotat: 0.256997
//      T 0.25 R 10 = high: 4.44983 low: 7.31353 - high: 1.26165 low: 3.18205 trans: 0.0603463 rotat: 0.25759
// search range:
//      50 for indoor
//      80 for outdoor - few points of a sign far away can change the localization result
struct CeresMatcherOption {
    double T_weight_xy = 0.25;
    double T_weight_z = 0.25;
    double R_weight_pitchroll = 10;
    double R_weight_yaw = 10;
    float voxel_filter_size = 0.5;
    float max_range = 80;
    int threads_num = 4;
};

class ScanMatcher;
class Grid;

class CeresMatcher : public MatcherInterface
{
public:
    CeresMatcher(const CeresMatcherOption& option);
    ~CeresMatcher();

    void LoadPointCloudMap(const PointCloudJ::Ptr pcloud_map,
                           float high_resolution, float low_resolution);
    float MatchPointCloud(const PointCloudJ::Ptr pcloud, const Eigen::Isometry3f init_pose,
                          Eigen::Isometry3f &final_pose);

private:
    void BuildGird(const PointCloudJ::Ptr pcloud_map, float resolution);
    void FreeGrids();

private:
    ScanMatcher* matcher_;
    std::vector<const Grid*> map_grids_;
    CeresMatcherOption options_;
};

}
