#include "icp_matcher.h"

#include <pcl/registration/gicp.h>
#include <glog/logging.h>

#include "utils/timer.h"
#include "utils/util.hpp"

using GicpMatcher = pcl::GeneralizedIterativeClosestPoint<PointJ, PointJ>;

namespace juefx {

ICPMatcher::ICPMatcher() {
    GicpMatcher *inner_matcher = new GicpMatcher();
    matcher_ = (ScanMatcher *)inner_matcher;
}

ICPMatcher::~ICPMatcher() {
    GicpMatcher *inner_matcher = (GicpMatcher *)matcher_;
    delete inner_matcher;
}

void ICPMatcher::LoadPointCloudMap(const PointCloudJ::Ptr pcloud_map) {
    GicpMatcher *inner_matcher = (GicpMatcher *)matcher_;
    inner_matcher->setInputTarget(pcloud_map);
    LOG(INFO) << "Load pcloud map finished.";
}

float ICPMatcher::MatchPointCloud(const PointCloudJ::Ptr pcloud, const Eigen::Isometry3f init_pose,
                                  Eigen::Isometry3f &final_pose) {
    GicpMatcher *inner_matcher = (GicpMatcher *)matcher_;
    if (inner_matcher->getInputTarget().get() == NULL)
        return -1;

    TIMER_START(start);
    inner_matcher->setInputSource(pcloud);
    inner_matcher->setMaximumIterations(50);
    PointCloudJ::Ptr result_pcloud(new PointCloudJ);
    inner_matcher->align(*result_pcloud, init_pose.matrix());
    final_pose.matrix() = inner_matcher->getFinalTransformation();

    char buf[1024];
    Eigen::Vector3d trans3 = final_pose.translation().cast<double>();
    Eigen::Vector3d euler3 = juefx::matrix_2_euler_rpy(final_pose.rotation().cast<double>());
    snprintf(buf, 1023, "{ pos: %.3f %.3f %.3f, euler: %.5f %.5f %.5f }",
             trans3.x(), trans3.y(), trans3.z(),
             euler3.x(), euler3.y(), euler3.z());
    TIMER_END(start, seconds);
    LOG(INFO) << "Match scan with map, transform: " << buf << ", cost " << seconds << " sec.";
    return 0;
}
}