#include "icp_ceres.h"

#include <math.h>

namespace juefx {

void IcpProblem::AddIcpPairSet(const IcpPairSet &pair_set, Eigen::Quaterniond &q, Eigen::Vector3d &t)
{
    return AddIcpPairSet(pair_set, Eigen::Quaterniond::Identity(), Eigen::Vector3d::Zero(), q, t);
}

void IcpProblem::AddIcpPairSet(const IcpPairSet &pair_set, Eigen::Quaterniond delta_q, Eigen::Vector3d delta_t,
                               Eigen::Quaterniond &q, Eigen::Vector3d &t)
{
    for (auto pair : pair_set.pairs)
    {
        double weight = 1;
        ceres::CostFunction *cost_function = PointToPlaneError::Create(
            pair.frame_s->raw_pts[pair.index_s].P,
            pair.frame_q->raw_pts[pair.index_q].P,-
            pair.frame_s->raw_pts[pair.index_s].N,
            pair.type, delta_q, delta_t, weight);

        problem_.AddResidualBlock(cost_function, NULL, q.coeffs().data(), t.data());
    }
#ifdef RAW_ICP_DEBUG
    printf("added %lu pairs into ceres\n", pair_set.pairs.size());
#endif
}

void IcpProblem::AddParameterization(Eigen::Quaterniond &q)
{
    auto *quaternion_parameterization =
        new ceres::EigenQuaternionParameterization;
    problem_.SetParameterization(q.coeffs().data(), quaternion_parameterization);
}

void IcpProblem::Sovle()
{
#ifdef RAW_ICP_DEBUG
    printf("start solve problem ...\n");
#endif
    ceres::Solver::Options options;
    options.linear_solver_type = ceres::SPARSE_NORMAL_CHOLESKY;
    options.use_nonmonotonic_steps = false;
    options.max_num_iterations = 50;
    options.num_threads = 10;
    ceres::Solve(options, &problem_, &summary_);
}

}
