#include "voxel_filter.h"

#include <unordered_set>

#define distance(p) std::sqrt(p.x * p.x + p.y * p.y + p.z * p.z)

inline uint64_t get_index(const PointJ &point, float resolution)
{
    int x = std::lroundf(point.x / resolution); // 24 bit, 1cm voxel for 16KM
    int y = std::lroundf(point.y / resolution); // 24 bit
    int z = std::lroundf(point.z / resolution); // 16 bit, 1cm voxel for 600M
    return (((uint64_t)x & 0xffffff) << 40) | (((uint64_t)y & 0xffffff) << 16) | (z & 0xffff);
}

void vector_voxel_filter(const PointCloudJ::Ptr pcloud, VectorCloud &vcloud,
                         float resolution, float max_range) {
    vcloud.clear();
    vcloud.reserve(pcloud->size() / 5);

    std::unordered_set<uint64_t> voxel_set;
    voxel_set.reserve(pcloud->size() * 5);
    for (const PointJ &point : pcloud->points) {
        if (max_range > 0 && distance(point) > max_range)
            continue;
        uint64_t key = get_index(point, resolution);
        if (voxel_set.count(key) == 0) {
            voxel_set.emplace(key);
            Eigen::Vector3f p(point.x, point.y, point.z);
            vcloud.push_back(p);
        }
    }
}

void clouds_voxel_filter(const PointCloudJ::Ptr pcloud, PointCloudJ::Ptr &clouds,
                         float resolution, float max_range) {
    clouds->points.clear();
    clouds->points.reserve(pcloud->size() / 5);

    std::unordered_set<uint64_t> voxel_set;
    voxel_set.reserve(pcloud->size() * 5);
    for (const PointJ &point : pcloud->points) {
        if (max_range > 0 && distance(point) > max_range)
            continue;
        uint64_t key = get_index(point, resolution);
        if (voxel_set.count(key) == 0) {
            voxel_set.emplace(key);
            clouds->points.push_back(point);
        }
    }
}