#ifndef BLOCKAREA_H
#define BLOCKAREA_H

#include "manager/squareareamanager.h"
#include "block/block_info.pb.h"
#include "distribution/distribution.pb.h"
#include "../../interface/voxel_map_define.h"

namespace localize {

class BlockArea : public SquareAreaManager
{
    typedef ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >
    ProtoRepeatedUint32;
public:
    BlockArea(const string& meshPath,
              const boost::shared_ptr<Distributions> &distributions,
              uint32_t blockInfoFileOffset,
              uint32_t blockFileSize,
              uint32_t gridCntPerBlockSide);

    void backendFunc(uint8_t type, Vector2d point = Vector2d::Zero());

    bool isLoad(){
        return loaded_;
    }

    void load();

    inline juefx::Distribution* getGridDistribution(
            const uint8_t &layerId, const uint8_t &shiftedX,
            const uint8_t &shiftedY, uint8_t &length) const
    {
        juefx::Distribution * ret;
        juefx::DistributionIndex index = indexTable_[layerId][shiftedX][shiftedY];
        length = index.length;
        if(length == 0){
            return nullptr;
        }
        ret = const_cast<juefx::Distribution*>(
                    &(distributionVec_.data()[index.start]));
        return ret;
    }

    inline juefx::Distribution* GetCellValue(
            uint8_t type, const juefx::BlockGridIndex &index, uint8_t &length) const{
        uint32_t gridIndexX = index.x.gridId;
        uint32_t gridIndexY = index.y.gridId;
        return getGridDistribution(type, gridIndexX, gridIndexY, length);
    }

    inline bool calRPByY(const double y,
                         double& r,
                         double& p){
        if(!plainParamValid_){
            return false;
        }
        double C = fabs(plainParams_[2]) < 1e-3 ?
                    1e-3 : plainParams_[2];
        Vector3d xVec(1, 0, 0);
        Vector3d frontVec = AngleAxisd(y, Vector3d::UnitZ()) * xVec;
        double pZ = -(plainParams_[0] * frontVec.x() +
                plainParams_[1] * frontVec.y()) / C;
        p = atan2(pZ, 1);
        Vector3d leftVec = AngleAxisd(M_PI / 2, Vector3d::UnitZ()) * frontVec;        
        double rZ = -(plainParams_[0] * leftVec.x() +
                plainParams_[1] * leftVec.y()) / C;
        r = atan2(rZ, 1);

        LOG(INFO) << " A: " << plainParams_[0]
                  << " B: " << plainParams_[1]
                  << " C: " << plainParams_[2]
//                  << " y: " << y
//                  << " frontVec.x(): " << frontVec.x()
//                  << " frontVec.y(): " << frontVec.y()
//                  << " leftVec.x(): " << leftVec.x()
//                  << " leftVec.y(): " << leftVec.y()
                  << " pZ: " << pZ
                  << " rZ: " << rZ;
        return true;
    }
private:
    void parseBitmapBlock(const BlockInfo &blockInfo);

    void parseIndexBlock(const BlockInfo &blockInfo);

    void parseFromShiftedAndFeatureVec(
            const vector<pair<uint32_t, uint32_t>> &shiftedAndFeatureVec);

    void gatherGridData(const juefx::GridData &gridData,
                        const Vector2i &index,
                        uint8_t layerId = 0);

    vector<uint32_t> parseBitMap(
            const ProtoRepeatedUint32& bitmap,
            const uint32_t size);

    void configMultiLayer(uint8_t layerCnt = 5);

    bool mergeGridData(
            juefx::GridData &preGridData,
            juefx::GridData &merged);

private:
    string meshPath_;

    uint32_t blockInfoFileOffset_;

    uint32_t blockFileSize_;

    bool loaded_ = false;

    uint32_t gridCntPerBlockSide_;

    vector<juefx::Distribution> distributionVec_;

    vector<vector<juefx::DistributionIndex>> indexTable_[5];

    boost::shared_ptr<Distributions> distributions_;

    unordered_map<int64_t, juefx::DistributionIndex> hashMap_;

    vector<double> plainParams_;

    bool plainParamValid_ = false;
};

}
#endif // BLOCKAREA_H
