#include "maparea.h"
using namespace boost::filesystem;

namespace localize {

MapArea::MapArea()
{
//    commandQueue_.reset(new SynchronizedQueue<Command>);
//    backendTrd_.reset(new boost::thread(boost::bind(
//                                            &MapArea::backendThreadFunc, this)));

    std::ifstream ifs("/home/juefx/map_fuma/cartographer.bag.calib");
    if(ifs) {
        string line;
        double r, p, y;
        getline(ifs, line);
        {
            std::istringstream iss(line);
            string subline;
            getline(iss, subline, ',');
            std::istringstream(subline) >> r;
            getline(iss, subline, ',');
            std::istringstream(subline) >> p;
            getline(iss, subline, ',');
            std::istringstream(subline) >> y;
        }
        ifs.close();
        imu2base_.linear() =
                Matrix3d(AngleAxisd(y, Vector3d::UnitZ())
                * AngleAxisd(p, Vector3d::UnitY())
                * AngleAxisd(r, Vector3d::UnitX()));
    }
}

MapArea::~MapArea()
{
//    commandQueue_->stopQueue();
//    backendTrd_->join();
//    backendTrd_.reset();
//    commandQueue_.reset();
}

bool MapArea::SetMapPath(string path_name)
{
    mapPath_ = path_name;
    init();
    return true;
}

bool MapArea::LoadMap()
{
    if(!currMesh_){
        return 0;
    }
    currMesh_->load();
    return true;
}

float MapArea::getResolution()
{
    if(!currMesh_){
        return 0;
    }
    return currMesh_->getGridResolution();
}

void MapArea::backendThreadFunc()
{
    Command command;
    while(commandQueue_->dequeue(command)){
        TIMER_START(start)
        command.manager->backendFunc(command.type, command.point);
        TIMER_END(start, seconds);
        LOG(INFO) << "command " << uint16_t(command.type)
                  << " take " << seconds << " seconds";
    }
}

void MapArea::backendFunc(uint8_t type, Vector2d point)
{
    if(type == Init){
        init();
    }
}

void MapArea::Enqueue(
        const MapArea::Command &command)
{
    commandQueue_->enqueue(command);
}

void MapArea::init()
{
    TIMER_START(start)
    map<ulong, MeshInfo> meshInfoMap = getMeshInfoMap();
    if(meshInfoMap.size() == 0){
        return;
    }
    auto fstMeshInfoIter = meshInfoMap.begin();
    ulong meshId = fstMeshInfoIter->first;
    vector<ulong> checkedMesh = {meshId};
    checkMeshIndexDiff(meshId, meshInfoMap,
                       checkedMesh, {0, 0});

    reSetupMeshIndex(meshInfoMap);

    constructMesh(meshInfoMap);

    connectNeighbors();

    currMesh_ = meshAreaTable_[0][0];
    currMesh_->init();

    TIMER_END(start, seconds)
    LOG(INFO) << "init take " << seconds << " seconds";
}

map<ulong, MapArea::MeshInfo> MapArea::getMeshInfoMap()
{
//    directory_iterator mesh_iter(mapPath_);
//    directory_iterator mesh_end_iter;
//    map<ulong, MeshInfo> meshInfoMap;
//    for(; mesh_iter != mesh_end_iter; mesh_iter++) {
//        if(mesh_iter->path().extension().string() != ".stream"){
//            continue;
//        }
//        string mesh_path = mesh_iter->path().string();
//        string mesh_name = mesh_iter->path().stem().string();
//        ulong meshId = 000;
//        try {
//            meshId = stoi(mesh_name);
//        }
//        catch (std::invalid_argument&){
//            LOG(INFO) << "header.mesh_id() is not int";
//        }
//        MeshInfo meshInfo;
//        meshInfo.meshId = meshId;
//        meshInfo.meshPath = mesh_path;
//        meshInfoMap.insert(make_pair(meshId, meshInfo));
//    }

    map<ulong, MeshInfo> meshInfoMap;
    MeshInfo meshInfo;
    meshInfo.meshId = stoi(path(mapPath_).stem().string());
    meshInfo.meshPath = mapPath_;
    meshInfoMap.insert(make_pair(meshInfo.meshId, meshInfo));
    return meshInfoMap;
}

void MapArea::checkMeshIndexDiff(const ulong currMeshId,
                                 map<ulong, MeshInfo> &meshInfoMap,
                                 vector<ulong> &checkedMesh,
                                 Vector2i index){
    for(NeighborType neighborType : neighborTypeVec){
        ulong newMeshId = GetNeighborGeoHash(currMeshId, neighborType);
        Vector2i newIndex = index + neighborIndexDiff[size_t(neighborType)];
        auto it = find(checkedMesh.begin(), checkedMesh.end(), newMeshId);
        if(it != checkedMesh.end()){
            continue;
        }
        auto iter = meshInfoMap.find(newMeshId);
        if(iter != meshInfoMap.end()){
            MeshInfo &newMeshInfo = iter->second;
            newMeshInfo.meshIndex = newIndex;
            checkedMesh.push_back(newMeshId);
            checkMeshIndexDiff(newMeshId, meshInfoMap,
                               checkedMesh, newIndex);
        }
    }
}

void MapArea::reSetupMeshIndex(
        map<ulong, MapArea::MeshInfo> &meshInfoMap)
{
    auto meshInfoMapIter = meshInfoMap.begin();
    int32_t maxest = numeric_limits<int32_t>::max();
    int32_t lowest = numeric_limits<int32_t>::lowest();
    Vector2i minimum = {maxest, maxest};
    Vector2i maximum = {lowest, lowest};
    for(; meshInfoMapIter != meshInfoMap.end(); meshInfoMapIter++){
        const Vector2i &index = meshInfoMapIter->second.meshIndex;
        minimum.x() = min(minimum.x(), index.x());
        minimum.y() = min(minimum.y(), index.y());
        maximum.x() = max(maximum.x(), index.x());
        maximum.y() = max(maximum.y(), index.y());
    }

    meshInfoMapIter = meshInfoMap.begin();
    for(; meshInfoMapIter != meshInfoMap.end(); meshInfoMapIter++){
        Vector2i &index = meshInfoMapIter->second.meshIndex;
        index -= minimum;
    }
    maximum -= minimum;
    setItemCnt(maximum.x() + 1, maximum.y() + 1);
}

void MapArea::constructMesh(
        const map<ulong, MapArea::MeshInfo> &meshInfoMap)
{
    meshAreaTable_.resize(rowItemCnt_);
    for(auto& col : meshAreaTable_){
        col.resize(colItemCnt_);
    }
    auto meshInfoMapIter = meshInfoMap.begin();
    for(; meshInfoMapIter != meshInfoMap.end(); meshInfoMapIter++){
        const MeshInfo &meshInfo = meshInfoMapIter->second;
        auto &currMesh = meshAreaTable_[meshInfo.meshIndex.x()][meshInfo.meshIndex.y()];
        currMesh.reset(new MeshArea(meshInfo.meshPath));
        currMesh->setMeshId(meshInfo.meshId);
        Vector2i meshIndex;
        meshIndex.x() = (uint16_t)meshInfo.meshIndex.x();
        meshIndex.y() = (uint16_t)meshInfo.meshIndex.y();
        meshIndexMap_.insert(make_pair(meshInfo.meshId, meshIndex));
    }
}

void MapArea::connectNeighbors()
{
    for(size_t x = 0; x < rowItemCnt_; x++){
        for(size_t y = 0; y < colItemCnt_; y++){
            auto currMesh = meshAreaTable_[x][y];
            if(!currMesh){
                continue;
            }
            Vector2i index{x, y};
            for(NeighborType neighborType : neighborTypeVec){
                Vector2i newIndex = index + neighborIndexDiff[size_t(neighborType)];
                if(newIndex.x() < 0 || newIndex.x() >= rowItemCnt_ ||
                        newIndex.y() < 0 || newIndex.y() >= colItemCnt_){
                    continue;
                }
                MeshArea::MeshContent meshContent;
                meshContent.neighborMesh = meshAreaTable_[newIndex.x()][newIndex.y()];
                if(!meshContent.neighborMesh){
                    continue;
                }
                currMesh->insertNeighborMesh(meshContent.neighborMesh->getMeshId(), meshContent);
            }
        }
    }
}

}//end of namespace
