﻿#ifndef MAPAREA_H
#define MAPAREA_H

#include "manager/areamanager.h"
#include "utils/synchronizedqueue.h"
#include "utils/basic_functions.h"
#include "mesharea.h"
#include "../../interface/voxel_map_define.h"

//Vector2d oldCenter = GetMeshCenter(322718375);
//Vector2d newCenter(31.2755157846516, 121.654476884526);
//Vector3d translation;
//GeographicLib::LocalCartesian proj(oldCenter.y(), oldCenter.x(), 0);
//proj.Forward(newCenter.x(), newCenter.y(), 0,
//             translation.x(), translation.y(), translation.z());
//LOG(INFO) << "translation: " << translation.transpose();
//exit(0);
//Eigen::Vector3d translation(-511.776, 20.6567, -0.0205473);
//PointCloudJ pc;
//for (auto p : scan->points)
//{
//    p.x -= translation.x();
//    p.y -= translation.y();
//    pc.push_back(p);
//}
//pcl::io::savePCDFileBinary("/home/juefx/localize_data/test_data/transformed.pcd", pc);
//exit(0);

using namespace juefx;

namespace localize {

class MapArea : public AreaManager
{
    struct Command{
        AreaManager* manager;
        uint8_t type;
        Vector2d point = Vector2d::Zero();
    };
    struct MeshInfo{
        ulong meshId = 0;
        string meshPath;
        Vector2i meshIndex = {0, 0};
    };

    enum MapBackEndType{
        Init = 0,
    };

public:
    MapArea();

    ~MapArea();

    bool SetMapPath(std::string path_name);

    bool LoadMap();

    inline void loadArea(const Vector3d &pose){
        currMesh_->loadArea(pose);
    }

    inline bool FindMeshByBLH(const GnssPoint &blh_pose,
                              Vector2i &meshIndex){
        ulong mesh = GetMeshFromPosition(blh_pose.L, blh_pose.B);
        if(mesh == INVALID_MESHID){
            return false;
        }
        auto iter = meshIndexMap_.find(mesh);
        if(iter == meshIndexMap_.end()){
            return false;
        }
        meshIndex = iter->second;
        return true;
    }

    inline bool IsMapLoaded(const GnssPoint &blh_pose){

        bool areaLoaded = currMesh_->IsAreaLoaded(blh_pose);
        if(!areaLoaded){
            Command command;
            command.manager = currMesh_.get();
            command.type = MeshArea::LoadAread;
            command.point = {blh_pose.B, blh_pose.L};
            Enqueue(command);
        }
        return areaLoaded;
    }
    inline bool IsMapLoaded(const Vector3d& pose)
    {
        return currMesh_->IsAreaLoaded(pose);
    }

    template <typename T>
    Eigen::Matrix<T, 3, 1> RotationQuaternionToEulerVector(
        const Eigen::Quaternion<T>& quaternion) {
      Eigen::Quaternion<T> n_q = quaternion.normalized();
      // We choose the quaternion with positive 'w', i.e., the one with a smaller
      // angle that represents this orientation.
      if (n_q.w() < 0.) {
        // Multiply by -1. http://eigen.tuxfamily.org/bz/show_bug.cgi?id=560
        n_q.w() = -1. * n_q.w();
        n_q.x() = -1. * n_q.x();
        n_q.y() = -1. * n_q.y();
        n_q.z() = -1. * n_q.z();
      }
      // We convert the normalized_quaternion(n_q) into a vector along the rotation
      // axis with length of the rotation angle.
      const T roll = atan2(2 * (n_q.w() * n_q.x() + n_q.y() * n_q.z()),
                           1 - 2 * (n_q.x() * n_q.x() + n_q.y() * n_q.y()));
      const T pitch = asin(2 * (n_q.w() * n_q.y() - n_q.z() * n_q.x()));
      const T yaw = atan2(2 * (n_q.w() * n_q.z() + n_q.x() * n_q.y()),
                          1 - 2 * (n_q.y() * n_q.y() + n_q.z() * n_q.z()));
      return Eigen::Matrix<T, 3, 1>(roll, pitch, yaw);
    }

    inline bool GuessPosition(
            const GnssPoint &init_pose, Eigen::Isometry3d &pose_in_map){
        currMesh_ = meshAreaTable_[0][0];
        if(!currMesh_){
            return false;
        }
        gridResolution_ = round(currMesh_->getGridResolution() * 100);

        currMesh_->Forward(
                    init_pose.B, init_pose.L, init_pose.H,
                    pose_in_map.translation().x(),
                    pose_in_map.translation().y(),
                    pose_in_map.translation().z());
        const auto& block = currMesh_->getBlockByPoint(
                    pose_in_map.translation().x(),
                    pose_in_map.translation().y());
        auto rpy =  init_pose.rpy;
//        LOG(INFO) << " br: " << rpy[0]
//                  << " bp: " << rpy[1]
//                  << " by: " << rpy[2];
        if(block && block->isLoad()){
            if(block->calRPByY(rpy.z(), rpy.x(), rpy.y())){
//                LOG(INFO) << " ar: " << rpy[0]
//                          << " ap: " << rpy[1]
//                          << " ay: " << rpy[2];
            }
        }
        auto base2global = Matrix3d(AngleAxisd(rpy.z(), Vector3d::UnitZ())
                * AngleAxisd(rpy.y(), Vector3d::UnitY())
                * AngleAxisd(rpy.x(), Vector3d::UnitX()));
        auto imu2global =  base2global * imu2base_.linear();
        auto imuRotRPY = RotationQuaternionToEulerVector(Quaterniond(imu2global));
//        LOG(INFO) << " rr: " << imuRotRPY[0]
//                  << " rp: " << imuRotRPY[1]
//                  << " ry: " << imuRotRPY[2];
        double simpleRadio = 1;
        rpy = init_pose.rpy * simpleRadio +
                imuRotRPY * (1 - simpleRadio);
//        LOG(INFO) << " fr: " << rpy[0]
//                  << " fp: " << rpy[1]
//                  << " fy: " << rpy[2];
        pose_in_map.linear() = Matrix3d(AngleAxisd(rpy.z(), Vector3d::UnitZ())
                * AngleAxisd(rpy.y(), Vector3d::UnitY())
                * AngleAxisd(rpy.x(), Vector3d::UnitX()));


//        auto pose1 = RotationQuaternionToEulerVector(Quaterniond(pose_in_map.linear()));
//        LOG(INFO) << " pose1: " << pose1.transpose();
//        auto imu2base = RotationQuaternionToEulerVector(Quaterniond(imu2base_.linear()));
//        LOG(INFO) << " imu2base: " << imu2base.transpose();
//        pose_in_map.linear() = pose_in_map.linear() * imu2base_.linear();
//        auto pose2 = RotationQuaternionToEulerVector(Quaterniond(pose_in_map.linear()));
//        LOG(INFO) << " pose2: " << pose2.transpose();
        return true;
    }

    inline void ConfirmPosition(const Eigen::Vector3d &pose_in_map, GnssPoint &final_pose)
    {
        if(!currMesh_){
            return;
        }
        currMesh_->Reverse(
                    pose_in_map.x(), pose_in_map.y(), pose_in_map.z(),
                    final_pose.B, final_pose.L, final_pose.H);
    }

    inline juefx::BlockGridIndex GetCellIndex(
            float x, float y) const{
        juefx::BlockGridIndex ret;
        if(!currMesh_){
            return ret;
        }
        return currMesh_->GetCellIndex(x, y);
    }

    inline Distribution* GetCellValue(
            uint8_t type, const BlockGridIndex &index, uint8_t &length) const{
        if(!currMesh_){
            return nullptr;
        }
        auto ret = currMesh_->GetCellValue(type, index, length);

//        std::ofstream ofs;
//        ofs.open("/home/juefx/grid_use_" + to_string(type) + ".txt", std::ios::app);
//        int32_t blockIndexX = index.x.blockId;
//        int32_t blockIndexY = index.y.blockId;
//        uint32_t gridIndexX = index.x.gridId;
//        uint32_t gridIndexY = index.y.gridId;
//        Vector2f gridCenter;
//        gridCenter.x() = blockIndexX * 23.04 + (0.5 + gridIndexX) * 0.09;
//        gridCenter.y() = blockIndexY * 23.04 + (0.5 + gridIndexY) * 0.09;
////        if(fabs(gridCenter.x() - (-0.045)) < 0.01
////                && fabs(gridCenter.y() - 9.315) < 0.01){
////            LOG(INFO) << "blockIndexX: " << blockIndexX << " blockIndexY: " << blockIndexY
////                      << " gridIndexX: " << gridIndexX << " gridIndexY: " << gridIndexY
////                      << " type: " << (uint16_t)type << " length: " << (uint16_t)length;
////        }

//        if(ret == nullptr){
////            std::ofstream nullOfs;
////            nullOfs.open("/home/juefx/null_" + to_string(type) + ".txt", std::ios::app);
////            nullOfs << gridCenter.x() << "," << gridCenter.y() << "," << 0 << std::endl;
////            nullOfs.close();
//            return nullptr;
//        }
//        for(int i = 0; i < length; i++){
//            for(int32_t j = 0;
//                j < (ret + i)->height / 3;
//                j++){
//                int32_t indexZ = j + (ret + i)->floor / 3;
//                float point[4];
//                point[0] = gridCenter.x();
//                point[1] = gridCenter.y();
//                point[2] = ((float)indexZ + 0.5) * 3 / 100;
//                point[3] = (ret + i)->intensity;
//                ofs << point[0] << "," << point[1] << ","
//                                << point[2] << "," << point[3] << std::endl;
//            }
//        }
//        ofs.close();

        return ret;
    }

    inline Eigen::Vector2f GetCenterOfCell(
            uint8_t type, float x, float y) const
    {
        Vector2f ret = {-9999, -9999};
        if(!currMesh_){
            return ret;
        }
        x -= currMesh_->GetOffset().x();
        y -= currMesh_->GetOffset().y();
        float gridResolution = currMesh_->getGridResolution();
//        uint32_t typeGridResolution = round(gridResolution * 100) * (1 << type);

        Vector2i gridIndex;
        gridIndex.x() = floor(x * 100 / lround(gridResolution * 100));
        gridIndex.y() = floor(y * 100 / lround(gridResolution * 100));

        ret.x() = (0.5 + gridIndex.x()) * gridResolution;
        ret.y() = (0.5 + gridIndex.y()) * gridResolution;
        return ret;
    }

    float getResolution();

private:
    void backendThreadFunc();

    void backendFunc(uint8_t type, Vector2d point = Vector2d::Zero()) override;

    void Enqueue(const Command& command);

    void init();

    map<ulong, MeshInfo> getMeshInfoMap();

    void checkMeshIndexDiff(
            const ulong currMeshId,
            map<ulong, MeshInfo> &meshInfoMap,
            vector<ulong> &checkedMesh,
            Vector2i index);

    void reSetupMeshIndex(map<ulong, MeshInfo> &meshInfoMap);

    void constructMesh(const map<ulong, MeshInfo> &meshInfoMap);

    void connectNeighbors();
private:
    string mapPath_;

    boost::shared_ptr<SynchronizedQueue<Command>> commandQueue_;

    ThreadPtr backendTrd_;

    map<ulong, Vector2i> meshIndexMap_;

    vector<vector<boost::shared_ptr<MeshArea>>> meshAreaTable_;

    boost::shared_ptr<MeshArea> currMesh_;

    uint32_t gridResolution_;

    Isometry3d imu2base_ = Isometry3d::Identity();
};

}
#endif // MAPAREA_H
