#ifndef DISPATCH_H
#define DISPATCH_H

#include "utils/basic_types.h"
#include "utils/basic_functions.h"
#include <boost/filesystem.hpp>
#include <pcl/io/pcd_io.h>
#include "utils/pcl_point_type.h"
#include "grid/gridmanager.h"
#include <GeographicLib/LocalCartesian.hpp>
using namespace GeographicLib;

class Dispatch
{
public:
    Dispatch(const string &path);

    Dispatch(PointCloudExport::Ptr pointcloud);

    bool loadPCD(const string &path);

    void setPointcloud(PointCloudExport::Ptr &pointcloud);

    boost::shared_ptr<GridLayer> getMeshLayer();

    void dispatch();

    Vector3d getOffset();

    void setMeshId(ulong meshId);

    ulong getMeshId();

    void setBLH(const Vector3d& blh){
        blh_ = blh;
    }
    void configOffset();

private:
    void dispatchPoint(const PointExport &point);

    void configBlockInfo(
            boost::shared_ptr<LocateGridBase> blockGrid);

private:
    PointCloudExport::Ptr pointcloud_;

    boost::shared_ptr<GridLayer> meshLayer_;

    Vector3d offset_ = Vector3d::Zero();

    ulong meshId_;

    Vector3d blh_;
};

#endif // DISPATCH_H
