#include <iostream>
#include <string>
#include <pcl/common/common_headers.h>
#include <pcl/io/pcd_io.h>
#include <pcl/console/parse.h>
#include <pcl/point_types.h>
#include <algorithm>
#include "featureExtraction.h"
using namespace std;

//#define LABEL_ROAD 20000.0f
//#define LABEL_GROUND 40000.0f
//#define LABEL_NONGROUND 10000.0f
#define LABEL_ROAD -1.0f
#define LABEL_GROUND -2.0f
#define LABEL_NONGROUND 0.0f
#define LABEL_NOISE 60000.0f
#define NUM_DISTRIBUTION 3
#define MIN_INTERVAL 15
#define NUM_INTERVAL 2



bool featureExtraction::init(PointCloudExport::Ptr cloud) {
	if (cloud == nullptr || cloud->points.size() == 0) {
		cout << "cloud or ptr of cloud is empty" << endl;
		return false;
	}

	float f_max = -9999.9, f_min = 9999.9;
	for (auto point : cloud->points) {
		f_max = max(f_max, point.z);
		f_min = min(f_min, point.z);
	}

	maxRange = ceil(f_max * 100 / (float)resolution) * resolution;
	minRange = floor(f_min * 100 / (float)resolution) * resolution;

	numOfIndex = (maxRange - minRange) / resolution;

	
	for (auto point : cloud->points) {
		
		int index = floor((point.z * 100 - (float)minRange) / (float)resolution);
		if (index == numOfIndex) {
			index--;
		}
		voxel vo(index, point.label, point.intensity);
        if (point.label == LABEL_GROUND || point.label == LABEL_ROAD){
            vec_voxel_ground.push_back(vo);

        }
        else{
            vec_voxel_nonground.push_back(vo);
        }
    }
	
    sort(vec_voxel_ground.begin(), vec_voxel_ground.end());
    sort(vec_voxel_nonground.begin(),vec_voxel_nonground.end());
    return true;
}

void featureExtraction::getFeature(vector<gridFeature>& result) {
    int ng = vec_voxel_ground.size();
    int nn = vec_voxel_nonground.size();
    if(ng==0&&nn==0){
        return;
    }
    vector<pair<int, int>> sections_ground;
    vector<pair<int, int>> sections_nonground;
    if(ng!=0){
        vector<pair<int, int>> intervals;

        for (int i = 1; i < ng; i++) {
            int interval = vec_voxel_ground[i].index - vec_voxel_ground[i - 1].index;
            if (interval < 50) {
                continue;
            }
            intervals.push_back(make_pair(interval, i-1));
        }
        int numOfInterval = intervals.size();
        int left = 0;
        int right = 0;
        for (int i = 0; i <= numOfInterval; i++) {
            if (i == 0) {
                left = 0;
            }
            else {
                left = right + 1;
            }
            if (i == numOfInterval) {
                right = ng - 1;
            }
            else {
                right = intervals[i].second;
            }
            sections_ground.push_back(make_pair(left, right));
        }
    }

    int numOfInterval = 0;
    if(sections_ground.size()==0){
        numOfInterval=2;
    }
    else{
        numOfInterval= 2*sections_ground.size();
    }
    if (nn!=0){
        vector<pair<int, int>> intervals; //<begin index,length> of interval
        for (int i = 1; i < nn; i++) {
            int interval = vec_voxel_nonground[i].index - vec_voxel_nonground[i - 1].index;
            if (interval < 10) {
                continue;
            }
            intervals.push_back(make_pair(interval, i-1));
        }

        if (intervals.size() <= numOfInterval ) {
            numOfInterval = intervals.size();
        }
        if(numOfInterval>0){
            sort(intervals.begin(), intervals.end(),
                 [](const pair<int, int>& lhs, const pair<int, int>& rhs)->bool {
                if (lhs.first > rhs.first) return true;
                else if (lhs.first < rhs.first) return false;
                else if (lhs.second < rhs.second) return true;
                else return false;
            });
            sort(intervals.begin(), intervals.begin() + numOfInterval,
                 [](const pair<int, int>& lhs, const pair<int, int>& rhs)->bool {
                if (lhs.second < rhs.second) return true;
                else return false;
            });
        }
        vector<pair<int, int>> sections;
        int left = 0;
        int right = 0;
        for (int i = 0; i <= numOfInterval; i++) {
            if (i == 0) {
                left = 0;
            }
            else {
                left = right + 1;
            }
            if (i == numOfInterval) {
                right = nn - 1;
            }
            else {
                right = intervals[i].second;
            }
            sections_nonground.push_back(make_pair(left, right));
        }
    }

//    for(int i=0;i<sections_nonground.size();i++){
//        for(int j=0;j<sections_ground.size();j++){
//            int temp=vec_voxel_nonground[sections_nonground[i].second].index-vec_voxel_ground[sections_ground[j].second].index;
//            if(temp<=2){
//                sections_nonground[i].second=sections_ground[j].first;
//                if(sections_nonground[i].second<sections_nonground[i].first+2){
//                    sections_nonground[i].first=-1;
//                }
//            }
//        }
//    }


    vector<int> vec_intensity_ground;
    for (int i = 0; i < sections_ground.size(); i++) {
        float sum_intensity = 0;
        for (int j = sections_ground[i].first; j <= sections_ground[i].second; j++) {
            sum_intensity += vec_voxel_ground[j].intensity;
        }
        vec_intensity_ground.push_back(round(sum_intensity / (sections_ground[i].second - sections_ground[i].first + 1)));
    }

    vector<int> vec_intensity_nonground;
    for (int i = 0; i < sections_nonground.size(); i++) {
        float sum_intensity = 0;
        if(sections_nonground[i].first==-1){
            vec_intensity_nonground.push_back(0);
            continue;
        }
        for (int j = sections_nonground[i].first; j <= sections_nonground[i].second; j++) {
            sum_intensity += vec_voxel_nonground[j].intensity;
        }
        vec_intensity_nonground.push_back(round(sum_intensity / (sections_nonground[i].second - sections_nonground[i].first + 1)));
    }

    for (int i = 0; i < sections_ground.size(); i++) {
        gridFeature gf;
        gf.minz_devidedByResolution = minRange / resolution + vec_voxel_ground[sections_ground[i].first].index;
        gf.deltaz_devidedByResolution = vec_voxel_ground[sections_ground[i].second].index - vec_voxel_ground[sections_ground[i].first].index + 1;
        gf.isGround = true;
        gf.intensity = vec_intensity_ground[i];
        if(gf.deltaz_devidedByResolution<1){
            cout<<"error"<<endl;
        }
        result.push_back(gf);
    }

    for (int i = 0; i < sections_nonground.size(); i++) {
        if(sections_nonground[i].first==-1){
            continue;
        }
        gridFeature gf;
        gf.minz_devidedByResolution = minRange / resolution + vec_voxel_nonground[sections_nonground[i].first].index;
        gf.deltaz_devidedByResolution = vec_voxel_nonground[sections_nonground[i].second].index - vec_voxel_nonground[sections_nonground[i].first].index + 1;

        int temp = 0;
        for(int j=0; j<sections_ground.size();j++){
            temp = gf.minz_devidedByResolution+gf.deltaz_devidedByResolution-(result[j].minz_devidedByResolution+1);
            if (temp<=2&&temp>0){
                gf.deltaz_devidedByResolution-=temp;
                break;
            }
        }
        if(temp<=2&&temp>0&&gf.deltaz_devidedByResolution<=2){
            continue;
        }


        if(gf.deltaz_devidedByResolution<1){
            cout<<"error"<<endl;
        }

        gf.isGround = false;
        gf.intensity = vec_intensity_nonground[i];
        result.push_back(gf);
    }

}


void featureExtraction::getFeature22(vector<gridFeature>& result) {
	int n = vec_voxel.size();
	if (n == 0) {
		return;
	}

	vector<pair<int, int>> intervals; //<begin index,length> of interval
	for (int i = 1; i < n; i++) {
		int interval = vec_voxel[i].index - vec_voxel[i - 1].index;
		if (interval < MIN_INTERVAL) {
			continue;
		}
		intervals.push_back(make_pair(interval, i-1));
	}

	int numOfInterval = 0;
	if (intervals.size() <= NUM_INTERVAL ) {
		numOfInterval = intervals.size();
	}
	else {
		numOfInterval = NUM_INTERVAL;
	}

	sort(intervals.begin(), intervals.end(),
		[](const pair<int, int>& lhs, const pair<int, int>& rhs)->bool {
			if (lhs.first > rhs.first) return true;
			else if (lhs.first < rhs.first) return false;
			else if (lhs.second < rhs.second) return true;
			else return false;
		});
	sort(intervals.begin(), intervals.begin() + numOfInterval -1,
		[](const pair<int, int>& lhs, const pair<int, int>& rhs)->bool {
			if (lhs.second < rhs.second) return true;
			else return false;
		});
	vector<pair<int, int>> sections;
	int left = 0;
	int right = 0;
	for (int i = 0; i <= numOfInterval; i++) {
		if (i == 0) {
			left = 0;
		}
		else {
			left = right + 1;
		}
		if (i == numOfInterval) {
			right = n - 1;
		}
		else {
			right = intervals[i].second;
		}
		sections.push_back(make_pair(left, right));
	}

	int index_ground = -1;
	int max_ground = 0;
	vector<int> vec_intensity;
	for (int i = 0; i < sections.size(); i++) {
		int count_ground = 0;
		float sum_intensity = 0;
		for (int j = sections[i].first; j <= sections[i].second; j++) {
			if (vec_voxel[j].label == LABEL_GROUND || vec_voxel[j].label == LABEL_ROAD) {
				count_ground++;
			}
			sum_intensity += vec_voxel[j].intensity;
		}
		vec_intensity.push_back(round(sum_intensity / (sections[i].second - sections[i].first + 1)));
		if (count_ground > max_ground) {
			max_ground = count_ground;
			index_ground = i;
		}
	}

	for (int i = 0; i < sections.size(); i++) {
		gridFeature gf;
		gf.minz_devidedByResolution = minRange / resolution + vec_voxel[sections[i].first].index;
		gf.deltaz_devidedByResolution = vec_voxel[sections[i].second].index - vec_voxel[sections[i].first].index + 1;
        if (i == index_ground) {
			gf.isGround = true;
		}
		else {
			gf.isGround = false;
		}
		gf.intensity = vec_intensity[i];


		//if (result.size() != 0 && gf.minz_devidedByResolution >= result[0].minz_devidedByResolution - 2
		//	&& nongroundFeature.minz_devidedByResolution + nongroundFeature.deltaz_devidedByResolution <= result[0].minz_devidedByResolution + result[0].deltaz_devidedByResolution + 2) {
		//	continue;
		//}

	
		result.push_back(gf);
	}

}

void featureExtraction::getFeature3(vector<gridFeature>& result) {
	if (vec_voxel.size() == 0) {
		return;
	}
	
	//ҵֲvector޳
	int count_ground = 0;
	float sum_intensity_ground = 0;
	int lowerBoundGround = 99999;
	int upperBoundGround = -99999;
	int erase_end_index = -1;
	vector<voxel>::iterator iter = vec_voxel.begin();
	vector<voxel>::iterator iter_erase = vec_voxel.begin();
	while (iter!= vec_voxel.end()) {
		if (iter->label == LABEL_GROUND || iter->label == LABEL_ROAD) {
			count_ground++;
			sum_intensity_ground += iter->intensity;
			if (count_ground == 1 && lowerBoundGround == 99999) {
				lowerBoundGround = iter->index;
			}
			upperBoundGround = iter->index;
			iter_erase = iter;
		}
		iter++;
	}
	
	if (count_ground) {
		vec_voxel.erase(vec_voxel.begin(), iter_erase+1);
		gridFeature gf_ground;
		gf_ground.minz_devidedByResolution = minRange / resolution + lowerBoundGround;
		gf_ground.deltaz_devidedByResolution = upperBoundGround - lowerBoundGround + 1;
		gf_ground.intensity = round(sum_intensity_ground / count_ground);
		gf_ground.isGround = true;
		result.push_back(gf_ground);
	}

	int n = vec_voxel.size();
	//ȡķǵֲĿ϶
	vector<pair<int, int>> intervals; //<begin index,length> of interval
	int begin_iter = 0;
	for (int i = 1; i < n; i++) {
		int interval = vec_voxel[i].index - vec_voxel[i - 1].index;
		if (count_ground&&vec_voxel[i - 1].index - upperBoundGround <= 3 && interval > 10) {
			begin_iter = i;
			continue;
		}
		if (interval < MIN_INTERVAL) {
			continue;
		}
		intervals.push_back(make_pair(interval, i - 1));
		
	}

	//жϵһǵֲǷֲΪһֲ
	bool combine = false;
	if (vec_voxel[begin_iter].index - upperBoundGround <= 4) {
		combine = true;
	}
	
	//ʹõĿ϶
	int numOfInterval = 0;
	if (count_ground) {
		if (combine) {
			if (intervals.size() >= NUM_INTERVAL) {
				numOfInterval = NUM_INTERVAL;
			}
			else {
				numOfInterval = intervals.size();
			}
		}
		else {
			if (intervals.size() >= NUM_INTERVAL - 1) {
				numOfInterval = NUM_INTERVAL-1;
			}
			else {
				numOfInterval = intervals.size();
			}
		}
	}
	else {
		if (intervals.size() >= NUM_INTERVAL) {
			numOfInterval = NUM_INTERVAL;
		}
		else {
			numOfInterval = intervals.size();
		}
	}

	sort(intervals.begin(), intervals.end(),
		[](const pair<int, int>& lhs, const pair<int, int>& rhs)->bool {
			if (lhs.first > rhs.first) return true;
			else if (lhs.first < rhs.first) return false;
			else if (lhs.second < rhs.second) return true;
			else return false;
		});
	sort(intervals.begin(), intervals.begin() + numOfInterval,
		[](const pair<int, int>& lhs, const pair<int, int>& rhs)->bool {
			if (lhs.second < rhs.second) return true;
			else return false;
		});
	
	vector<pair<int, int>> sections;
	int left = begin_iter;
	int right = begin_iter;
	for (int i = 0; i <= numOfInterval; i++) {
		if (i == 0) {
			left = begin_iter;
		}
		else {
			left = right + 1;
		}
		if (i == numOfInterval) {
			right = n - 1;
		}
		else {
			right = intervals[i].second;
		}
		sections.push_back(make_pair(left, right));
	}

	//int index_ground = -1;
	//int max_ground = 0;
	//vector<int> vec_intensity;
	//for (int i = 0; i < sections.size(); i++) {
	//	int count_ground = 0;
	//	float sum_intensity = 0;
	//	for (int j = sections[i].first; j <= sections[i].second; j++) {
	//		if (vec_voxel[j].label == LABEL_GROUND || vec_voxel[j].label == LABEL_ROAD) {
	//			count_ground++;
	//		}
	//		sum_intensity += vec_voxel[j].intensity;
	//	}
	//	vec_intensity.push_back(round(sum_intensity / (sections[i].second - sections[i].first + 1)));
	//	if (count_ground > max_ground) {
	//		max_ground = count_ground;
	//		index_ground = i;
	//	}
	//}

	for (int i = 0; i < sections.size(); i++) {
		gridFeature gf;
		if (combine&&i==0) {
			result[0].deltaz_devidedByResolution = vec_voxel[sections[0].second].index - lowerBoundGround + 1;
			for (int j = sections[0].first; j <= sections[0].second; j++) {
				sum_intensity_ground += vec_voxel[j].intensity;
				count_ground++;
			}
			result[0].intensity = round(sum_intensity_ground / count_ground);
			continue;
		}
		gf.minz_devidedByResolution = minRange / resolution + vec_voxel[sections[i].first].index;
		gf.deltaz_devidedByResolution = vec_voxel[sections[i].second].index - vec_voxel[sections[i].first].index + 1;
		gf.isGround = false;
		float sum_intensity = 0;
		int count=0;
		for (int j = sections[0].first; j <= sections[0].second; j++) {
			sum_intensity += vec_voxel[j].intensity;
			count++;
		}
		gf.intensity = round(sum_intensity / count);


		//if (result.size() != 0 && gf.minz_devidedByResolution >= result[0].minz_devidedByResolution - 2
		//	&& nongroundFeature.minz_devidedByResolution + nongroundFeature.deltaz_devidedByResolution <= result[0].minz_devidedByResolution + result[0].deltaz_devidedByResolution + 2) {
		//	continue;
		//}


		result.push_back(gf);
	}

}

bool featureExtraction::init2(pcl::PointCloud<pcl::PointExport>::Ptr cloud) {
	if (cloud == nullptr || cloud->points.size() == 0) {
		cout << "cloud or ptr of cloud is empty" << endl;
		return false;
	}
	maxRangeGround = 0;
	minRangeGround = 0;

	maxRangeNonground = 0;
	minRangeNonground = 0;

	numOfIndexNonground = 0;
	numOfIndexGround = 0;

	vec_groundIndex.clear();
	vec_nongroundIndex.clear();

	pcl::PointCloud<pcl::PointExport>::Ptr groundCloud(new pcl::PointCloud<pcl::PointExport>);
	pcl::PointCloud<pcl::PointExport>::Ptr nongroundCloud(new pcl::PointCloud<pcl::PointExport>);

	this->groundCloud = groundCloud;
	this->nongroundCloud = nongroundCloud;


	for (auto point : cloud->points) {
		if (point.label == LABEL_ROAD || point.label== LABEL_GROUND) {
			groundCloud->points.push_back(point);
		}
		else if (point.label == LABEL_NONGROUND) {
			nongroundCloud->points.push_back(point);
		}
		else if (point.label == LABEL_NOISE) {
			continue;
		}
		else {
			nongroundCloud->points.push_back(point);
		}
	}
	setRangeAndIndex();

	toVolumePixelAndSort();
	
	return true;
}

void featureExtraction::getFeature2(vector<gridFeature>& result) {
	if (groundCloud->points.size() != 0) {
		gridFeature groundFeature;
		groundFeature.minz_devidedByResolution = minRangeGround / resolutionGround;
		groundFeature.deltaz_devidedByResolution = numOfIndexGround;
		groundFeature.isGround = true;
		int sum = 0;
		for (auto point : groundCloud->points) {
			sum += point.intensity;
		}
		groundFeature.intensity = round((float)sum / (float)groundCloud->points.size());
		result.push_back(groundFeature);
	}
	getNongroundFeature(result);
}

void featureExtraction::getNongroundFeature(vector<gridFeature>& result) {
	int n = vec_nongroundIndex.size();
	if (n == 0) {
		return;
	}
	int upperBound = vec_nongroundIndex[n-1];
	int lowerBound = 0;

	vector<pair<int, int>> intervals;
	int min_interval = 0;
	if (result.size() > 0) {
		min_interval = 5;
	}
	else {
		min_interval = 15;
	}
	for (int i = 1; i < n;i++) {
		int interval = vec_nongroundIndex[i] - vec_nongroundIndex[i - 1];
		if (interval < min_interval) {
			continue;
		}
		intervals.push_back(make_pair(interval, i - 1));
	}	
	/////////Ʒֲ
	int numOfInterval = 0;


	if (result.size() > 0) {
		if (intervals.size() <= numOfDistribution - 2) {
			numOfInterval = intervals.size();
		}
		else {
			numOfInterval = numOfDistribution - 2;
		}
	}
	else {
		if (intervals.size() <= numOfDistribution - 1) {
			numOfInterval = intervals.size();
		}
		else {
			numOfInterval = numOfDistribution - 1;
		}
	}

	sort(intervals.begin(), intervals.end(),
		[](const pair<int, int>& lhs, const pair<int, int>& rhs)->bool {
			if (lhs.first > rhs.first) return true;
			else if (lhs.first < rhs.first) return false;
			else if (lhs.second < rhs.second) return true;
			else return false;
		});
	sort(intervals.begin(), intervals.begin()+ numOfInterval,
		[](const pair<int, int>& lhs, const pair<int, int>& rhs)->bool {
			if (lhs.second < rhs.second) return true;
			else return false;
		});

	vector<int> lowerBounds;
	vector<int> upperBounds;
	int left =0;
	int right = left;

	for (int i = 0; i <= numOfInterval; i++) {
		if (i == 0) {
			left = 0;
		}
		else {
			left = right + 1;
		}
		if (i == numOfInterval) {
			right = n - 1;
		}
		else {
			right = intervals[i].second;
		}
		int leftAfterFilter = left;
		int rightAfterFilter = right;

		int upperbound = vec_nongroundIndex[right];
		int lowerbound = vec_nongroundIndex[left];
		upperBounds.push_back(upperbound);
		lowerBounds.push_back(lowerbound);
	}


//#ifdef OPTION_FILTER
//	for (int i = 0; i < numOfIndexNonground; i++) {
//		for (int j = i; j < numOfIndexNonground; j++) {
//			double score = 0;
//			for (int k = i; k <= j; k++) {
//				if (binarySearch(k)) {
//					score += scoreHitHit;
//				}
//				else {
//					score += scoreHitMiss;
//				}
//			}
//			for (int k = 0; k < n; k++) {
//				if (vec_unlabeledPointIndex[k] > i || vec_unlabeledPointIndex[k] < j) {
//					score += scoreMissHit;
//				}
//			}
//			if (score >= maxScore) {
//				maxScore = score;
//				upperBound = j;
//				lowerBound = i;
//			}
//		}
//	}
//#endif
//	vector<int> lowerBounds;
//	vector<int> upperBounds;
//#ifdef OPTION_MULTI_DISTRIBUTION
//
//
//	int left = lowerBound;
//	int right = lowerBound;
//	int blank = 0;
//	int interval_size = 5;
//	if (result.size() == 0) {
//		interval_size == 17;
//	}
//
//	for (int i = lowerBound; i <= upperBound; i++) {
//
//		if (binarySearch(i, vec_nongroundIndex)) {
//
//			// INTERVAL_SIZE*resolutionNonground
//			if (blank >= interval_size) {
//				lowerBounds.push_back(left);
//				upperBounds.push_back(right);
//				left = i;
//				right = left;
//			}
//			else {
//				right = i;
//				if (i == upperBound) {
//					lowerBounds.push_back(left);
//					upperBounds.push_back(right);
//				}
//			}
//			blank = 0;
//		}
//		else {
//			blank++;
//		}
//	}
//#else
//	lowerBounds.push_back(lowerBound);
//	upperBounds.push_back(upperBound);
//#endif

	for (int i = 0; i < upperBounds.size(); i++) {
		gridFeature nongroundFeature;
		nongroundFeature.minz_devidedByResolution = minRangeNonground / resolutionNonground + lowerBounds[i];
		nongroundFeature.deltaz_devidedByResolution = upperBounds[i] - lowerBounds[i] + 1;
		nongroundFeature.isGround = false;

		if (result.size() != 0 && nongroundFeature.minz_devidedByResolution >= result[0].minz_devidedByResolution - 2
			&& nongroundFeature.minz_devidedByResolution + nongroundFeature.deltaz_devidedByResolution <= result[0].minz_devidedByResolution + result[0].deltaz_devidedByResolution + 2) {
			continue;
		}

		double sum = 0;
		int count = 0;
		float l = (minRangeNonground + lowerBounds[i] * resolutionNonground)/100.0;
		float r = minRangeNonground + (upperBounds[i] + 1) * resolutionNonground/100.0;

		for (auto point : nongroundCloud->points) {
			
			if (point.z >= l && point.z < r) {
				sum += point.intensity;
				count++;
			}
			
		}
		nongroundFeature.intensity = round((double)sum / (double)count);
		result.push_back(nongroundFeature);
	}
}

void featureExtraction::scoreFilter(vector<int> vec_pointIndex, int left, int right, int& leftAfterFilter, int& rightAfterFilter) {
}

bool featureExtraction::binarySearch(int target,vector<int>& vec_pointIndex) {
	int left = 0; int right = vec_pointIndex.size() - 1;
	while (left <= right) {
		int mid = (right + left) / 2;
		if (vec_pointIndex[mid] == target)
			return true;
		else if (vec_pointIndex[mid] < target)
			left = mid + 1;
		else if (vec_pointIndex[mid] > target)
			right = mid - 1;
	}
	return false;
}

void featureExtraction::setRangeAndIndex() {
	//Ѱұ߽
	float f_max = -9999.9, f_min = 9999.9;
	for (auto point : groundCloud->points) {
		f_max = max(f_max, point.z);
		f_min = min(f_min, point.z);
	}
	//趨߽
	maxRangeGround = ceil(f_max * 100.0 / (float)resolutionGround) * resolutionGround;
	minRangeGround = floor(f_min * 100.0 / (float)resolutionGround) * resolutionGround;
	//趨index
	numOfIndexGround = (maxRangeGround- minRangeGround) / resolutionGround;

	f_max = -9999.9; f_min = 9999.9;
	for (auto point : nongroundCloud->points) {
		f_max = max(f_max, point.z);
		f_min = min(f_min, point.z);
	}
	//趨߽
	maxRangeNonground = ceil(f_max * 100.0 / (float)resolutionNonground) * resolutionNonground;
	minRangeNonground = floor(f_min * 100.0 / (float)resolutionNonground) * resolutionNonground;
	//趨index
	numOfIndexNonground = (maxRangeNonground - minRangeNonground) / resolutionNonground;
}

void featureExtraction::toVolumePixelAndSort() {
	int z=0;

	for (auto point : groundCloud->points) {
		z = floor((point.z * 100 - (float)minRangeGround) / (float)resolutionGround);
		if (z == numOfIndexGround) {
			z--;
		}
		vec_groundIndex.push_back(z);
	}
	sort(vec_groundIndex.begin(), vec_groundIndex.end());
	vec_groundIndex.erase(unique(vec_groundIndex.begin(), vec_groundIndex.end()), vec_groundIndex.end());



	for (auto point : nongroundCloud->points) {
		z = floor((point.z * 100 - (float)minRangeNonground) / (float)resolutionNonground);
		if (z == numOfIndexNonground) {
			z--;
		}
		vec_nongroundIndex.push_back(z);
	}
	sort(vec_nongroundIndex.begin(), vec_nongroundIndex.end());
	vec_nongroundIndex.erase(unique(vec_nongroundIndex.begin(), vec_nongroundIndex.end()), vec_nongroundIndex.end());
}


float calDerivative(vector<gridFeature>& result, int index_z, bool isGround) {
	int n = result.size();
	if (n == 0) {
		cout << "can't calculate derivative because result is empty" << endl;
		return -1;
	}
	for (int i = 0; i < n ;i++) {
		if (result[i].isGround != isGround) {
			continue;
		}
		if (index_z < result[i].minz_devidedByResolution) {
			if (i == 0) {
				float numerator = result[i].deltaz_devidedByResolution;
				float denominator = pow(result[i].minz_devidedByResolution + result[i].deltaz_devidedByResolution - index_z, 2);
				return numerator / denominator;
			}
			else {
				float p_left = float(result[i - 1].deltaz_devidedByResolution) / float(index_z - result[i - 1].deltaz_devidedByResolution);
				float p_right = float(result[i].deltaz_devidedByResolution) / float(result[i].minz_devidedByResolution - index_z + result[i].deltaz_devidedByResolution);
				if (p_left > p_right) {
					float numerator= -result[i-1].deltaz_devidedByResolution;
					float denominator = pow(index_z - result[i - 1].minz_devidedByResolution, 2);
					return numerator / denominator;
				}
				else if (p_left < p_right) {
					float numerator = result[i].deltaz_devidedByResolution;
					float denominator = pow(result[i].minz_devidedByResolution + result[i].deltaz_devidedByResolution - index_z, 2);
					return numerator / denominator;
				}
				else {
					return 0;
				}
			}
		}
		else if (index_z >= result[i].minz_devidedByResolution && index_z < result[i].minz_devidedByResolution + result[i].deltaz_devidedByResolution) {
			return 0;
		}
	}
	float numerator = -result[n-1].deltaz_devidedByResolution;
	float denominator = pow(index_z - result[n - 1].minz_devidedByResolution, 2);
	return numerator / denominator;

}
