#pragma once
#include <iostream>
#include <string>
#include <pcl/common/common_headers.h>
#include <pcl/io/pcd_io.h>
#include <pcl/console/parse.h>
#include <pcl/point_types.h>
#include "utils/pcl_point_type.h"
#include <algorithm>
#include <vector>



//float scoreHitMiss = -0.1;
//float scoreHitHit = 1;
//float scoreMissHit = -0.1;



using namespace std;

struct featureExtractionOption{
//    float label_road = 20000.0;
//    float label_ground = 40000.0;
    float label_road = -1.0;
    float label_ground = -2.0;
    float label_noise = 60000.0;
    float label_other = 10000.0;
    int min_interval = /*15*/6;
    int interval_ground_noise_below = 3;
    int interval_ground_noise_above = 10; //must less than min_interval
    int min_combine_interval = 3;
    int num_distribution = 30;
    int resolution = 3;
};

struct gridFeature {
    int minz_devidedByResolution = 0;
    int deltaz_devidedByResolution = 0;
    int intensity = 0;
    bool isGround = false;
};

struct voxel {
	voxel(int index, float label, float intensity) {
		this->index = index;
		this->label = label;
		this->intensity = intensity;
	}
	int index;
	float label;
	float intensity;


};

float calDerivative(vector<gridFeature>&, int, bool);

class featureExtraction {
public:
    featureExtraction(featureExtractionOption option) {
        this->option = option;
    };
    ~featureExtraction() {};
    
    bool init(pcl::PointCloud<pcl::PointExport>::Ptr);
	bool init2(pcl::PointCloud<pcl::PointExport>::Ptr cloud);

    void setResolution(int resolutionGround , int resolutionNonground ) {
        this->resolutionGround = resolutionGround;
        this->resolutionNonground = resolutionNonground;
    };

	void setNumOfDistribution(int num) {
		this->numOfDistribution = num;

	}
    
    void getFeature(vector<gridFeature>&);
	void getFeature2(vector<gridFeature>&);
	void getFeature3(vector<gridFeature>&);
    
private:
    void setRangeAndIndex();
    void toVolumePixelAndSort();

    void getNongroundFeature(vector<gridFeature>&);
    bool binarySearch(int,vector<int>&);
    static bool comp(const voxel& lhs, const voxel& rhs){
        if( lhs.index < rhs.index)
            return true;
        else if (lhs.index > rhs.index) {
            return false;
        }
//        else if ((lhs.label == option.label_road ||lhs.label == option.label_ground) &&
//                 rhs.label!= option.label_road &&rhs.label!= option.label_ground) {
//            return true;
//        }
        return false;
    }

    int maxRangeGround = 0;
    int minRangeGround = 0;

    int maxRangeNonground = 0;
    int minRangeNonground = 0;

	int maxRange = 0;
	int minRange = 0;

    int resolutionGround;
    int resolutionNonground;
	int resolution = 3;

    int numOfIndexNonground = 0;
    int numOfIndexGround = 0;
	int numOfIndex = 0;

	int numOfDistribution = 3;

	pcl::PointCloud<pcl::PointExport>::Ptr groundCloud;
	pcl::PointCloud<pcl::PointExport>::Ptr nongroundCloud;
    vector<int> vec_groundIndex;
    vector<int> vec_nongroundIndex;
	vector<voxel> vec_voxel;
	void scoreFilter(vector<int>, int, int, int&, int&);
    featureExtractionOption option;
};

