#define PCL_NO_PRECOMPILE
#include "block.h"
#include <pcl/common/transforms.h>
#include <pcl/segmentation/extract_clusters.h>
#include <pcl/filters/extract_indices.h>
BlockGrid::BlockGrid(const Vector2i &index,
        const float &sideLength,
        const vector<double> &resolutions)
    :LocateGridBase(index, sideLength),
      GridManager(sideLength, resolutions)
{
    setOrigin(LocateGridBase::origin_);
    planeParam_.reset(new pcl::ModelCoefficients);
}

void BlockGrid::configInfo()
{
//    blockInfo_.set_local_offset_x(GridBase::center_.x());
//    blockInfo_.set_local_offset_y(GridBase::center_.y());

    locate::BlockDistribution blockDistribution =
            layers_.front()->config();
    blockDistribution_ = blockDistribution;
    hasGround_ = layers_.front()->IsHasGround();


//    if(index_.x() == -10 && index_.y() == 6){
//        PointCloudExport pc = parseBlockPC();
//        pcl::io::savePCDFileBinary(
//                    "/home/juefx/111.pcd", pc);
//        exit(0);

//    }

    PointCloudExport bloockCloud = getBlockPC();
    auto gridTabel = layers_[0]->getGridTable();
    for(auto &raw : gridTabel){
        for(auto &grid : raw){
            if(!grid){
                continue;
            }
            grid.reset();
        }
    }
    PointCloudExport::Ptr groundCloud =
            PointCloudExport::Ptr(new PointCloudExport);
    for(const auto& p : bloockCloud){
        if(fabs(p.label - 20000) < 1e-3 ||
                fabs(p.label - 40000) < 1e-3){
            groundCloud->push_back(p);
        }
    }
    if(groundCloud->size() < 4){
        return;
    }
//    auto natherGround = *groundCloud;
//    Isometry3f m = Isometry3f::Identity();
//    m.translation().z() = 3;
//    pcl::transformPointCloud(natherGround, natherGround, m);
//    auto cloud = groundCloud;
//    *cloud += natherGround;

//    vector<PointCloudExport::Ptr> groundVec = departGround(cloud);
////    pcl::io::savePCDFileBinary("/home/juefx/temp.pcd", *cloud);
////    exit(0);
//    static int index = 0;
//    LOG(INFO) << "groundVec.size(): " << groundVec.size();
//    if(groundVec.size() != 2){
//        pcl::io::savePCDFileBinary("/home/juefx/" + to_string(index) + ".pcd", *cloud);
//        for(int i = 0; i < groundVec.size(); i++){
//            pcl::io::savePCDFileBinary("/home/juefx/" + to_string(index) + "_" + to_string(i) + ".pcd", *groundVec[i]);
//        }
//    }
//    index++;

    segmentPlane(groundCloud, 0.1, true);
}

locate::BlockDistribution BlockGrid::getblockDistribution()
{
    return blockDistribution_;
}

bool BlockGrid::IsHasGround()
{
    return hasGround_;
}

bool mergeGridData(
        const vector<juefx::GridData> &lastGridDataWindow,
        juefx::GridData& merged){
    if(lastGridDataWindow.size() == 0){
        return false;
    }
    for(const juefx::GridData& oneGridData : lastGridDataWindow){
        for(juefx::Distribution oneDstbt : oneGridData){
            if(merged.size() == 0){
                merged.push_back(oneDstbt);
                continue;
            }
            vector<size_t> toMergeIndexes;
            for(size_t i = 0; i < merged.size(); i++){
                const juefx::Distribution& dstbt = merged[i];
                int32_t dstbtCeil = dstbt.floor + dstbt.height;
                int32_t oneDstbtCeil = oneDstbt.floor + oneDstbt.height;
                if(!(oneDstbtCeil < dstbt.floor ||
                        oneDstbt.floor > dstbtCeil)){
                    toMergeIndexes.push_back(i);
                }
            }
            if(toMergeIndexes.size() == 0){
                merged.push_back(oneDstbt);
                continue;
            }

            for(size_t i : toMergeIndexes){
                const juefx::Distribution& dstbt = merged[i];
                oneDstbt.floor = min(oneDstbt.floor, dstbt.floor);
                int32_t ceil = max(oneDstbt.floor + oneDstbt.height,
                                   dstbt.floor + dstbt.height);
                oneDstbt.height = ceil - oneDstbt.floor;
                oneDstbt.intensity += dstbt.intensity;
            }
            oneDstbt.intensity /= (toMergeIndexes.size() + 1);

            juefx::GridData newMerged;
            newMerged.reserve(4);
            newMerged.push_back(oneDstbt);
            for(size_t i = 0; i < merged.size(); i++){
                if(find(toMergeIndexes.begin(), toMergeIndexes.end(), i)
                        == toMergeIndexes.end()){
                    newMerged.push_back(merged[i]);
                }
            }
            merged.swap(newMerged);
        }
    }
    return true;
}

void BlockGrid::configMultiLayer(
        uint8_t laterCnt)
{
    for(uint8_t i = 0; i < laterCnt; i++){
        addLayer(resolutions_[0] * (2 << i));
        boost::shared_ptr<GridLayer> &currBlockLayer = layers_.back();
        boost::shared_ptr<GridLayer> &lastBlockLayer = layers_.at(layers_.size() - 2);
        uint32_t currItemCntPerSide = currBlockLayer->getItemCntPerSide();
        for(size_t row = 0; row < currItemCntPerSide; row++){
            for(size_t col = 0; col < currItemCntPerSide; col++){
                vector<juefx::GridData> lastGridDataWindow;
                lastGridDataWindow.reserve(4);
                size_t rowStart = row << 1;
                size_t colStart = col << 1;
                for(size_t lastRow = rowStart; lastRow < rowStart + 2; lastRow++){
                    for(size_t lastCol = colStart; lastCol < colStart + 2; lastCol++){
                        LocateGridBase* lastGrid;
                        if(!lastBlockLayer->getGridBase({lastRow, lastCol}, lastGrid)){
                            continue;
                        }
                        Grid* lastGridPtr = reinterpret_cast<Grid*>(lastGrid);
                        const juefx::GridData* lastGridData = lastGridPtr->getGridData();
                        if(lastGridData->size() == 0){
                            continue;
                        }
                        lastGridDataWindow.emplace_back(*lastGridData);
                    }
                }
                juefx::GridData mergedGridData;
                mergedGridData.reserve(4);
                if(!mergeGridData(lastGridDataWindow, mergedGridData)){
                    continue;
                }
                auto currGrid = currBlockLayer->getGrid({row, col});
                Grid* currGridPtr = reinterpret_cast<Grid*>(currGrid.get());
                currGridPtr->setGridData(mergedGridData);
            }
        }
    }
}

PointCloudExport BlockGrid::getBlockPC()
{
    PointCloudExport ret;
    auto gridTabel = layers_[0]->getGridTable();
    for(auto &raw : gridTabel){
        for(auto &grid : raw){
            if(!grid){
                continue;
            }
            Grid* sGrid = reinterpret_cast<Grid*>(grid.get());
            ret += *sGrid->getPointCloud();
        }
    }
    return ret;
}

PointCloudExport BlockGrid::parseBlockPC()
{
    PointCloudExport ret;
    auto gridTabel = layers_[0]->getGridTable();
    for(const auto &raw : gridTabel){
        for(const auto &grid : raw){
            if(!grid){
                continue;
            }
            Grid* sGrid = reinterpret_cast<Grid*>(grid.get());
            ret += sGrid->parsePC();
        }
    }
    return ret;
}

void BlockGrid::segmentPlane(const PointCloudExport::Ptr cloud,
                             double distance_m,
                             bool ransac)
{
    pcl::SACSegmentation<PointExport> seg;
    seg.setModelType(pcl::SACMODEL_PLANE);
    seg.setOptimizeCoefficients(false);
    if (ransac)
    {
        seg.setMethodType(pcl::SAC_RANSAC);
        seg.setMaxIterations(2000);
    }
    else
    {
        seg.setMethodType(pcl::SAC_LMEDS);
        if (cloud->points.size() > 10000)
            seg.setMaxIterations(1000);
        else
            seg.setMaxIterations(100);
    }
    seg.setDistanceThreshold(distance_m);
    seg.setInputCloud(cloud);
    // 一个模型参数对象，对于平面 ax+by+cz+d = 0，最终 a,b,c 即为法向量
    pcl::PointIndices::Ptr inliers(new pcl::PointIndices);
    seg.segment(*inliers, *planeParam_);

    if (planeParam_->values[3] < 0)
    {
        planeParam_->values[0] *= -1;
        planeParam_->values[1] *= -1;
        planeParam_->values[2] *= -1;
        planeParam_->values[3] *= -1;
    }


}

vector<PointCloudExport::Ptr> BlockGrid::departGround(
        const PointCloudExport::Ptr cloud)
{
//    vector<PointCloudExport::Ptr> clusterRet;
//    pcl::search::KdTree<PointExport>::Ptr tree (new pcl::search::KdTree<PointExport>);
//    tree->setInputCloud (cloud);

//    std::vector<pcl::PointIndices> cluster_indices;
//    pcl::EuclideanClusterExtraction<PointExport> ec;

//    ec.setClusterTolerance (1.5);
//    ec.setMinClusterSize (1000);
//    ec.setMaxClusterSize (600000);
//    ec.setSearchMethod (tree);
//    ec.setInputCloud (cloud);
//    ec.extract (cluster_indices);
//    clusterRet.reserve (cluster_indices.size());
//    for (std::vector<pcl::PointIndices>::const_iterator it = cluster_indices.begin(); it != cluster_indices.end(); ++it)
//    {
//        pcl::ExtractIndices<PointExport> extract;
//        extract.setInputCloud(cloud);
//        extract.setIndices(boost::make_shared<const pcl::PointIndices>(*it));
//        PointCloudExport::Ptr out (new PointCloudExport);
//        extract.filter(*out);
//        clusterRet.push_back(out);
////        Vector3d offset = Vector3d(-395.747, 0-474.33, 014.5708);
////        for(auto &p : *out){
////            p.x += offset.x(); p.y += offset.y(); p.z += offset.z();
////        }
////        pcl::io::savePCDFileBinary("/tmp/temp/" + to_string(it - cluster_indices.begin()) + ".pcd", *out);
//    }
//    return clusterRet;

//    vector<PointCloudExport::Ptr> ret;
//    vector<float> levelVec;
//    float levelThrehold = 0.5;
//    for(const auto& p : *cloud){
//        bool find = false;
//        for(size_t index = 0; index < levelVec.size(); index++){
//            if(fabs(p.z - levelVec[index]) < levelThrehold){
//                levelVec[index] = (levelVec[index] * ret[index]->size()
//                        + p.z) / (ret[index]->size() + 1);
//                ret[index]->push_back(p);
//                find = true;
//                break;
//            }
//        }
//        if(!find){
//            ret.push_back(PointCloudExport::Ptr(new PointCloudExport));
//            ret.back()->push_back(p);
//            levelVec.push_back(p.z);
//        }
//    }
//    return ret;
}
