#pragma once
#ifndef BLOCK_H
#define BLOCK_H

#include "gridbase.h"
#include "grid.h"
#include "gridmanager.h"
#include "block/block_info.pb.h"
#include <pcl/segmentation/extract_clusters.h>
#include <pcl/segmentation/sac_segmentation.h>

class BlockGrid : public LocateGridBase, public GridManager
{
public:
    BlockGrid(const Vector2i &index,
              const float &sideLength,
              const vector<double> &resolutions);

    void configInfo();

    locate::BlockDistribution getblockDistribution();

    bool IsHasGround();


    inline const juefx::GridData *getGridData(
            uint8_t type,
            const juefx::GridIndex::ShortId &gridIndex)
    {
        juefx::GridData* ret = NULL;
        LocateGridBase* gridGrid;
        if(type >= layers_.size()){
            return ret;
        }
        if(!layers_[type]->getGridBase({gridIndex.x, gridIndex.y}, gridGrid)){
            return ret;
        }
        auto grid = reinterpret_cast<Grid*>(gridGrid);
        return grid->getGridData();
    }

    void configMultiLayer(uint8_t laterCnt = 4);

    PointCloudExport getBlockPC();

    PointCloudExport parseBlockPC();

    void segmentPlane(const PointCloudExport::Ptr cloud,
                      double distance_m, bool ransac);

    vector<PointCloudExport::Ptr> departGround(
            const PointCloudExport::Ptr cloud);

    pcl::ModelCoefficients::Ptr getplaneParam(){
        return planeParam_;
    }

private:

    locate::BlockDistribution blockDistribution_;

    bool hasGround_ = false;

    pcl::ModelCoefficients::Ptr planeParam_;

};//end of class


#endif // BLOCK_H
