#include "grid.h"

Grid::Grid(const Vector2i &index, const float &resolution)
    :LocateGridBase(index, resolution)
{
    pointcloud_.reset(new PointCloudExport);
    gridDtribution_.reset(new locate::GridDistribution);
}



PointCloudExport::Ptr Grid::getPointCloud()
{
    return pointcloud_;
}

boost::shared_ptr<locate::GridDistribution> Grid::getGridDistribution()
{
    return gridDtribution_;
}

void Grid::setGridDistribution(const locate::GridDistribution &gridDtribution)
{
    gridData_.reserve(gridDtribution.oneDistributions.size());
    for(const auto& distr : gridDtribution.oneDistributions){
        juefx::Distribution distribution;
        distribution.floor = distr.floor * 3;
        distribution.height = distr.z_delta_level * 3;
        distribution.intensity = distr.intensity * 8;
        gridData_.push_back(distribution);
    }
}

PointCloudExport Grid::parsePC()
{
    PointCloudExport ret;
    float centerX = ((int)(pointcloud_->front().x / resolution_) + 0.5) * resolution_;
    float centerY = ((int)(pointcloud_->front().y / resolution_) + 0.5) * resolution_;
    for(const auto& distr : gridData_){
        for(int32_t j = 0;
            j < distr.height;
            j++){
            int32_t indexZ = j + distr.floor;
            PointExport pointExport;
            pointExport.x = centerX;
            pointExport.y = centerY;
            pointExport.z = ((float)indexZ + 0.5) * 0.01;
            pointExport.intensity = distr.intensity;
            ret.push_back(pointExport);
        }
    }
    return ret;
}
