#ifndef GRID_H
#define GRID_H

#include "gridbase.h"
#include "utils/basic_functions.h"
#include "distribution/distribution.pb.h"
#include "feature/featureExtraction.h"

class Grid : public LocateGridBase
{
public:
    Grid(const Vector2i &index, const float &resolution);

    ~Grid(){
        pointcloud_.reset();
    }

    inline void addPoint(const PointExport &point)
    {
        //    boost::unique_lock<boost::shared_mutex> lock(insertMtx_);
        pointcloud_->push_back(point);
    }

    PointCloudExport::Ptr getPointCloud();

    inline void configInfo()
    {
//        pcl::io::savePCDFileBinary("/home/juefx/temp.pcd", *pointcloud_);
//        exit(0);
        vector<gridFeature> result;
        //构建特征提取类
        featureExtractionOption option;
        featureExtraction fe(option);
        //使用点云初始化
        fe.init(pointcloud_);
        //获取结果
        fe.getFeature(result);

        for(const gridFeature& distribution : result){
            locate::OneDistribution oneDistribution;
            juefx::Distribution oneData;
            oneDistribution.floor = distribution.minz_devidedByResolution;
            oneDistribution.z_delta_level = distribution.deltaz_devidedByResolution;
            oneDistribution.intensity = distribution.intensity / 8;
            gridDtribution_->oneDistributions.push_back(oneDistribution);
            oneData.floor = oneDistribution.floor * 3;
            oneData.height = oneDistribution.z_delta_level * 3;
            oneData.intensity = distribution.intensity;
            gridData_.push_back(oneData);
            hasGround_ = hasGround_ || distribution.isGround;
        }

//        sort(pointcloud_->points.begin(), pointcloud_->points.end(),
//             [](const PointExport &src, const PointExport &tar){
//            return src.z < tar.z;
//        });
//        PointCloudExport extractedPc = parsePC();
//        for(auto& extractedP : extractedPc){
//            bool found = false;
//            for(const auto& originP : pointcloud_->points){
//                if(int(originP.z / 0.03) == int(extractedP.z / 0.03)){
//                    extractedP.label = 1;
//                    found = true;
//                    break;
//                }
//            }
//            if(!found){
//                extractedP.label = 0;
//            }
//        }
//        static int i = 0;
//        pcl::io::savePCDFile("/home/juefx/data/distrubute/" + to_string(i++) + ".pcd", extractedPc);

    }

    boost::shared_ptr<locate::GridDistribution> getGridDistribution();

    inline const juefx::GridData *getGridData() const
    {
        return &gridData_;
    }

    inline void setGridData(const juefx::GridData& gridData)
    {
        gridData_ = gridData;
    }

    void setGridDistribution(const locate::GridDistribution &gridDtribution);

    inline bool getHasGround()
    {
        return hasGround_;
    }

    PointCloudExport parsePC();

private:
    PointCloudExport::Ptr pointcloud_;

    boost::shared_ptr<locate::GridDistribution> gridDtribution_;

    bool hasGround_ = false;

    juefx::GridData gridData_;

};

#endif // GRID_H
