#include "gridlayer.h"
#include "block.h"

GridLayer::GridLayer(const float sideLength,
        const double resolution)
    :resolution_(resolution),
      sideLength_(sideLength)
{
//    grids_.reset(new unordered_map<uint32_t, boost::shared_ptr<GridBase>>);
    itemCntPerSide_ = (round)(sideLength_ / resolution_);
    layerResolutions = DoubleLayerResolutions;

//    gridTabel_.resize(pow(itemCntPerSide_, 2));
//    for(auto &col : gridTabel_){
//        col.resize(itemCntPerSide_);
//    }
}



boost::shared_ptr<LocateGridBase> GridLayer::getBlock(
        const Vector2i &index)
{
    Vector2i shiftedIndex = index + Vector2i{itemCntPerSide_ / 2, itemCntPerSide_ / 2};
    if(shiftedIndex.x() + 1 > gridTabel_.size()){
        gridTabel_.resize(shiftedIndex.x() + 1);
        maxXIndex = max(maxXIndex, (size_t)shiftedIndex.x());
    }
    if(shiftedIndex.y() + 1 > gridTabel_[shiftedIndex.x()].size()){
        gridTabel_[shiftedIndex.x()].resize(shiftedIndex.y() + 1);
        maxYIndex = max(maxYIndex, (size_t)shiftedIndex.y());
    }
    boost::shared_ptr<LocateGridBase> &ret = gridTabel_[shiftedIndex.x()][shiftedIndex.y()];
    if(!ret){
        ret = boost::shared_ptr<LocateGridBase>(
                    new BlockGrid(index, resolution_, DoubleLayerResolutions));
    }
    return ret;

//    if(!(index.x() >= 0 && index.x() < itemCntPerSide_
//            && index.y() >= 0 && index.y() < itemCntPerSide_)){
//        return boost::shared_ptr<GridBase>();
//    }
//    if(index.x() + 1 > gridTabel_.size()){
//        gridTabel_.resize(index.x() + 1);
//        maxXIndex = max(maxXIndex, (size_t)index.x());
//    }
//    if(index.y() + 1 > gridTabel_[index.x()].size()){
//        gridTabel_[index.x()].resize(index.y() + 1);
//        maxYIndex = max(maxYIndex, (size_t)index.y());
//    }
//    boost::shared_ptr<GridBase> &ret = gridTabel_[index.x()][index.y()];
//    if(!ret){
//        ret = boost::shared_ptr<GridBase>(
//                    new BlockGrid(index, resolution_, DoubleLayerResolutions));
//    }
//    return ret;
}

void GridLayer::setOrigin(const Vector2f &origin)
{
    origin_ = origin * 100;
    origin_ = {round(origin_.x()), round(origin_.y())};
}

const vector<vector<boost::shared_ptr<LocateGridBase> > >
GridLayer::getGridTable()
{
    return gridTabel_;
}

float GridLayer::getResolution()
{
    return resolution_;
}

void GridLayer::setResolution(float resolution)
{
    resolution_ = resolution;
}

void GridLayer::setSideLength(float sideLength)
{
    sideLength_ = sideLength;
}

Vector2i GridLayer::decodeIndex(uint32_t encoded)
{
    return Vector2i(encoded / itemCntPerSide_, encoded % itemCntPerSide_);
}

void GridLayer::setLayerResolutions(
        vector<double> resolutions)
{
    layerResolutions = resolutions;
}

uint32_t GridLayer::getItemCntPerSide()
{
    return itemCntPerSide_;
}

locate::BlockDistribution GridLayer::config()
{
    locate::BlockDistribution blockDistribution;
    blockDistribution.gridDistributions.resize(pow(round(itemCntPerSide_), 2));
    for(auto raw : gridTabel_){
        for(auto gridPtr : raw){
            if(!gridPtr){
                continue;
            }
            Grid* grid = reinterpret_cast<Grid*>(gridPtr.get());

            grid->configInfo();
            if(grid->getHasGround()){
                hasGround_ = true;
            }
            auto distributions = grid->getGridDistribution();
            int32_t grid_id = encodeIndex(grid->getIndex());
            int32_t shiftGridId = grid_id +
                    (itemCntPerSide_ / 2) * (itemCntPerSide_ + 1);
            blockDistribution.gridDistributions.at(shiftGridId) = *distributions;
        }
    }
    return blockDistribution;
}

bool GridLayer::IsHasGround()
{
    return hasGround_;
}
