#ifndef GRIDLAYER_H
#define GRIDLAYER_H

#include "utils/basic_types.h"
#include "grid.h"
#include "utils/pcl_point_type.h"

class GridLayer
{
public:
    GridLayer(const float sideLength,
              const double resolution);

    inline bool getGridBase(const Vector2i &index,
                        LocateGridBase *&grid)
    {
        Vector2i shiftedIndex = index + Vector2i{itemCntPerSide_ / 2, itemCntPerSide_ / 2};
        if(shiftedIndex.x() + 1 > gridTabel_.size()){
            return false;
        }
        if(shiftedIndex.y() + 1 > gridTabel_[shiftedIndex.x()].size()){
            return false;
        }
        if(gridTabel_[shiftedIndex.x()][shiftedIndex.y()]){
            grid = gridTabel_[shiftedIndex.x()][shiftedIndex.y()].get();
            return true;
        }
        return false;
    }

    inline int8_t IsGridActive(const Vector2i &index){
        if(index.x() > maxXIndex){
            if(index.y() < 0){
                return (int8_t)RightDown;
            }else if(index.y() > maxYIndex){
                return (int8_t)RightUp;
            }else {
                return (int8_t)Right;
            }
        }else if(index.x() < 0){
            if(index.y() < 0){
                return (int8_t)LeftDown;
            }else if(index.y() > maxYIndex){
                return (int8_t)LeftUp;
            }else {
                return (int8_t)Left;
            }
        }else{
            if(index.y() < 0){
                return (int8_t)Down;
            }else if(index.y() > maxYIndex){
                return (int8_t)Up;
            }else {
                return 100;
            }
        }
    }

    inline boost::shared_ptr<LocateGridBase> getGrid(
            const Vector2i &index){
        Vector2i shiftedIndex = index + Vector2i{itemCntPerSide_ / 2, itemCntPerSide_ / 2};

        if(shiftedIndex.x() + 1 > gridTabel_.size()){
            gridTabel_.resize(shiftedIndex.x() + 1);
            maxXIndex = max(maxXIndex, (size_t)shiftedIndex.x());
        }
        if(shiftedIndex.y() + 1 > gridTabel_[shiftedIndex.x()].size()){
            gridTabel_[shiftedIndex.x()].resize(shiftedIndex.y() + 1);
            maxYIndex = max(maxYIndex, (size_t)shiftedIndex.y());
        }
        boost::shared_ptr<LocateGridBase> &ret = gridTabel_[shiftedIndex.x()][shiftedIndex.y()];
        if(!ret){
            ret = boost::shared_ptr<LocateGridBase>(
                        new Grid(index, resolution_));
        }
        return ret;
    }

    boost::shared_ptr<LocateGridBase> getBlock(const Vector2i &index);

    inline Vector2i getGridIndexByPoint(const PointExport &point)
    {
        Vector2i index;
        index.x() = floor((point.x * 100 - origin_.x()) / lround(resolution_ * 100));
        index.y() = floor((point.y * 100 - origin_.y()) / lround(resolution_ * 100));
        return index;
    }

    inline Vector2i getGridIndexByPoint(float x, float y)
    {
        Vector2i index;
        index.x() = floor((x * 100 - origin_.x()) / round(resolution_ * 100));
        index.y() = floor((y * 100 - origin_.y()) / round(resolution_ * 100));
        return index;
    }

    void setOrigin(const Vector2f &origin);

    const vector<vector<boost::shared_ptr<LocateGridBase>>> getGridTable();

    void setResolution(float resolution);

    void setSideLength(float sideLength);

    float getResolution();

    inline uint32_t encodeIndex(const Vector2i &index)
    {
        return index.x() * itemCntPerSide_ + index.y();
    }

    Vector2i decodeIndex(uint32_t encoded);

    void setLayerResolutions(vector<double> resolutions);

    uint32_t getItemCntPerSide();

    locate::BlockDistribution config();

    bool IsHasGround();

protected:
    Vector2f origin_ = Vector2f::Zero();

    vector<vector<boost::shared_ptr<LocateGridBase>>> gridTabel_;

    double resolution_;

    float sideLength_;

    uint32_t itemCntPerSide_ = 0;
private:

    vector<double> layerResolutions;

    size_t maxXIndex = 0;

    size_t maxYIndex = 0;

    bool hasGround_ = false;
};

#endif // GRIDLAYER_H
