#include "gridmanager.h"
#include "block.h"

GridManager::GridManager(
        const float sideLength,
        const vector<double> resolutions)
    :sideLength_(sideLength),
      resolutions_(resolutions)
{
    for(double resolution : resolutions){
        layers_.push_back(
                    boost::shared_ptr<GridLayer>(new GridLayer(sideLength, resolution)));
    }
}

Vector2i GridManager::getGridIndexByPoint(
        const PointExport &point,
        size_t layerLevel)
{
    if(layerLevel >= layers_.size()){
        return nullIndex;
    }
    Vector2i index = nullIndex;
    index.x() = floor((point.x - origin_.x()) / layers_[layerLevel]->getResolution());
    index.y() = floor((point.y - origin_.y()) / layers_[layerLevel]->getResolution());
    return index;
}

void GridManager::setOrigin(const Vector2f &origin)
{
    origin_ = origin;
    for(auto &layer : layers_){
        layer->setOrigin(origin);
    }
}

void GridManager::setSideLength(float sideLength)
{
    sideLength_ = sideLength;
}

vector<boost::shared_ptr<GridLayer>> GridManager::getLayers() const
{
    return layers_;
}

void GridManager::addLayer(const double &resolution)
{
    layers_.push_back(
                boost::shared_ptr<GridLayer>(new GridLayer(sideLength_, resolution)));
}
