#include "grid.h"
#include "extractFunction.h"
#include "feature/featureExtraction.h"

Grid::Grid(const Vector2i &index, const float &resolution)
    :GridBase(index, resolution)
{
    pointcloud_.reset(new PointCloudExport);
    gridDtribution_.reset(new GridDistribution_);
}

void Grid::addPoint(const PointExport &point)
{
//    boost::unique_lock<boost::shared_mutex> lock(insertMtx_);
    pointcloud_->push_back(point);
}

PointCloudExport::Ptr Grid::getPointCloud()
{
    return pointcloud_;
}

void Grid::configInfo()
{
//    float zMax = numeric_limits<float>::lowest();
//    float zMin = numeric_limits<float>::max();
//    for(const auto &point : pointcloud_->points){
//        zMax = zMax > point.z ? zMax : point.z;
//        zMin = zMin < point.z ? zMin : point.z;
//    }
//    OneDistribution_ oneDistribution;
//    oneDistribution.floor = floor(zMin / ZResolution);
//    oneDistribution.z_delta_leval = (zMax - zMin) / ZResolution;
//    gridDtribution_.oneDistributions.push_back(oneDistribution);

//    vector<extrac> ret = extractFromBlock(pointcloud_);
//    for(const extrac& distribution : ret){
//        OneDistribution_ oneDistribution;
//        oneDistribution.floor = distribution.minz_devidedBy3;
//        oneDistribution.z_delta_level = distribution.deltaz_devidedBy3;
//        oneDistribution.intensity = distribution.intensity / 8;
//        gridDtribution_.oneDistributions.push_back(oneDistribution);
//    }

    vector<gridFeature> result;
    //构建特征提取类
    featureExtraction fe;
    //使用点云初始化
    fe.init(pointcloud_);
    //获取结果
    fe.getFeature(result);

    for(const gridFeature& distribution : result){
        OneDistribution_ oneDistribution;
        oneDistribution.floor = distribution.minz_devidedByResolution;
        oneDistribution.z_delta_level = distribution.deltaz_devidedByResolution;
        oneDistribution.intensity = distribution.intensity / 8;
        gridDtribution_->oneDistributions.push_back(oneDistribution);
        hasGround_ = hasGround_ || distribution.isGround;
    }
}

boost::shared_ptr<GridDistribution_> Grid::getGridDistribution()
{
    return gridDtribution_;
}

void Grid::setGridDistribution(
        const GridDistribution_ &gridDtribution)
{
    gridData_.reserve(gridDtribution.oneDistributions.size());
    for(const auto& distr : gridDtribution.oneDistributions){
        juefx::Distribution distribution;
        distribution.floor = distr.floor * 3;
        distribution.height = distr.z_delta_level * 3;
        distribution.intensity = distr.intensity * 8;
        gridData_.push_back(distribution);
    }
}

bool Grid::getHasGround()
{
    return hasGround_;
}
