#include "gridlayer.h"
#include "block.h"

GridLayer::GridLayer(const float sideLength,
        const double resolution)
    :resolution_(resolution),
      sideLength_(sideLength)
{
    grids_.reset(new unordered_map<uint32_t, boost::shared_ptr<GridBase>>);
    itemCntPerSide_ = (uint32_t)(sideLength_ / resolution_);
    layerResolutions = DoubleLayerResolutions;
}

bool GridLayer::getGrid(
        const Vector2i &index, boost::shared_ptr<GridBase> &grid)
{
    uint32_t encodedIndex = encodeIndex(index);
    auto iter = grids_->find(encodedIndex);
    if(iter != grids_->end()){
        grid = iter->second;
        return true;
    }else{
        return false;
    }
}



boost::shared_ptr<GridBase> GridLayer::getGrid(
        const Vector2i &index, const GridType type)
{
    boost::shared_ptr<GridBase> grid;
    uint32_t encodedIndex = encodeIndex(index);
    if(encodedIndex > 30000){
        int i = 1;
        i++;
    }
    auto iter = grids_->find(encodedIndex);
    if(iter != grids_->end()){
        grid = iter->second;
    }else{
        if(type == gridType){
            grid = boost::shared_ptr<GridBase>(
                        new Grid(index, resolution_));
        }else if(type == blockType){
            grid = boost::shared_ptr<GridBase>(
                        new BlockGrid(index, resolution_, DoubleLayerResolutions));
        }
        grids_->insert(make_pair(encodedIndex, grid));
    }
    return grid;
}

void GridLayer::addGridByIndex(
        const Vector2i &index, const GridType type)
{
    uint32_t encodedIndex = encodeIndex(index);
    boost::shared_ptr<GridBase> grid;
    if(type == gridType){
        grid = boost::shared_ptr<GridBase>(new Grid(index, resolution_));
    }else if(type == blockType){
        grid = boost::shared_ptr<GridBase>(new BlockGrid(index, resolution_, DoubleLayerResolutions));
    }
    boost::unique_lock<boost::shared_mutex> lock(insertMtx_);
    grids_->insert(make_pair(encodedIndex, grid));
}

Vector2i GridLayer::getGridIndexByPoint(const PointExport &point)
{
    Vector2i index;
    index.x() = (point.x - origin_.x()) / resolution_;
    index.y() = (point.y - origin_.y()) / resolution_;
    return index;
}

Vector2i GridLayer::getGridIndexByPoint(float x, float y)
{
    Vector2i index;
    index.x() = floor((x - origin_.x()) / resolution_);
    index.y() = floor((y - origin_.y()) / resolution_);
    return index;
}

void GridLayer::setOrigin(const Vector2f &origin)
{
    origin_ = origin;
}

const boost::shared_ptr<unordered_map<uint32_t, boost::shared_ptr<GridBase> > >
GridLayer::getGridMap()
{
    return grids_;
}

float GridLayer::getResolution()
{
    return resolution_;
}

void GridLayer::setResolution(float resolution)
{
    resolution_ = resolution;
}

void GridLayer::setSideLength(float sideLength)
{
    sideLength_ = sideLength;
}

uint32_t GridLayer::encodeIndex(const Vector2i &index)
{
    return index.x() * itemCntPerSide_ + index.y();
}

Vector2i GridLayer::decodeIndex(uint32_t encoded)
{
    return Vector2i(encoded / itemCntPerSide_, encoded % itemCntPerSide_);
}

void GridLayer::setLayerResolutions(
        vector<double> resolutions)
{
    layerResolutions = resolutions;
}

uint32_t GridLayer::getItemCntPerSide()
{
    return itemCntPerSide_;
}
