#ifndef GRIDLAYER_H
#define GRIDLAYER_H

#include "utils/basic_types.h"
#include "grid.h"
#include "utils/pcl_point_type.h"

class GridLayer
{
public:
    GridLayer(const float sideLength,
              const double resolution);

    bool getGrid(const Vector2i &index,
                 boost::shared_ptr<GridBase> &grid);

    boost::shared_ptr<GridBase> getGrid(
            const Vector2i &index, const GridType type = gridType);

    void addGridByIndex(
            const Vector2i &index, const GridType type = gridType);

    Vector2i getGridIndexByPoint(const PointExport &point);

    Vector2i getGridIndexByPoint(float x, float y);

    void setOrigin(const Vector2f &origin);

    const boost::shared_ptr<unordered_map<uint32_t, boost::shared_ptr<GridBase> > > getGridMap();

    void setResolution(float resolution);

    void setSideLength(float sideLength);

    float getResolution();

    uint32_t encodeIndex(const Vector2i &index);

    Vector2i decodeIndex(uint32_t encoded);

    void setLayerResolutions(vector<double> resolutions);

    uint32_t getItemCntPerSide();

protected:
    Vector2f origin_ = Vector2f::Zero();

    boost::shared_ptr<unordered_map<uint32_t, boost::shared_ptr<GridBase>>> grids_;

    double resolution_;

    float sideLength_;

    uint32_t itemCntPerSide_ = 0;
private:
    boost::shared_mutex insertMtx_;

    vector<double> layerResolutions;
};

#endif // GRIDLAYER_H
