#ifndef StreamInfoLoad_H
#define StreamInfoLoad_H

#include "utils/basic_types.h"
#include "common/header.pb.h"
#include "block/block_index.pb.h"
#include "block/block_info.pb.h"
#include "distribution/distribution.pb.h"
#include <GeographicLib/LocalCartesian.hpp>
#include "grid/gridmanager.h"
#include "grid/block.h"

typedef pcl::PointCloud<pcl::PointXYZI> PointCloudXYZI;
using namespace GeographicLib;

namespace Internal {
struct BlockDescribe{
    int32_t blockId = 0;
    uint32_t blockOffset = 0;
    uint32_t blockSize = 0;
};
}

typedef ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >
ProtoRepeatedUint32;

class StreamInfoLoad
{
public:
    StreamInfoLoad(const string& streamPath);

    bool load();

    void parseOneBlock(
            const Internal::BlockDescribe &internalBlockDescribe);

    vector<uint32_t> parseBitMap(
            const ProtoRepeatedUint32& bitmap,
            const uint32_t size);

    void Forward(double lat, double lon, double h, double& x, double& y, double& z);

    void Reverse(double x, double y, double z, double& lat, double& lon, double& h);

    float getHighResolution();

    float getLowResolution();

    float getBlockResolution();

    boost::shared_ptr<GridManager> getBlockManager();

    string getMeshId();

    Vector2f getOrigin();
    Vector3f getOffset();
private:
    PointCloudXYZI::Ptr parseBitmapBlock(
            const BlockInfo &blockInfo,
            const Distributions &distribution,
            const Internal::BlockDescribe &internalBlockDescribe);

    PointCloudXYZI::Ptr parseIndexBlock(
            const BlockInfo &blockInfo,
            const Distributions &distribution,
            const Internal::BlockDescribe &internalBlockDescribe);

private:
    string streamPath_;

    uint16_t headerLength_;

    Header header_;

    BlockIndex blockIndex_;

    vector<Internal::BlockDescribe> blockDescribeVec_;

    Distributions Distributions_;

    PointCloudXYZI::Ptr extractedPointcloudLayers_;

    Vector3f offset_ = Vector3f::Zero();

    boost::shared_ptr<LocalCartesian> proj_;

    float highResolution = 0;

    float lowResolution = 0;

    float blockResolution = 0;

    boost::shared_ptr<GridManager> blockManager_;

    boost::shared_ptr<GridLayer> meshLayer_;

    string meshId_;

    Vector2f origin_ = Vector2f::Zero();
};

#endif // StreamInfoLoad_H
