#ifndef STREAMMANAGER_H
#define STREAMMANAGER_H

#include "streaminfo.h"

#include "../../interface/voxel_map_define.h"
#include "utils/basic_functions.h"

class StreamManager
{
public:
    StreamManager();
    bool SetMapPath(std::string path_name);

    inline bool IsMapLoaded(const GnssPoint blh_pose){
        ulong mesh = GetMeshFromPosition(blh_pose.L, blh_pose.B);
        if(mesh == INVALID_MESHID){
            return false;
        }
        if(IsMeshLoaded(mesh)){
            return true;
        }
        ThreadPtr trd = ThreadPtr(new boost::thread(
                                      bind(&StreamManager::checkAndLoad,
                                           this,
                                           mesh)));
        return false;
    }

    void checkAndLoad(ulong mesh);

    bool loadMesh(ulong mesh);

    inline bool findMesh(const ulong &mesh,
                         boost::shared_ptr<StreamInfo>& meshStream) const{
//        map<ulong, boost::shared_ptr<StreamInfo>> meshStreamMap;
//        {
//            boost::shared_lock<boost::shared_mutex> readLock(meshStreaMtx_);
//            meshStreamMap = meshStreamMap_;
//        }
        auto iter = meshStreamMap_.find(mesh);
        if(iter == meshStreamMap_.end()){
            return false;
        }
        meshStream = iter->second;
        return true;
    }

    inline bool GuessPosition(
            const GnssPoint &init_pose, Eigen::Isometry3d &pose_in_map)
    {
        ulong mesh = GetMeshFromPosition(init_pose.L, init_pose.B);
        if(mesh == INVALID_MESHID){
            return false;
        }
        if(!checkCurrMesh(mesh)){
            boost::shared_ptr<StreamInfo> meshStream;
            if(!findMesh(mesh, meshStream)){
                return false;
            }
            {
                boost::unique_lock<boost::shared_mutex> writeLock(currMashMtx_);
                currMeshStream_ = meshStream;
                currMeshOffset_ = currMeshStream_->getOffset();
            }
        }

//        uint8_t resolution_type = 2;
//        Vector2f curr(1755.5, 932.87);
//        juefx::GridIndex index = GetCellIndex(resolution_type, curr.x(), curr.y());
//        index = index.Move(-23, -23);
//        auto voxel = GetCellValue(resolution_type, index);
//        auto gridCenter = GetCenterOfCell(resolution_type, curr.x(), curr.y());
//        std::ofstream ofs("/home/juefx/grid.txt");
//        for(auto distribution : *voxel){
//            for(int32_t j = 0;
//                j < distribution.height / 3;
//                j++){
//                int32_t indexZ = j + distribution.floor / 3;
//                double point[4];
//                point[0] = gridCenter.x();
//                point[1] = gridCenter.y();
//                point[2] = ((float)indexZ + 0.5) * 3 / 100;
//                point[3] = distribution.intensity;
//                ofs << point[0] << "," << point[1] << ","
//                                << point[2] << "," << point[3] << std::endl;
//            }
//        }
//        ofs.close();
//        exit(0);

//        uint8_t resolution_type = 1;
//        Vector2f c(1741.298, 929.727);
//        std::ofstream ofs("/home/juefx/grid_" + to_string((uint16_t)resolution_type) + ".txt");
//        for(int32_t x = -1000; x < 1000; x++){
//            for(int32_t y = -1000; y < 1000; y++){
//                Vector2f curr = c + Vector2f((float)x * GridResolutionHigh * (1 << resolution_type),
//                                             (float)y * GridResolutionHigh * (1 << resolution_type));
//                juefx::GridIndex index = GetCellIndex(resolution_type, curr.x(), curr.y());
//                const juefx::GridData* voxel = GetCellValue(resolution_type, index);
//                if(voxel == NULL){
//                    continue;
//                }
//                auto gridCenter = GetCenterOfCell(resolution_type, index);
//                for(auto distribution : *voxel){
//                    for(int32_t j = 0;
//                        j < distribution.height / 3;
//                        j++){
//                        int32_t indexZ = j + distribution.floor / 3;
//                        float point[4];
//                        point[0] = gridCenter.x();
//                        point[1] = gridCenter.y();
//                        point[2] = ((float)indexZ + 0.5) * 3 / 100;
//                        point[3] = distribution.intensity;
//                        ofs << point[0] << "," << point[1] << ","
//                                        << point[2] << "," << point[3] << std::endl;
//                    }
//                }
//            }
//        }
//        ofs.close();
//        exit(0);

        currMeshStream_->Forward(
                    init_pose.B, init_pose.L, init_pose.H,
                    pose_in_map.translation().x(),
                    pose_in_map.translation().y(),
                    pose_in_map.translation().z());
        ThreadPtr trd = ThreadPtr(new boost::thread(
                                      bind(&StreamManager::checkMeshBound,
                                           this,
                                           pose_in_map.translation())));
        return true;
    }

    inline void ConfirmPosition(const Eigen::Vector3d &pose_in_map, GnssPoint &final_pose)
    {
        if(!currMeshStream_){
            return;
        }
        currMeshStream_->Reverse(
                    pose_in_map.x(),
                    pose_in_map.y(),
                    pose_in_map.z(),
                    final_pose.B, final_pose.L, final_pose.H);
    }

    float getHighResolution() const;

    float getLowResolution() const;

    float getBlockResolution() const;

    inline juefx::GridIndex GetCellIndex(
            uint8_t type, float x, float y,
            const boost::shared_ptr<StreamInfo> &meshStream) const{
        juefx::GridIndex ret;
        Vector2i blockIndex;
        Vector2f meshOffset = meshStream->getOffset();
        float XInMesh = x - meshOffset.x();
        float YInMesh = y - meshOffset.y();
        blockIndex.x() = floor(XInMesh / blockResolution_);
        blockIndex.y() = floor(YInMesh / blockResolution_);
        Vector2f blockOrigin =
                Vector2f((float)blockIndex.x(), (float)blockIndex.y()) * blockResolution_;

        float gridResolution = highResolution_ * (1 << type);

        Vector2i gridIndex;
        gridIndex.x() = floor((x - blockOrigin.x()) / gridResolution);
        gridIndex.y() = floor((y - blockOrigin.y()) / gridResolution);
        ret.blockIndex.x = (uint8_t)blockIndex.x();
        ret.blockIndex.y = (uint8_t)blockIndex.y();
        ret.gridIndex.x = (int16_t)gridIndex.x();
        ret.gridIndex.y = (int16_t)gridIndex.y();
        return ret;
    }

    inline juefx::GridIndex GetCellIndex(
            uint8_t type, float x, float y) const{
        juefx::GridIndex ret;
        Vector2i blockIndex;
        float XInMesh = x - currMeshOffset_.x();
        float YInMesh = y - currMeshOffset_.y();
        blockIndex.x() = floor(XInMesh / blockResolution_);
        blockIndex.y() = floor(YInMesh / blockResolution_);

        int8_t blockPosition = currMeshStream_->IsGridInArea(blockIndex);
        if(blockPosition != 100){   //in neighbor
            ulong newMesh = GetNeighborGeoHash(
                        currMeshStream_->getMeshId(), (NeighborType)blockPosition);
            Vector2i pos = checkNeighborMeshPos((NeighborType)blockPosition);
            boost::shared_ptr<StreamInfo> meshStream;
            if(!findMesh(newMesh, meshStream)){
                ret.meshIndex.x = (int8_t)pos.x();
                ret.meshIndex.y = (int8_t)pos.y();
                return ret;
            }
            ret = GetCellIndex(type, x, y, meshStream);
            ret.meshIndex.x = (int8_t)pos.x();
            ret.meshIndex.y = (int8_t)pos.y();
            return ret;
        }

        Vector2f blockOrigin =
                Vector2f((float)blockIndex.x(), (float)blockIndex.y()) * blockResolution_;

        float gridResolution = highResolution_ * (1 << type);
        Vector2i gridIndex;
        gridIndex.x() = floor((XInMesh - blockOrigin.x()) / gridResolution);
        gridIndex.y() = floor((YInMesh - blockOrigin.y()) / gridResolution);
        ret.blockIndex.x = (uint8_t)blockIndex.x();
        ret.blockIndex.y = (uint8_t)blockIndex.y();
        ret.gridIndex.x = (int16_t)gridIndex.x();
        ret.gridIndex.y = (int16_t)gridIndex.y();
        ret.meshIndex.x = 0;
        ret.meshIndex.y = 0;
        return ret;
    }

    inline const juefx::GridData *GetCellValue(
        uint8_t type, const juefx::GridIndex &index) const
    {
        juefx::GridData* ret = NULL;
        juefx::GridIndex calibedGridIndex;
        if(!calibGridIndex(type, index, calibedGridIndex)){
            return ret;
        }
        if(index.meshIndex.x == 0 && index.meshIndex.y == 0){
            return currMeshStream_->getGridData(
                        type, calibedGridIndex.blockIndex, calibedGridIndex.gridIndex);
        }else{
            boost::shared_ptr<StreamInfo> currMeshStream;
            if(!getMeshStream(calibedGridIndex.meshIndex, currMeshStream)){
                return ret;
            }
            return currMeshStream->getGridData(
                        type, calibedGridIndex.blockIndex, calibedGridIndex.gridIndex);
        }
    }

    inline Eigen::Vector2f GetCenterOfCell(
            uint8_t type, const juefx::GridIndex &index) const
    {
        Vector2f ret = {-9999, -9999};
        int32_t blockIndexX = index.blockIndex.x;
        int32_t blockIndexY = index.blockIndex.y;
        float blockCenterX = ((float)blockIndexX + 0.5) * blockResolution_;
        float blockCenterY = ((float)blockIndexY + 0.5) * blockResolution_;
        uint32_t gridIndexX = index.gridIndex.x;
        uint32_t gridIndexY = index.gridIndex.y;
        float gridResolution = highResolution_ * (1 << type);

        float centerX = ((float)gridIndexX + 0.5) * gridResolution - blockResolution_ / 2;
        float centerY = ((float)gridIndexY + 0.5) * gridResolution - blockResolution_ / 2;
        ret.x() = centerX + blockCenterX + currMeshOffset_.x();
        ret.y() = centerY + blockCenterY + currMeshOffset_.y();
        return ret;
    }

    inline Eigen::Vector2f GetCenterOfCell(
            uint8_t type, float x, float y) const
    {
        Vector2f ret = {-9999, -9999};
        float gridResolution = highResolution_ * (1 << type);


        Vector2f gridFloor;
        gridFloor.x() = floor((x - currMeshOffset_.x()) / gridResolution) * gridResolution;
        gridFloor.y() = floor((y - currMeshOffset_.y()) / gridResolution) * gridResolution;
        ret = gridFloor + Vector2f(
                    gridResolution / 2 + currMeshOffset_.x(),
                    gridResolution / 2 + currMeshOffset_.y());
        return ret;
    }

    inline bool IsMeshLoaded(ulong mesh)
    {
        map<ulong, boost::shared_ptr<StreamInfo>> meshStreamMap;
        {
            boost::shared_lock<boost::shared_mutex> readLock(meshStreaMtx_);
            meshStreamMap = meshStreamMap_;
        }

        auto iter = meshStreamMap.find(mesh);
        if(iter != meshStreamMap.end()){
            return true;
        }
        return false;
    }

    inline bool checkCurrMesh(ulong mesh)
    {
        if(currMeshStream_ && currMeshStream_->getMeshId() == mesh){
            return true;
        }
        return false;
    }
private:
    bool getCurrMeshGridResolution(
            uint8_t type, float& resolution) const;

    inline bool calibGridIndex(
            uint8_t type,
            const juefx::GridIndex &query,
            juefx::GridIndex &ret) const
    {
        size_t gridCntPerBlockSide = highGridCntPerBlockSide_ / (1 << type);
        if(query.gridIndex.x >= 0 && query.gridIndex.x < gridCntPerBlockSide
                && query.gridIndex.y >= 0 && query.gridIndex.y < gridCntPerBlockSide){
            ret = query;
            return true;
        }
        juefx::GridIndex::ShortId blockCarry;
        checkCarry(query.gridIndex, gridCntPerBlockSide, ret.gridIndex, blockCarry);
        juefx::GridIndex::ShortId blockIndex;
        blockIndex.x = (int16_t)(query.blockIndex.x) + blockCarry.x;
        blockIndex.y = (int16_t)(query.blockIndex.y) + blockCarry.y;
        juefx::GridIndex::ShortId finalBlockIndex, meshCarry;
        checkCarry(blockIndex, blockCntPerMeshSide_, finalBlockIndex, meshCarry);
        ret.blockIndex.x = (uint8_t)(finalBlockIndex.x);
        ret.blockIndex.y = (uint8_t)(finalBlockIndex.y);
        ret.meshIndex.x = query.meshIndex.x + meshCarry.x;
        ret.meshIndex.y = query.meshIndex.y + meshCarry.y;
        return true;
    }

    inline bool getMeshStream(const juefx::GridIndex::SByteId &index,
                       boost::shared_ptr<StreamInfo> &meshStream) const
    {
        NeighborType neighborType;
        if(!getNeighborTypeByIndex(index, neighborType)){
            return false;
        }
        ulong neighborMeshId = GetNeighborGeoHash(
                    currMeshStream_->getMeshId(), neighborType);
        {
            boost::shared_lock<boost::shared_mutex> readLock;
            auto iter = meshStreamMap_.find(neighborMeshId);
            if(iter == meshStreamMap_.end()){
                return false;
            }
            meshStream = iter->second;
        }
        return true;
    }

    void checkMeshBound(const Vector3d currPos);

    vector<ulong> getNeighborsByMask(ulong currMeshId, uint8_t mask);

    void checkNeighborToLoad(const Vector3d currPos);

    void checkNeighborToFree(const Vector3d currPos);

    bool getNeighborTypeByIndex(
            const juefx::GridIndex::SByteId &index,
            NeighborType &neighborType) const;
    inline void checkCarry(
            const juefx::GridIndex::ShortId &query,
            const size_t &cntPerSide,
            juefx::GridIndex::ShortId &ret,
            juefx::GridIndex::ShortId &carry) const
    {
        if(query.x >= 0 && query.x < cntPerSide
                && query.y >= 0 && query.y < cntPerSide){
            ret = query;
            return;
        }
        ret.x = (query.x % cntPerSide + cntPerSide) % cntPerSide;
        ret.y = (query.y % cntPerSide + cntPerSide) % cntPerSide;
        carry.x = floor((float)query.x / cntPerSide);
        carry.y = floor((float)query.y / cntPerSide);
    }

private:
    string streamDir;

    vector<ulong> loadingMashVec_;

    mutable boost::shared_mutex meshStreaMtx_;

    boost::shared_mutex loadingMashMtx_;

    boost::shared_mutex currMashMtx_;

    map<ulong, boost::shared_ptr<StreamInfo>> meshStreamMap_;

    boost::shared_ptr<StreamInfo> currMeshStream_;

    Vector2f currMeshOffset_ = Vector2f::Zero();

    bool fstMeshLoaded = false;

    float highResolution_ = 0;

    float lowResolution_ = 0;

    float blockResolution_ = 0;

    uint32_t highGridCntPerBlockSide_ = 0;

    uint32_t lowGridCntPerBlockSide_ = 0;

    uint32_t blockCntPerMeshSide_ = 0;

    bool loadingNeighbors = false;

    bool freeingNeighbors = false;

};

#endif // STREAMMANAGER_H
