#pragma once
#ifndef BASIC_FUNCTIONS_H
#define BASIC_FUNCTIONS_H

#include "basic_types.h"
#include <chrono>
#include "pcl_point_type.h"
using namespace chrono;

#define TIMER_START(start) \
    auto start = std::chrono::system_clock::now();

#define TIMER_END(start, seconds)                                                       \
    auto end = std::chrono::system_clock::now();                                        \
    auto duration = std::chrono::duration_cast<std::chrono::microseconds>(end - start); \
    double seconds = double(duration.count()) / std::chrono::microseconds::period::den;


//template <typename T>
//Eigen::Matrix<T, 3, 1> RotationQuaternionToEulerVector(
//    const Eigen::Quaternion<T>& quaternion) {
//  Eigen::Quaternion<T> n_q = quaternion.normalized();
//  // We choose the quaternion with positive 'w', i.e., the one with a smaller
//  // angle that represents this orientation.
//  if (n_q.w() < 0.) {
//    // Multiply by -1. http://eigen.tuxfamily.org/bz/show_bug.cgi?id=560
//    n_q.w() = -1. * n_q.w();
//    n_q.x() = -1. * n_q.x();
//    n_q.y() = -1. * n_q.y();
//    n_q.z() = -1. * n_q.z();
//  }
//  // We convert the normalized_quaternion(n_q) into a vector along the rotation
//  // axis with length of the rotation angle.
//  const T roll = atan2(2 * (n_q.w() * n_q.x() + n_q.y() * n_q.z()),
//                       1 - 2 * (n_q.x() * n_q.x() + n_q.y() * n_q.y()));
//  const T pitch = asin(2 * (n_q.w() * n_q.y() - n_q.z() * n_q.x()));
//  const T yaw = atan2(2 * (n_q.w() * n_q.z() + n_q.x() * n_q.y()),
//                      1 - 2 * (n_q.y() * n_q.y() + n_q.z() * n_q.z()));
//  return Eigen::Matrix<T, 3, 1>(roll, pitch, yaw);
//}


vector<pair<uint32_t, uint32_t>> getIndexPeriod(const uint32_t size,
                                                const uint8_t threadCnt = MAX_THREAD_CNT);

//template<typename ItemType>
//void periodProcessThreadFunc(const PointCloudExport::Ptr &Vector,
//                             pair<uint32_t, uint32_t> period,
//                             boost::function<void(const ItemType &)> processItemFunc)
//{
//    vector<ItemType> tempOutputVec;
//    uint32_t start_index = period.first;
//    uint32_t index_cnt = period.second;
//    uint32_t final_index = start_index + index_cnt;
//    for(uint32_t index = start_index; index < final_index; index++){
//        processItemFunc(Vector->at(index));
//    }
//}

//template<typename ItemType>
//void multiThreadProcess(const PointCloudExport::Ptr &Vector,
//                        boost::function<void(const ItemType &)> processItemFunc)
//{
//    auto start = system_clock::now();
//    vector<pair<uint32_t, uint32_t>> indexPeriod = getIndexPeriod(Vector->size(),
//                                                                  std::thread::hardware_concurrency());
//    vector<ThreadPtr> threadVec;
//    boost::mutex mutex;
//    for(size_t thread_index = 0; thread_index < indexPeriod.size(); thread_index++){
//        threadVec.emplace_back(
//                    ThreadPtr(new boost::thread(boost::bind(&periodProcessThreadFunc<ItemType>,
//                                                            Vector,
//                                                            indexPeriod[thread_index],
//                                                            processItemFunc))));
//    }
//    for(uint8_t thread_index = 0; thread_index < indexPeriod.size(); thread_index++){
//        threadVec[thread_index]->join();
//        threadVec[thread_index].reset();

//    }
//    auto end   = system_clock::now();
//    auto duration = duration_cast<milliseconds>(end - start);
//    LOG(INFO) << "process take " << double(duration.count()) / 1000 << " seconds,"
//              << " output size: " << Vector->size();
//}

template<typename ItemType>
void periodProcessThreadFunc(const vector<ItemType> *Vector,
                             pair<uint32_t, uint32_t> period,
                             boost::function<void(const ItemType &)> processItemFunc)
{
    uint32_t start_index = period.first;
    uint32_t index_cnt = period.second;
    uint32_t final_index = start_index + index_cnt;
    for(uint32_t index = start_index; index < final_index; index++){
        processItemFunc(Vector->at(index));
    }
}

template<typename ItemType>
void multiThreadProcess(const vector<ItemType> &Vector,
                        boost::function<void(const ItemType &)> processItemFunc)
{
    auto start = system_clock::now();
    vector<pair<uint32_t, uint32_t>> indexPeriod = getIndexPeriod(Vector.size(),
                                                                  std::thread::hardware_concurrency());
    vector<ThreadPtr> threadVec;
    boost::mutex mutex;
    for(size_t thread_index = 0; thread_index < indexPeriod.size(); thread_index++){
        threadVec.emplace_back(
                    ThreadPtr(new boost::thread(boost::bind(&periodProcessThreadFunc<ItemType>,
                                                            &Vector,
                                                            indexPeriod[thread_index],
                                                            processItemFunc))));
    }
    for(uint8_t thread_index = 0; thread_index < indexPeriod.size(); thread_index++){
        threadVec[thread_index]->join();
        threadVec[thread_index].reset();

    }
    auto end   = system_clock::now();
    auto duration = duration_cast<milliseconds>(end - start);
    LOG(INFO) << "process take " << double(duration.count()) / 1000 << " seconds,"
              << " output size: " << Vector.size();
}


template<typename ItemType>
void syncProcessThreadFunc(const vector<ItemType> *inputVector,
                           int32_t *currentIndex,
//                           boost::mutex *mutex,
                           boost::function<void(const ItemType &)> processItemFunc)
{
    uint32_t size = inputVector->size();
    int32_t index;
    while(true){
        {
//            boost::unique_lock<boost::mutex> guard(*mutex);
            index = ++(*currentIndex);
        }
        if((uint32_t)index >= size){
//            LOG(INFO) << "index: " << index;
            break;
        }
        processItemFunc(inputVector->at((uint32_t)index));
    }
}

template<typename ItemType>
void syncProcess(const vector<ItemType> &inputVector,
                 boost::function<void(const ItemType &)> processItemFunc)
{
    size_t threadCnt = std::thread::hardware_concurrency();
    threadCnt = 1;
    LOG(INFO) << "threadCnt: " << threadCnt;
    auto start = system_clock::now();
    int32_t currentIndex = -1;
    boost::mutex mutex;
    vector<ThreadPtr> threadVec;
    for(size_t thread_index = 0; thread_index < threadCnt; thread_index++){
        threadVec.emplace_back(
                    ThreadPtr(new boost::thread(boost::bind(&syncProcessThreadFunc<ItemType>,
                                                            &inputVector,
                                                            &currentIndex,
//                                                            &mutex,
                                                            processItemFunc))));
    }
    for(uint8_t thread_index = 0; thread_index < threadCnt; thread_index++){
        threadVec[thread_index]->join();
        threadVec[thread_index].reset();
    }
    auto end = system_clock::now();
    auto duration = duration_cast<milliseconds>(end - start);
    LOG(INFO) << "process take " << double(duration.count()) / 1000 << " seconds";
}

//-----------------------------------------------------------------------------
const int MAX_MESH_LEVEL = 15;
const int NDS_MESH_LEVEL = 15;       // Nearly 2 km
const uint INVALID_MESHID = 0;


inline Vector2d GetMeshCenter(ulong meshId, int level = NDS_MESH_LEVEL)
{
    ulong x = 0, y = 0;
    for (int i = 0; i < level; i++)
    {
        x += ((meshId >> (i * 2)) & 1) << i;
        y += ((meshId >> (i * 2 + 1)) & 1) << i;
    }

    Vector2d center_point;

    center_point.x() = ( (x << (31 - level)) * 90.0 / (1 << 30) + ((x + 1) << (31 - level)) * 90.0 / (1 << 30) ) / 2.0;
    center_point.y() = ( (y << (31 - level)) * 90.0 / (1 << 30) + ((y + 1) << (31 - level)) * 90.0 / (1 << 30) ) / 2.0;

    return center_point;
}

inline ulong Part1By1(ulong x)
{
    x &= 0x0000ffff; // x = ---- ---- ---- ---- fedc ba98 7654 3210

    x = (x ^ (x << 8)) & 0x00ff00ff; // x = ---- ---- fedc ba98 ---- ---- 7654 3210

    x = (x ^ (x << 4)) & 0x0f0f0f0f; // x = ---- fedc ---- ba98 ---- 7654 ---- 3210

    x = (x ^ (x << 2)) & 0x33333333; // x = --fe --dc --ba --98 --76 --54 --32 --10

    x = (x ^ (x << 1)) & 0x55555555; // x = -f-e -d-c -b-a -9-8 -7-6 -5-4 -3-2 -1-0

    return x;
}

inline ulong EncodeMorton(ulong x, ulong y)
{
    return (Part1By1(y) << 1) + Part1By1(x);
}

inline double GetMeshLatitudeSize(int level = NDS_MESH_LEVEL)
{
    return 180.0 / (1 << level);
}

inline ulong GetMeshFromPosition(
        double L, double B,
        int level = NDS_MESH_LEVEL,
        int maxLevel = MAX_MESH_LEVEL)
{
    if (level > maxLevel || level < 0/* || OutofChina(L, B)*/)
        return INVALID_MESHID;

    double size = GetMeshLatitudeSize(level);
    ulong col = (ulong)(L / size);
    ulong row = (ulong)(B / size);
    return EncodeMorton(col, row);
}

inline Vector4d GetMeshBound(ulong meshId, int level = NDS_MESH_LEVEL)
{
    ulong x = 0, y = 0;
    for (int i = 0; i < level; i++)
    {
        x += ((meshId >> (i * 2)) & 1) << i;
        y += ((meshId >> (i * 2 + 1)) & 1) << i;
    }

    return Vector4d(
                (x << (31 - level)) * 90.0 / (1 << 30),
                ((y + 1) << (31 - level)) * 90.0 / (1 << 30),
                ((x + 1) << (31 - level)) * 90.0 / (1 << 30),
                (y << (31 - level)) * 90.0 / (1 << 30));
}

enum NeighborType
{
    Left = 0,
    Right,
    Up,
    Down,
    LeftDown,
    LeftUp,
    RightDown,
    RightUp
};

const vector<NeighborType> neighborTypeVec = {
    Left, Right, Up, Down, LeftDown, LeftUp, RightDown, RightUp};

const vector<Vector2i> neighborIndexDiff = {
    {-1, 0}, {1, 0}, {0, 1}, {0, -1}, {-1, -1}, {-1, 1}, {1, -1}, {1, 1}
};

inline Vector2i checkNeighborMeshPos(
        const NeighborType &neighborType){
    Vector2i ret = {0, 0};
    switch (neighborType)
    {
    case NeighborType::Left:
        ret = {-1, 0};
        break;
    case NeighborType::Right:
        ret = {1, 0};
        break;
    case NeighborType::Up:
        ret = {0, 1};
        break;
    case NeighborType::Down:
        ret = {0, -1};
        break;
    case NeighborType::LeftDown:
        ret = {-1, -1};
        break;
    case NeighborType::LeftUp:
        ret = {-1, 1};
        break;
    case NeighborType::RightDown:
        ret = {1, -1};
        break;
    case NeighborType::RightUp:
        ret = {1, 1};
        break;
    default:
        break;
    }
    return ret;
}

inline ulong Compact1By1(ulong x)
{
    x &= 0x55555555; // x = -f-e -d-c -b-a -9-8 -7-6 -5-4 -3-2 -1-0

    x = (x ^ (x >> 1)) & 0x33333333; // x = --fe --dc --ba --98 --76 --54 --32 --10

    x = (x ^ (x >> 2)) & 0x0f0f0f0f; // x = ---- fedc ---- ba98 ---- 7654 ---- 3210

    x = (x ^ (x >> 4)) & 0x00ff00ff; // x = ---- ---- fedc ba98 ---- ---- 7654 3210

    x = (x ^ (x >> 8)) & 0x0000ffff; // x = ---- ---- ---- ---- fedc ba98 7654 3210

    return x;
}

inline ulong DecodeMorton2X(ulong code)
{
    return Compact1By1(code >> 0);
}

inline ulong DecodeMorton2Y(ulong code)
{
    return Compact1By1(code >> 1);
}

inline ulong GetNeighborGeoHash(ulong hash, NeighborType neighborType)
{
    ulong x = DecodeMorton2X(hash);
    ulong y = DecodeMorton2Y(hash);
    switch (neighborType)
    {
    case NeighborType::Left:
        x -= 1;
        break;
    case NeighborType::Right:
        x += 1;
        break;
    case NeighborType::Up:
        y += 1;
        break;
    case NeighborType::Down:
        y -= 1;
        break;
    case NeighborType::LeftDown:
        x -= 1;
        y -= 1;
        break;
    case NeighborType::LeftUp:
        x -= 1;
        y += 1;
        break;
    case NeighborType::RightDown:
        x += 1;
        y -= 1;
        break;
    case NeighborType::RightUp:
        x += 1;
        y += 1;
        break;
    default:
        break;
    }
    return EncodeMorton(x, y);
}

#include <GeographicLib/LocalCartesian.hpp>
#include <ceres/ceres.h>
#include <ceres/autodiff_local_parameterization.h>
#include <ceres/ceres.h>
#include <ceres/local_parameterization.h>
#include <ceres/rotation.h>
#include <ceres/types.h>
#include <ceres/loss_function.h>
class DistanceError
{
public:
    static ceres::CostFunction *Create(Eigen::Vector3d before, Eigen::Vector3d after)
    {
        return new ceres::AutoDiffCostFunction<DistanceError, 3 /* residuals */,
                                               4 /* rotation variables */,
                                               3 /* translation variables */>(
            new DistanceError(before, after));
    }

    template <typename T>
    bool operator()(const T *const rotation, const T *const translation, T *const e) const
    {
        Eigen::Matrix<T, 3, 1> before;
        before << T(before_[0]), T(before_[1]), T(before_[2]);
        Eigen::Matrix<T, 3, 1> after;
        after << T(after_[0]), T(after_[1]), T(after_[2]);
        Eigen::Matrix<T, 3, 1> trans;
        trans << translation[0], translation[1], translation[2];
        Eigen::Quaternion<T> q = Eigen::Map<const Eigen::Quaternion<T>>(rotation);
        Eigen::Matrix<T, 3, 1> dis = after - (q.toRotationMatrix() * before + trans);
        e[0] = dis(0,0);
        e[1] = dis(1,0);
        e[2] = dis(2,0);
        return true;
    }

private:
    explicit DistanceError(Eigen::Vector3d before, Eigen::Vector3d after)
        : before_(before), after_(after) {}

    Eigen::Vector3d before_;
    Eigen::Vector3d after_;
};

inline Eigen::Vector3d matrix_2_euler_rpy(const Eigen::Matrix3d mat)
{
    double temp = mat(2, 1) * mat(2, 1) + mat(2, 2) * mat(2, 2);
    double roll = atan2(mat(2, 1), mat(2, 2));
    double pitch = atan2(-mat(2, 0), sqrt(temp));
    double yaw = atan2(mat(1, 0), mat(0, 0));
    return Eigen::Vector3d(roll, pitch, yaw);
}

Eigen::Matrix3d get_rotation(double old_B, double old_L, double old_H,
                             double new_B, double new_L, double new_H,
                             double old_start_x = 0, double old_start_y = 0, double old_start_z = 0,
                             double new_start_x = 0, double new_start_y = 0, double new_start_z = 0,
                             bool print = false);


#endif // BASIC_FUNCTIONS_H
